#!/bin/sh
###IBM Confidential
###
###OCO Source Materials
###
###IBM Cognos Products: rsvp
###
###(C) Copyright IBM Corp. 2010
###
###US Government Users Restricted Rights - Use, duplication or disclosure 
###restricted by GSA ADP Schedule Contract with IBM Corp.
### COPYRIGHT_DATA {'RELEASE': ['2010', 'colorado_wave1', 'colorado_wave2'], 'VISIBLE': 'YES', 'COMPONENT': 'rsvp'}
###All lines above the COPYRIGHT_DATA  line will be replaced when copyright notices are generated.

###############################################################################
# This script executes the final command to send a PDF document to the print  #
# queue on all unix and linux platforms.                                      #
#                                                                             #
# Usage: rsprintpdf.sh [-p <printer>] [-o <page orientation>] [-m <pagesize>] #
#            [-h <height> -w <width>] [-L <logfile>] file                     #
#                                                                             #
#     -p <printer> Specify the printer queue where this request will be sent. #
#                  If no printer is specified, the default queue is used.     #
#     -o [l|p]     Specify the page orientation of the file. Possible options #
#                  are l for landscape or p for portrait. If no orientation   #
#                  is specified, portrait is assumed.                         #
#     -m <media>   Specify the media size of the output. eg: Letter, A4       #
#                  If no media and no height and width are specified, then    #
#                  the default paper tray is used.                            #
#     -h <height>  For custom page sizes, the height of the page in points.   #
#                  Only valid if specified with a -w option, and without the  #
#                  -m option.                                                 #
#     -w <width>   For custom page sizes, the width of the page in points.    #
#                  Only valid if specified with a -h option, and without the  #
#                  -m option.                                                 #
#     -L <logfile> The path to a user-specified log file to redirect error    #
#                  messages. Defaults to rsprintpdf.errors.log.               #
#                                                                             #
###############################################################################


################################
# Parse command line arguments 

# default log if not specified
logfile="rsprintpdf.errors.log"

while getopts p:o:m:h:w:L: o
do case "$o" in
    p)  printer="$OPTARG";;
    o)  orientation="$OPTARG";;
    m)  media=`echo "$OPTARG" | tr "[:upper:]" "[:lower:]"`;;
    h)  height="$OPTARG";;
    w)  width="$OPTARG";;
    L)  logfile="$OPTARG";;
    \?)  echo "Usage: $0 [-p <printer>] [-o <page orientation>] [-m <pagesize>] [-h <height> -w <width>] [-L <logfile>] file" && exit 1;;
esac
done

# Reset $@ to get filename
shift `expr $OPTIND - 1`

file=$1

#####################
# Validate arguments

# Need a document to print
if [ -z "$file" ]; then
  echo "$0: No PDF document specified. Exiting." >>$logfile
  exit 1;
fi

# media takes precedence over width, height
if [ -n "$media" ]; then
  height=""; 
  width="";
fi

# only one of height, width specified
if [ -n "$width" -a  -z "$height" ]; then
    echo "$0: No height specified. Use -h to specify the page height. Exiting." >>$logfile
    exit 1;
fi 

if [ -z  "$width" -a -n "$height" ]; then
  echo "$0: No width specified. Use -w to specify the page width. Exiting." >>$logfile
  exit 1; 
fi

###########################
# Setup command parameters

acroreadParams="-toPostScript"
if [ "$orientation" = "l" ]; then
  acroreadParams="$acroreadParams -landscape"
fi

if [ -n "$media" ]; then
  acroreadParams="$acroreadParams -size $media"
elif [ -n "$width" -a -n "$height" ]; then
  acroreadParams="$acroreadParams -size ${width}x${height}"
fi

###########################
# Queue the file for print

ostype="`uname -s`"

case "$ostype" in
    SunOS)  
         if [ "$orientation" = "l" -a -z "$media" -a -z "$height" -a -z "$width" ]; then
           # Without this patch, lp prints landscape reports rotated 180 
           # degrees instead of 90.
           acroreadParams="$acroreadParams -size letter"
         fi

         lpParams=""
         if [ -n "$printer" ]; then
           lpParams="$lpParams -d'$printer'"
         fi

         # On solaris, lp has no option for page orientation. Instead, patch the 
         # PostScript file generated by acroread to include the following
         # commands that specify print media size:
         # mark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark
         #{<</PageSize [<width> <height>] /Policies <</PageSize 1>> >> setpagedevice }stopped pop
         awkParams="'{ if (\$0 ~ /HiResBoundingBox/) { width=\$4; height=\$5 }; if (\$0 ~ /BeginPageSetup/) { print \$0 \"\\nmark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark\" \"\\n{<</PageSize [\" width  \" \" height \"] /Policies <</PageSize 1>> >> setpagedevice }stopped pop\" } else { print \$0 } }'"
         command="cat \"$file\" | acroread $acroreadParams | awk $awkParams | lp $lpParams"
         ;;

      AIX)  
         if [ "$orientation" = "l" -a -z "$media" -a -z "$height" -a -z "$width" ]; then
           # Known bug on HP/AIX, acroread will generate incorrect PostScript 
           # if -landscape is used without -size. If size is not passed in, 
           # default to letter. See: http://forums.adobe.com/thread/395523
           acroreadParams="$acroreadParams -size letter"
         fi

         lpParams=""
         if [ -n "$printer" ]; then
           lpParams="$lpParams -d'$printer'"
         fi

         # On AIX, lp has no option for media size. Instead, patch the 
         # PostScript file generated by acroread to include the following
         # commands that specify print media size:
         # mark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark
         #{<</PageSize [<width> <height>] /Policies <</PageSize 1>> >> setpagedevice }stopped pop
         awkParams="'{ if (\$0 ~ /HiResBoundingBox/) { width=\$4; height=\$5 }; if (\$0 ~ /BeginPageSetup/) { print \$0 \"\\nmark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark\" \"\\n{<</PageSize [\" width  \" \" height \"] /Policies <</PageSize 1>> >> setpagedevice }stopped pop\" } else { print \$0 } }'"
         command="cat \"$file\" | acroread $acroreadParams | awk $awkParams | lp $lpParams"
         ;;

    HP-UX)  
         if [ "$orientation" = "l" -a -z "$media" -a -z "$height" -a -z "$width" ]; then
           # Known bug on HP/AIX, acroread will generate incorrect PostScript 
           # if -landscape is used without -size. If size is not passed in, 
           # default to letter. See: http://forums.adobe.com/thread/395523
           acroreadParams="$acroreadParams -size letter"
         fi

         lpParams=""
         if [ -n "$printer" ]; then
           lpParams="$lpParams -d'$printer'"
         fi

         # On HP, lp has no option for media size. Instead, patch the 
         # PostScript file generated by acroread to include the following
         # commands that specify print media size:
         # mark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark
         #{<</PageSize [<width> <height>] /Policies <</PageSize 1>> >> setpagedevice }stopped pop
         awkParams="'{ if (\$0 ~ /HiResBoundingBox/) { width=\$4; height=\$5 }; if (\$0 ~ /BeginPageSetup/) { print \$0 \"\\nmark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark\" \"\\n{<</PageSize [\" width  \" \" height \"] /Policies <</PageSize 1>> >> setpagedevice }stopped pop\" } else { print \$0 } }'"

         command="cat \"$file\" | acroread $acroreadParams | awk $awkParams | lp $lpParams"
         ;;

    Linux)  
         if [ "$orientation" = "l" -a -z "$media" -a -z "$height" -a -z "$width" ]; then
           # Without this patch, lp prints landscape reports rotated 0 
           # degrees instead of 90.
           acroreadParams="$acroreadParams -size letter"
         fi

         lpParams=""
         if [ -n "$printer" ]; then
           lpParams="$lpParams -d'$printer'"
         fi

         # Some printer configurations do not seem to support the CUPS -o media="size" option.
         # Instead, patch the PostScript file generated by acroread to include the following
         # commands that specify print media size:
         # mark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark
         #{<</PageSize [<width> <height>] /Policies <</PageSize 1>> >> setpagedevice }stopped pop
         awkParams="'{ if (\$0 ~ /HiResBoundingBox/) { width=\$4; height=\$5 }; if (\$0 ~ /BeginPageSetup/) { print \$0 \"\\nmark {<</DeferredMediaSelection false >> setpagedevice }stopped cleartomark\" \"\\n{<</PageSize [\" width  \" \" height \"] /Policies <</PageSize 1>> >> setpagedevice }stopped pop\" } else { print \$0 } }'"

         command="cat \"$file\" | acroread $acroreadParams | awk $awkParams | lp $lpParams"
         ;;
       \?)  echo "$0: Printing not yet supported on this platform: $ostype" >>$logfile;
            exit 1;
         ;;
esac 

         
########################
# Execute print command
echo "$0: Executing print command '$command'" >>$logfile 

eval $command 2>>$logfile 

errorStatus=$?
if [ $errorStatus = 0 ]; then
  echo "$0: Print command succeeded." >>$logfile
else
  echo "$0: Print command failed with error status $errorStatus." >>$logfile;
  exit $errorStatus
fi


# Done.

