/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
/**
	@fileoverview
	This file implements all that is necessary to send requests to the server side of Cognos Viewer (using the URL Api),
	process the responses and to perform actions on a report like next page or email.
	It manages the report state (conversation, tracking, prompting), the report output and the features on the client for the viewer.
*/

/**
	Initialize the CCognosViewer object.
	@requires CCognosViewerRequest
	@constructor
	@param {string} sId Identifier used for this object. This identifier will be used to mark elements used by this class.
	@param {string} sGateway Path to "cognos.cgi". For example: "/cognos8/cgi-bin/cognos.cgi".
*/

var CV_BACKGROUND_LAYER_ID = "CV_BACK";

if (typeof window.gaRV_INSTANCES == "undefined")
{
	window.gaRV_INSTANCES = [];
}


function CognosViewerSession(oCV)
{
	this.m_sConversation = oCV.getConversation();
	this.m_sParameters = oCV.getExecutionParameters();
	this.m_envParams = oCV.envParams;
	this.m_bRefreshPage = false;
}

function CCognosViewer(sId, sGateway)
{
	// Declare a global instance of CCognosViewer if none exists [for backward compatibility].
	// This global instance will be refered too by global prompt functions like setPromptControl() and promptButtonNext().
	if (typeof window.gCognosViewer == "undefined")
	{
		window.gCognosViewer = this;
	}

	this.m_sActionState = "";

	this.m_bKeepSessionAlive = false;

	/**
	 * @type array
	 * @private
	 */
	this.m_undoStack = [];

	/**
		@type array
		@private
	*/
	this.m_aSecRequests = [];

	/**
		@type string
		@private
	*/
	this.m_sSessionPrompts = "";

	/**
		@type bool
		@private
	*/
	this.m_bDebug = false;

	/**
		@type string
		@private
	*/
	this.m_sCAFContext = "";
	/**
		@type string
		@private
	*/
	this.m_sContextInfoXML = "";
	/**
		@type string
		@private
	*/
	this.m_sConversation = "";
	/**
	 * @type string
	 * @private
	 */
	this.m_sStatus = "";
	/**
		@type string
		@private
	*/
	this.m_sGateway = sGateway;
	/**
		@type string
		@private
	*/
	this.m_sId = sId;
	/**
		@type string
		@private
	*/
	this.m_sMetadataInfoXML = "";
	/**
		@type string
		@private
	*/
	this.m_sParameters = "";
	/**
		@type string
		@private
	*/
	this.m_sReportState = "";
	/**
		@type string
		@private
	*/
	this.m_sTracking = "";
	/**
		@type string
		@private
	*/
	this.m_sSoapFault = "";
	/**
		@type string
		@private
	*/
	this.m_sWaitHTML = "";
	/**
		@type CDrillManager
		@private
	*/
	this.m_oDrillMgr = null;
	/**
		@type CDrillManager
		@private
	*/
	this.goDrillManager = null;

	/**
	 *  @type CWaitPage
	 *  @private
	 */
	this.m_waitPage = new CWaitPage(sId, sGateway);

	/**
		@type CSubscriptionManager
		@private
	*/
	this.m_oSubscriptionManager = null;

	/**
		@type CViewerManager
		@private
	*/
	this.m_oCVMgr = null;

	/**
		@type boolean
		@private
	*/
	this.m_bUseSafeMode = true;

	if (typeof CViewerManager == "function")
	{
		this.m_oCVMgr = new CViewerManager(this);
	}

	if (typeof CDrillManager == "function")
	{
		this.m_oDrillMgr = new CDrillManager(this);
		this.goDrillManager = this.m_oDrillMgr;
	}

	if (typeof CSubscriptionManager == "function")
	{
		this.m_oSubscriptionManager = new CSubscriptionManager(this);
	}

	if (window.gaRV_INSTANCES)
	{
		// only add the cognos viewer object to the array if it isn't already in it
		var bFound = false;
		for (var iIndex=0; iIndex < window.gaRV_INSTANCES.length; iIndex++)
		{
			if (window.gaRV_INSTANCES[iIndex].m_sId == sId)
			{
				window.gaRV_INSTANCES[iIndex] = this;
				bFound = true;
				break;
			}
		}

		if (!bFound)
		{
			window.gaRV_INSTANCES = window.gaRV_INSTANCES.concat(this);
		}
	}

	this.m_oServerRequest = new ServerRequest(this.m_sId);

	this.m_bReportHasPrompts = false;

	this.m_viewerWidget = null;

	this.m_flashChartsObjectIds = [];

	this.m_raiseSharePromptEvent = true;

	this.m_actionFactory = null;

	this.m_calculationCache = {};

	this.m_drillTargets = [];

	this.m_reportRenderingDone = false;

}

CCognosViewer.prototype.createDispatcherEntryServerRequest = function(callbacks, isAsynch)
{
	//Creating the server part of an ActionDispatcherEntry here makes it easier to mock/test.
	var asynchRequest=null;
	if(isAsynch == true) {
		asynchRequest = new AsynchDATARequest(this.getGateway(), this.getWebContentRoot(), callbacks);
		asynchRequest.setFaultDialog(new BuxFaultDialog());
		asynchRequest.setPromptDialog(new BuxPromptDialog(this.getId()));
		asynchRequest.setLogonDialog(new BuxLogOnDialog());
	} else {
		asynchRequest = new XmlHttpObject();
	}
	return asynchRequest;
};

CCognosViewer.prototype.createDispatcherEntryJSONServerRequest = function(callbacks, isAsynch)
{
	//Creating the server part of an ActionDispatcherEntry here makes it easier to mock/test.
	var asynchRequest=null;
	if(isAsynch == true) {
		asynchRequest = new AsynchJSONRequest(this.getGateway(), this.getWebContentRoot(), callbacks);
		asynchRequest.setFaultDialog(new BuxFaultDialog());
		asynchRequest.setPromptDialog(new BuxPromptDialog(this.getId()));
		asynchRequest.setLogonDialog(new BuxLogOnDialog());
	} else {
		asynchRequest = new XmlHttpObject();
	}
	return asynchRequest;
};

CCognosViewer.prototype.getActionFactory = function(){
	if (!this.m_actionFactory) {
		this.m_actionFactory = new ActionFactory(this);
	}

	return this.m_actionFactory;
};

CCognosViewer.prototype.getCalculationCache = function(){
	return this.m_calculationCache;
};

CCognosViewer.prototype.checkForHighContrast = function()
{
	var tempDiv = document.createElement("div");
	tempDiv.id = this.m_sId + "hc";
	tempDiv.style.border = "1px solid";
	tempDiv.style.borderColor = "red green";
	tempDiv.style.height = "10px";
	tempDiv.style.top = "-999px";
	tempDiv.style.position = "absolute";
	document.body.appendChild(tempDiv);

	var computedStyle = null;
	if (document.all)
	{
		computedStyle = tempDiv.currentStyle;
	}
	else
	{
		computedStyle = tempDiv.ownerDocument.defaultView.getComputedStyle(tempDiv, null);
	}

	this.m_bHighContrast = computedStyle.borderTopColor == computedStyle.borderRightColor;
	this.updateBorderCollapse();
	document.body.removeChild(tempDiv);
};

CCognosViewer.prototype.isHighContrast = function()
{
	if (typeof this.m_bHighContrast === "undefined") {
		this.checkForHighContrast();
	}
	return this.m_bHighContrast;
};

CCognosViewer.prototype.isLimitedInteractiveMode = function() {
	return this.envParams && this.envParams.limitedInteractiveMode && this.envParams.limitedInteractiveMode === "true";
};

CCognosViewer.prototype.updateBorderCollapse = function()
{

	if (this.isHighContrast() == true)
	{
		var reportDiv = null;
		if (this.envParams["ui.action"] == "view" && !this.isBux)
		{
			var iframe = document.getElementById("CVIFrame" + this.getId());
			reportDiv = iframe.contentWindow.document;
		}
		else
		{
			reportDiv = document.getElementById("CVReport" + this.getId());
		}

		var tables = reportDiv.getElementsByTagName("table");
		for (var i = 0; i < tables.length; i++)
		{
			if (tables[i].style.borderCollapse == "collapse")
			{
				tables[i].style.borderCollapse = "separate";
			}
		}
	}
};

CCognosViewer.prototype.isAccessibleMode = function()
{
	if (this.m_bAccessibleMode == true)
	{
		return true;
	}

	return false;
};

/**
 * @return true if the current report is a single page report.
 */
CCognosViewer.prototype.isSinglePageReport = function()
{
	for (var request in this.m_aSecRequests) {
		if (this.m_aSecRequests[request] == 'nextPage' || this.m_aSecRequests[request] == 'previousPage')
		{
			return false;
		}
	}
	return true;
};

/**
 * KeyDown events handler attached to the document, which disables hotkey page navigation within the viewer
 * @private
 */
CCognosViewer.prototype.captureHotkeyPageNavigation = function(evt)
{

	evt = (evt) ? evt : ((event) ? event : null);
	if(evt)
	{
		var node = getNodeFromEvent(evt);
		var nodeName = (node && node.nodeName) ? node.nodeName.toLowerCase() : null;
		if( (evt.keyCode == 8 && nodeName != "input" && nodeName != "textarea") || (evt.altKey == true && (evt.keyCode == 37 || evt.keyCode == 39)) )
		{
			evt.returnValue = false;
			evt.cancelBubble = true;
			if(typeof evt.stopPropagation != "undefined")
			{
				evt.stopPropagation();
			}
			if(typeof evt.preventDefault != "undefined")
			{
				evt.preventDefault();
			}

			return false;
		}
	}

	return true;
};

/**
 * Returns the wait page implementation
 * @return (CWaitPage)
 */
CCognosViewer.prototype.getWaitPageImpl = function() {
	return this.m_waitPage;
};

/**
 * Sets the wait page class to use
 */
CCognosViewer.prototype.setWaitPageImpl = function(waitPage) {
	this.m_waitPage = waitPage;
};


/**
 * Call this method to disable hot key page navigation within the viewer (back space, alt+left arrow, alt+right arrow)
 */
CCognosViewer.prototype.disableBrowserHotkeyPageNavigation = function() {
	if (document.attachEvent)
	{
		document.attachEvent("onkeydown", this.captureHotkeyPageNavigation);
	}
	else if (document.addEventListener)
	{
		document.addEventListener("keydown", this.captureHotkeyPageNavigation, false);
	}
};

/**
 * Hide/shows the viewer wait page
 * @private
 */
CCognosViewer.prototype.showWaitPage = function(bShow, cancelCallback) {
	if(bShow) {
		this.m_waitPage.show(false, cancelCallback);
	}
	else {
		this.m_waitPage.hide();
	}
};

/**
 * Sets if the current HTML being displayed contains prompt controls
 * @param (boolean) hasPrompts
*/
CCognosViewer.prototype.setHasPrompts = function(hasPrompts)
{
	this.m_bReportHasPrompts =  hasPrompts;
};

/**
 * Checks to see if the HTML being displayed contains prompt controls
 * @return (boolean)
 */
CCognosViewer.prototype.getHasPrompts = function() {
	return this.m_bReportHasPrompts;
};


/**
 * Sets the request mode (page/data)
 * @param (boolean) bPageRequest
*/
CCognosViewer.prototype.setUsePageRequest = function(bPageRequest)
{
	this.m_oServerRequest.setUsePageRequest(bPageRequest);
};

/**
 * Gets the request mode (page/data)
 * @return (boolean)
 */
CCognosViewer.prototype.getUsePageRequest = function() {
	return this.m_oServerRequest.getUsePageRequest();
};

/**
 * If you don't want the viewer to release the current conversation on a page reload, use this method to keep the session alive
 * on page reloads
 * @param (boolean)
 */
CCognosViewer.prototype.setKeepSessionAlive = function(bValue) {
	this.m_bKeepSessionAlive = bValue;
};

/**
 * Return the "Keep Session Alive" boolean value
 * @return (boolean)
 */
CCognosViewer.prototype.getKeepSessionAlive = function() {
	return this.m_bKeepSessionAlive;
};

/**
 * Returns the web content root
 * @return (string)
 */
CCognosViewer.prototype.getWebContentRoot = function()
{
	if (typeof this.sWebContentRoot != "undefined")
	{
		return this.sWebContentRoot;
	}
	else
	{
		return "..";
	}
};

/**
 * Returns the skin directory
 * @return (string)
 */
CCognosViewer.prototype.getSkin = function()
{
	if (typeof this.sSkin != "undefined")
	{
		return this.sSkin;
	}
	else
	{
		return this.getWebContentRoot() + "/skins/corporate";
	}
};

CCognosViewer.prototype.getSelectionController = function()
{
	var selectionController;
	try
	{
		selectionController = getCognosViewerSCObjectRef(this.m_sId);
	}
	catch(e)
	{
		selectionController = null;
	}

	return selectionController;
};

/**
	Set up a function to call for events in CCognosViewer.
	@param {string} sEventName Event names are:
		<ul>
		<li><code>done</code> (when a report page is returned)</li>
		<li><code>error</code> (when an error code is returned)</li>
		<li><code>prompt</code> (when an prompt page is returned)</li>
		<li><code>wait</code> (when we get a ?working? and ?stillWorking? state)</li>
		</ul>
	@param {function} oFct Function to call when this even occurs.
	@param {bool} bCaptureEvent If set to true, the normal behavior will not be used.
*/
CCognosViewer.prototype.addCallback = function(sEventName, oFct,  bCaptureEvent)
{
	if (!this.m_aCallback)
	{
		this.m_aCallback = [];
	}

	this.m_aCallback = this.m_aCallback.concat({
		m_sEvent: sEventName,
		m_oCallback: oFct,
		m_bCaptureEvent: (bCaptureEvent===true)
	});
};

/**
	Determines if the element supports drill down or not.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Default is <code>false</code>.
	@type bool
*/
CCognosViewer.prototype.canDrillDown = function(sId)
{
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			return (selectionController.canDrillDown(sCtx));
		}
	}
	return false;
};

/**
	Determines if the element supports drill up or not.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Default is <code>false</code>.
	@type bool
*/
CCognosViewer.prototype.canDrillUp = function(sId)
{
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			return (selectionController.canDrillUp(sCtx));
		}
	}
	return false;
};

/**
	Call this function to know if all required prompts have valid inputs.
*/
CCognosViewer.prototype.canSubmitPrompt = function()
{
	var oPromptElement = null;

	if (this.preProcessControlArray && this.preProcessControlArray instanceof Array)
	{
		var kCount = this.preProcessControlArray.length;
		for ( var k=0; k < kCount; k++ )
		{
			oPromptElement = eval( this.preProcessControlArray[k] );
			if ( oPromptElement.isValid() === false )
			{
				return false;
			}
		}
	}

	return true;
};

/**
	Context Menu Handler in Report Viewer.
	dcm == Display Context Menu
	@private
*/
CCognosViewer.prototype.dcm = function(event, selectNode)
{
	if (this.canDisplayContextMenu())
	{
		if (this.preSelectNode == true) {
			selectNode = false;
			this.preSelectNode = false;
		}
		if (this.rvMainWnd.displayContextMenu(event, selectNode) != false)
		{
			if(this.sBrowser=='ie')
			{
				event.cancelBubble = true;
				event.returnValue = false;
			}
			else
			{
				event.stopPropagation();
				event.preventDefault();
			}
		}
	}
};

/**
 * Determines if the viewer should show a context menu
 * @private
 */
CCognosViewer.prototype.canDisplayContextMenu = function()
{
	return ( !this.isWorkingOrPrompting() && this.rvMainWnd != null && typeof this.bCanUseCognosViewerContextMenu != "undefined" && this.bCanUseCognosViewerContextMenu);
};

/**
	Handler for drill actions in Report Viewer.
	de == Drill Event
	@private
*/
CCognosViewer.prototype.de = function(event)
{
	var oDrillMgr = this.getDrillMgr();
	if (oDrillMgr)
	{
		oDrillMgr.singleClickDrillEvent(event, 'RV');
	}
};

/**
	@param {string} Message
	@private
*/
CCognosViewer.prototype.debug = function(sMsg)
{
	if (this.m_bDebug)
	{
		var sCallee = "";
		var oCaller = this.debug.caller;
		if (typeof oCaller == "object" && oCaller !== null)
		{
			sCallee = oCaller.toString().match(/function (\w*)/)[1];
		}
		if (!sCallee)
		{
			sCallee = '?';
		}
		alert(sCallee + ": " + sMsg);
	}
};

/**
	@param {string} sEvent Event name to call the callback functions for.
	@type boolean
	@return <i>true</i> if there was a callback associated with the event and that it's capture event was set to true. <i>false</i> otherwise.
	@private
*/
CCognosViewer.prototype.executeCallback = function(sEvent)
{
	var bEventWasCaptured = false;

	if (this.m_aCallback && this.m_aCallback.length)
	{
		for (var idxCallback = 0; idxCallback < this.m_aCallback.length; ++idxCallback)
		{
			var oCB = this.m_aCallback[idxCallback];

			if (oCB.m_sEvent == sEvent)
			{
				if (typeof oCB.m_oCallback == "function")
				{
					oCB.m_oCallback();
				}
				if (oCB.m_bCaptureEvent)
				{
					bEventWasCaptured = true;
				}
			}

		}
	}
	return bEventWasCaptured;
};

/**
	CAF context information.
	@return CAF encoded string.
	@type string
*/
CCognosViewer.prototype.getCAFContext = function()
{
	return this.m_sCAFContext;
};


/**
	Get the fault
	@return string
*/
CCognosViewer.prototype.getSoapFault = function()
{
	return this.m_sSoapFault;
};

/**
	Returns the identifiers of the columns related to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return array of identifiers of the columns making up this element.
	@type string[]
*/
CCognosViewer.prototype.getColumnContextIds = function(sId)
{
	return  this.getContextIds(sId, 2);
};

/**
	Conversation info (encoded string).
	@return CAF encoded string.
	@type string
*/
CCognosViewer.prototype.getConversation = function()
{
	return this.m_sConversation;
};

/**
 * Status of the ongoing report server conversation
 * @return conversation status
 * @type string
 */
CCognosViewer.prototype.getStatus = function()
{
	return (this.m_sStatus ? this.m_sStatus : "");
};

/**
 * Checks if current status is a working state
 * @param {string} sState? Optional parameter. If not a string, the current state of CCognosViewer will be used.
 * @return true/false
 * @type boolean
 */
CCognosViewer.prototype.isWorking = function(sState)
{
	if (typeof sState != "string") {
		sState = this.getStatus();
	}
	return ((""+sState).match(/^(working|stillWorking)$/) ? true : false);
};

/**
 * Checks if current status is a working or prompting state
 * @return true/false
 * @type boolean
 */
CCognosViewer.prototype.isWorkingOrPrompting = function()
{
	return (this.getStatus().match(/^(working|stillWorking|prompting)$/) ? true : false);
};

/**
 * Returns the action state. This value will only be set if the action is in a interupted state (wait/prompting)
 * @return action state
 * @type string
 */
CCognosViewer.prototype.getActionState = function()
{
	return this.m_sActionState;
};

/**
	Get the name of the ref data item associated with the element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Name of the ref data item associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getDataItemName = function(sId)
{
	var sName = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aName = selectionController.getRefDataItem(sCtx);
			if (aName) {
				sName = aName;
			}
		}
	}
	return sName;
};

/**
	Get the Name of the data type associated with the element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Name of the data type associated with the element. Returns <code>null</code> if there is not any.
	@type int
*/
CCognosViewer.prototype.getDataType = function(sId)
{
	var sType = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aType = selectionController.getDataType(sCtx);
			if (aType) {
				sType = aType;
			}
		}
	}
	return sType;
};


/**
	Returns the level associated with the element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Depth (relative to the hierarchy) of the element. Returns <code>null</code> if there is not any.
	@type integer
*/
CCognosViewer.prototype.getDepth = function(sId)
{
	var sLevel = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aLevel = selectionController.getDepth(sCtx);
			if (aLevel) {
				sLevel = aLevel;
			}
		}
	}
	return sLevel;
};

/**
	@return CAF encoded string.
	@type string
	@private
*/
CCognosViewer.prototype.getDrillMgr = function()
{
	return this.m_oDrillMgr;
};


/**
	@return the subscription handler.
	@type object
	@private
*/
CCognosViewer.prototype.getSubscriptionManager = function()
{
	return this.m_oSubscriptionManager;
};


/**
	Execution Parameters (encoded string).
	@return CAF encoded string.
	@type string
*/
CCognosViewer.prototype.getExecutionParameters = function()
{
	return this.m_sParameters;
};

/**
	@private
	@return Gateway path.
	@type string
*/
CCognosViewer.prototype.getGateway = function()
{
	return this.m_sGateway;
};
/**
	@public
	@return current specification
	@type string
*/
CCognosViewer.prototype.getSpecification = function()
{
	return this.envParams["ui.spec"];
};

/**
	Returns the Hierarchy Unique Name (HUN) associated to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Name (HUN) of the hierarchy associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getHierarchyUniqueName = function(sId)
{
	var sHun = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aHUN = selectionController.getHun(sCtx);
			if (aHUN) {
				sHun = aHUN;
			}
		}
	}
	return sHun;
};

/**
	Returns the Dimension Unique Name (DUN) associated to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Name (DUN) of the dimension associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getDimensionUniqueName = function(sId)
{
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aDUN = selectionController.getDun(sCtx);
			if (aDUN) {
				return aDUN;
			}
		}
	}
	return null;
};

/**
	@private
	@return Id Identifier for this instance of CCognosViewer.
	@type string
*/
CCognosViewer.prototype.getId = function()
{
	return this.m_sId;
};

/**
	Returns the Level Id (LUN) associated to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Identifier for the level associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getLevelId = function(sId)
{
	var sLevel = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aLUN = selectionController.getLun(sCtx);
			if (aLUN) {
				sLevel = aLUN;
			}
		}
	}
	return sLevel;
};

/**
	Returns the Member Unique Name (MUN) associated to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Name (MUN) of the member associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getMemberUniqueName = function(sId)
{
	var sMUN = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aMUN = selectionController.getMun(sCtx);
			if (aMUN) {
				sMUN = aMUN;
			}
		}
	}
	return sMUN;
};

/**
	This function return the name of the javascript reference (variable) to this object. The default is "window" if the id is invalid.
	@private
	@return Id Identifier for this instance of CCognosViewer.
	@type string
*/
CCognosViewer.prototype.getObjectId = function()
{
	var sObjId = "window";
	if (typeof this.getId() == "string") {
		sObjId = getCognosViewerObjectRefAsString(this.getId());
	}
	return sObjId;
};

/**
	Returns the Query Id associated to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Model id associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getQueryModelId = function(sId)
{
	var sQuery = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aQMID = selectionController.getQueryModelId(sCtx);
			if (aQMID) {
				sQuery = aQMID;
			}
		}
	}
	return sQuery;
};

/**
	Returns the Query name associated to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Query name associated with the element. Returns <code>null</code> if there is not any.
	@type string
*/
CCognosViewer.prototype.getQueryName = function(sId)
{
	var sQuery = null;
	var sCtx = this.findCtx(sId).split("::")[0];
	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			var aQuery = selectionController.getRefQuery(sCtx);
			if (aQuery) {
				sQuery = aQuery;
			}
		}
	}
	return sQuery;
};

CCognosViewer.prototype.getContextIds = function(sId, index)
{
	var aIds = [];
	var sCtx = this.findCtx(sId);
	if (sCtx)
	{
		var aIDparts = sCtx.split("::");
		if (aIDparts && aIDparts.length > 1 && index < aIDparts.length)
		{
			aIds = aIDparts[index].split(":");
		}
	}
	return aIds;
};


/**
	Returns the identifiers of the rows related to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return array of identifiers of the rows making up this element.
	@type string[]
*/
CCognosViewer.prototype.getRowContextIds = function(sId)
{
	return this.getContextIds(sId, 1);
};

/**
	Returns the identifiers of the page related to an element.
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return array of identifiers of the rows making up this element.
	@type string[]
 */
CCognosViewer.prototype.getPageContextIds = function(sId)
{
	return this.getContextIds(sId, 3);
};

/**
	@private
	@param {string} sKey the key of the string
	@type string
	@return The localized string matching the key. Returns the key if it there is not string associated.
*/
CCognosViewer.prototype.getString = function(sKey)
{
	if (RV_RES && RV_RES[sKey]) {
		return RV_RES[sKey];
	}
	return sKey;
};

/**
	@private
	@type CViewerManager
	@return A CViewerManager instance or a "window" object if the former do not exists.
*/
CCognosViewer.prototype.getRV = function()
{
	if (typeof this.m_oCVMgr == "object")
	{
		return this.m_oCVMgr;
	}
	return window;
};

/**
	@private
	@type array
	@return available secondary requests
*/
CCognosViewer.prototype.getSecondaryRequests = function()
{
	return this.m_aSecRequests;
};

/**
	@private
	@type string
	@return session prompts (CAF string)
*/
CCognosViewer.prototype.getSessionPrompts = function()
{
	return this.m_sSessionPrompts;
};

/**
	Tracking info (encoded string).
	@type string
	@return CAF encoded string.
*/
CCognosViewer.prototype.getTracking = function()
{
	return this.m_sTracking;
};

/**
	@private
	@type string
	@param {string} sId Identifier or a HTML element
	@return Returns an empty string if not found.
*/
CCognosViewer.prototype.findCtx = function(sId)
{
	var sCtx = "";
	if (typeof sId == "string")
	{
		// return sId if it matches something in the context data
		var sRefDataItem = sId.split("::")[0];
		var selectionController = this.getSelectionController();
		if (selectionController) {
			if (selectionController.isContextId(sRefDataItem))
			{
				sCtx = sId;
			}
		}
	}
	if (!sCtx)
	{
		var oHTMLElement = this.findElementWithCtx(sId);
		if (oHTMLElement)
		{
			sCtx = oHTMLElement.getAttribute("ctx");
		}
	}
	return sCtx;
};

/**
	@private
	@type HTMLelement
	@param {string} sId Identifier or a HTML element
	@return Returns a HTML element with a <i>ctx</i> attribute. Processed recursively on the children. Returns <code>null</node> if there is not any.
*/
CCognosViewer.prototype.findElementWithCtx = function(sId)
{
	var oElement = sId;
	if (typeof sId == "string") {
		oElement = this.findElementWithCtx(document.getElementById(sId));
	}
	if (oElement)
	{
		if (oElement.getAttribute && oElement.getAttribute("ctx"))
		{
			return oElement;
		}
		for (var idxChild = 0; idxChild < oElement.childNodes.length; idxChild++)
		{
			var oChild = this.findElementWithCtx(oElement.childNodes[idxChild]);
			if (oChild) {
				return oChild;
			}
		}
	}
	return null;
};

/**
	Returns the use value associated to an element.
	@type string
	@param {string} sId Identifier or the HTML element with a ctxId attribute.
	@return Use value associated with the element. Returns <code>null</code> if there is not any.
*/
CCognosViewer.prototype.getUseValue = function(sId)
{
	var sVal = null;

	var sCtx = this.findCtx(sId).split("::")[0];

	if (sCtx) {
		var selectionController = this.getSelectionController();
		if (selectionController) {
			sVal = selectionController.getUseValue(sCtx);
		}
	}
	return sVal;
};

/**
	@private
	@param {object} oProp Properties to add to this instance of CCognosViewer, like flags and strings.
*/
CCognosViewer.prototype.init = function(oProperties)
{
	if (oProperties && typeof oProperties == "object")
	{
		for (var sProp in oProperties)
		{
			this[sProp] = oProperties[sProp];
		}
	}
};

/**
 * @private
 * Method that should only be called when the user hits the back button in the browser
 */
CCognosViewer.prototype.initStandaloneViewerState = function(originalState) {
	var backJaxForm = document.getElementById('formBackJax' + this.getId());
	if (backJaxForm && typeof backJaxForm.responseText != "undefined" && backJaxForm.responseText.value.length > 0) {
		this.processResponseText(backJaxForm.responseText.value);
	}
	else {
		this.processResponseState(originalState);
	}
};

/**
 * @private
 * @Saves the responseXML for a completed request
 */
CCognosViewer.prototype.saveBackJaxInformation = function(responseText) {
	var backJaxForm = document.getElementById('formBackJax' + this.getId());
	if (backJaxForm && typeof backJaxForm.responseText != "undefined") {
		backJaxForm.responseText.value = responseText;
	}
};

/**
	Pre-selects a node prior handling of the context menu.
	pcc == Page Context Clicked
	@private
*/
CCognosViewer.prototype.pcc = function(evt)
{
	// we only want to preselect the node if the right mouse button was clicked.
	if (evt && typeof evt.button != "undefined" && evt.button != "1")
	{
		this.preSelectNode = true;
		var selectionController = this.getSelectionController();
		if(selectionController) {
			selectionController.pageContextClicked(evt);
		}
	}
};

/**
 * @private
 * @param {XMLHTTP response} oResponse
 */
CCognosViewer.prototype.isValidAjaxResponse = function(responseXML)
{
	return (responseXML && responseXML.childNodes && responseXML.childNodes.length > 0 && responseXML.childNodes[0].nodeName != "parsererror" ? true : false);
};

/**
 * @private
 */
CCognosViewer.prototype.resubmitInSafeMode = function()
{
	//re-submit the request in "page" mode. We'll need to inform the user we're doing this (Post beta)
	if (this.m_bUseSafeMode) {
		this.setUsePageRequest(true);
		this.envParams["cv.useAjax"] = "false";
		this.m_oServerRequest.clearRequest();
		this.sendRequest(this["LAST_ACTION_PARAMS"].param1);
	}
};

/**
	@private
	@param {XMLHTTP response} oResponse
*/
CCognosViewer.prototype.processResponse = function(oResponse)
{
	var oResponseState = null;
	try
	{
		var sContentType = oResponse.getResponseHeader("Content-type");
		this.m_stateSet = false;
		if(sContentType.indexOf("text/html") != -1)
		{
			// make sure we clear the tracking and conversation before we unload the viewer
			// or we'll trigger a cancel request which will more then likely cause another fault.
			this.m_sTracking = "";
			this.m_sConversation = "";
			clearLastAction(this);

			// this is an HTML response, most likely a fault page.
			document.write(oResponse.responseText);
		}
		// application/xml should only be used for data only response (i.e. no HTML)
		else if (sContentType.indexOf("application/xml") != -1)
		{
			var responseXML = oResponse.responseXML;
			if( !this.isValidAjaxResponse(responseXML) ) {
				// try re-parsing client side
				responseXML = XMLBuilderLoadXMLFromString(oResponse.responseText);
			}

			this.processResponseXML(responseXML, null);
		}
		else
		{
			var responseText = oResponse.responseText;

			if (responseText == null)
			{
				this.resubmitInSafeMode();
			}
			this.processResponseText(responseText);
		}
	}
	catch (e)
	{
		oResponseState = null;
		this.executeCallback("error");
	}
};

/**
	@private
	@param {XMLHTTP response} sText
	@param {Boundary from the header} sText
*/
CCognosViewer.prototype.processResponseText = function (sText)
{
	var boundaryIndex = sText.indexOf("</state></xml>");
	if (boundaryIndex == -1)
	{
		this.resubmitInSafeMode();
	}

	// length of "</state></xml>"
	boundaryIndex += 14;

	var xmlPart = sText.substring(0, boundaryIndex);
	var responseXML = XMLBuilderLoadXMLFromString(xmlPart);

	var responseHTML = sText.substring(boundaryIndex);

	if (this.processResponseXML(responseXML, responseHTML))
	{
		if (!this.isWorking())
		{
			clearLastAction(this);
		}
		this.saveBackJaxInformation(sText);
	}
};

/**
	@private
	@param XML element
	@param responseHTML
*/
CCognosViewer.prototype.processResponseXML = function (responseXML, responseHTML)
{
	var oResponseState = null;
	if( !this.isValidAjaxResponse(responseXML) ) {
		this.resubmitInSafeMode();
		return false;
	}
	else
	{
		try
		{
			var stateData = responseXML.getElementsByTagName("state");
			if (stateData != null && stateData.length > 0)
			{
				if (typeof stateData[0].text != "undefined")
				{
					oResponseState = eval("(" + stateData[0].text + ")");
				}
				else
				{
					oResponseState = eval("(" + stateData[0].textContent + ")");
				}
			}
			if (oResponseState)
			{
				if( oResponseState.soapFault )
				{
					var widget = this.getViewerWidget();
					if (widget) {
						this.setSoapFault(oResponseState.soapFault);
						widget.handleFault();
						return false;
					}

					var soapFaultDocument = XMLBuilderLoadXMLFromString( oResponseState.soapFault);
					if( processAuthenticationFault( soapFaultDocument, this.getId() ) )
					{
						return false;
					}
				}

				var bWaitPage = this.isWorking(oResponseState.status);
				var oRVContent = null;
				if (!bWaitPage && this.sBrowser == 'moz')
				{
					oRVContent = document.getElementById('RVContent' + this.getId());
					if (oRVContent)
					{
						if (typeof oResponseState.outputFormat != "undefined" && oResponseState.outputFormat == 'XML')
						{
							oRVContent.style.overflow = "hidden";
						}
						else
						{
							if (!this.isBux) {
								oRVContent.style.overflow = 'auto';
							}
						}
					}
				}

				if (typeof responseHTML == "string")
				{
					oRVContent = document.getElementById('RVContent' + this.getId());
					if (!bWaitPage && oRVContent)
					{
						oRVContent.scrollTop=0;
					}

					if (bWaitPage)
					{
						this.m_waitPage.initializeWaitPage(responseHTML);
					}
					else if(this.processResponseOutput(responseHTML) === false)
					{
						return;
					}
				}

				this.gbPromptRequestSubmitted = false;

				this.processResponseState(oResponseState);
			}
		}
		catch (e)
		{
			oResponseState = null;
			this.executeCallback("error");
			return false;
		}
	}
	return true;
};

/**
	@private
	@param {string} sResponseText
	@return (boolean)
*/
CCognosViewer.prototype.processResponseOutput = function(sResponseText)
{
	try
	{
		if(this.m_undoStack.length > 0)
		{
			this.m_undoStack[this.m_undoStack.length-1].m_bRefreshPage = true;
		}

		this.pageNavigationObserverArray = [];
		this.m_flashChartsObjectIds = [];

		// IE bug: innerHTML do not handle <form>s, the page will be ignored if there is a <form> tag around the HTML code.
		// The main form (formWarpRequest) is wrapped around the <div> we are using for the report, we should not have a <form> in the output anyway.
		var sHTML = sResponseText.replace(/<form[^>]*>/gi,"").replace(/<\/form[^>]*>/gi,"");

		this.m_sHTML = sHTML;
		this.setHasPrompts(false);

		var id = this.getId();
		var oRVContent = document.getElementById("RVContent" + id);
		var oReportDiv = document.getElementById("CVReport" + id);

		// if the report contains prompts, then set the display to none to fix a possible flicker for
		// when the prompt sizes itself (trakker 531880)
		if (sHTML.match(/prompt\/control\.js|PRMTcompiled\.js/gi))
		{
			this.setHasPrompts( true );
			oRVContent.style.display = "none";
		}

		if (window.gScriptLoader)
		{
			var bUseNamespacedGRS = this.getViewerWidget() ? true : false;
			var oCVDiv = this.getViewerWidget() ? document.getElementById("_" + this.getViewerWidget().iContext.widgetId + "_cv") : oReportDiv;
			sHTML = window.gScriptLoader.loadCSS(sHTML, oCVDiv, bUseNamespacedGRS, id);
		}

		if (this.sBrowser == 'ie')
		{
			// IE specific 'fix' where if sHTML has script tags at the beginning they won't be loaded into the DOM. Adding
			// a valid tag first causes all subsequent scripts to be loaded into the DOM.
			sHTML = "<span style='display:none'>&nbsp;</span>" + sHTML;
		}

		oReportDiv.innerHTML = sHTML;

		if (window.gScriptLoader)
		{
			var callback = GUtil.generateCallback(this.showLoadedContent, [oRVContent], this);
			this.m_resizeReady = false;
			if (!window.gScriptLoader.loadAll(sHTML, oReportDiv, callback, id) ) {
				if (window.gScriptLoader.containsAjaxWarnings()) {
					this.resubmitInSafeMode();
					return false;
				}
			}
		}
		else
		{
			oRVContent.style.display = "block";
		}

		this.updateBorderCollapse();
	}
	catch (e)
	{
	}

	return true;
};

/**
 *
 */
CCognosViewer.prototype.showLoadedContent = function(oRVContent)
{
	if (oRVContent != null && typeof oRVContent != "undefined") {
		oRVContent.style.display = "block";
	}
	this.m_resizeReady = true;
	this.doneLoading();
};

/**
* Called once the HTML and scripts have been loaded and processResponseState is finished.
* Will resize the widget and take care of any accessibility stuff needed (focus, JAWS, ...)
*/
CCognosViewer.prototype.doneLoading = function()
{
	var viewerIWidget = this.getViewerWidget();
	if (viewerIWidget) {
		var status = this.getStatus();
		if (!this.m_reportRenderingDone && this.m_resizeReady && this.m_stateSet) {
			var noAutoResize =  status == "working" || status == "stillWorking" || status == "fault";
			viewerIWidget.fireEvent("com.ibm.bux.widget.render.done", null, {noAutoResize:noAutoResize});
			if (status == "complete") {
				this.m_reportRenderingDone = true;
				if (!noAutoResize) {
					var thisObj = this;
					//need a delay because bux widget delays the resize operation.
					setTimeout(function() { thisObj.m_readyToRespondToResizeEvent = true; }, 20);
				}
			}
		}

		if (status != "fault") {
			viewerIWidget.clearErrorDlg();
		}

		this.doneLoadingUpdateA11Y(status);
	}
};

CCognosViewer.prototype.doneLoadingUpdateA11Y = function(status) {
	// if we're refreshing the report because of a modify report request, set the focus
	// to the first table in the report
	if (this.getKeepFocus() != false) {
		var keepFocus = this.getKeepFocus();

		if (status == "complete") {
			this.setKeepFocus(false);
		}

		var focusOn = null;
		if (this.getVisibleDialog() != null) {
			focusOn = this.getVisibleDialog().getDialogDiv();
		} else if (keepFocus == true) {
			focusOn = document.getElementById("CVReport" + this.getId());
		} else if (typeof keepFocus == "string") {
			focusOn = document.getElementById(keepFocus);
		}

		setFocusToFirstTabItem(focusOn);

		if (status == "complete") {
			this.JAWSTalk(RV_RES.IDS_JS_READY);
		} else if (status == "working" || status == "stillWorking") {
			this.JAWSTalk(RV_RES.IDS_JS_WAIT_PAGE_LOADING);
		}
	}
};

/**
 * Creates a div with a role of alert so that JAWS speaks the content of the div
 * @param {Object} sString
 */
CCognosViewer.prototype.JAWSTalk = function(sString) {
	var id = this.getId();
	var div = document.getElementById("JAWS_Alert_" + id);
	if (div) {
		div.parentNode.removeChild(div);
	}

	div = document.createElement("div");
	div.id = "JAWS_Alert_" + id;
	div.style.position = "absolute";
	div.style.top = "-9000px";
	div.setAttribute("role", "alert");
	div.appendChild(document.createTextNode(sString));

	var content = document.getElementById("RVContent" + id);
	if (content) {
		content.appendChild(div);
	} else if (typeof console != "undefined" && console && console.log) {
		console.log("CCognosViewer: Could not find the Viewer div to append the JAWS alert.");
	}
};


CCognosViewer.prototype.reExecuteLastRequest = function()
{
	var oRequest = this["LAST_ACTION_PARAMS"].param1;
	this.sendRequest(oRequest);
};

CCognosViewer.prototype.addPageAdornments = function()
{
	this.m_layoutElements = null;
	this.initFlashCharts();
	this.insertSortIconsForAllLists();
	this.addInfoBar();
};

CCognosViewer.prototype.addFlashChart = function(refId)
{
	this.m_flashChartsObjectIds.push(refId);
};

CCognosViewer.prototype.flashChartError = function(parameters) {
	// turn off animations, and refresh the report
	var cvWidget = this.getViewerWidget();
	var properties = cvWidget.getProperties();
	properties.setProperty("flashCharts", false);

	var redrawAction = new RedrawAction();
	redrawAction.setCognosViewer(this);
	redrawAction.isUndoable = function() { return false; };
	redrawAction.execute();
};

CCognosViewer.prototype.initFlashCharts = function()
{
	var viewerIWidget = this.getViewerWidget();
	if(this.m_flashChartsObjectIds.length > 0)
	{
		var reportTable = document.getElementById("rt" + this.getId());

		if (window.addEventListener) {
			reportTable.addEventListener("mousedown", onFlashChartRightClick, true);
		}
		else {
			var objectIds = {};
			var onmouseup = function() {
					this.releaseCapture();
			};
			var onmousedown = function() {
					onFlashChartRightClick(event);
					this.setCapture();
			};
			for (var i = 0; i < this.m_flashChartsObjectIds.length; ++i) {
				var objectId = this.m_flashChartsObjectIds[i];
				var flashObject = document.getElementById(objectId);
				objectIds[objectId] = 1;
				flashObject.parentNode.onmouseup = onmouseup;
				flashObject.parentNode.onmousedown = onmousedown;
			}
			if (this.m_flashChartsObjectIds.length > 0) {
				reportTable.attachEvent("oncontextmenu", function(){
					if (objectIds[window.event.srcElement.id]) {
						return false;
					}
				});
			}
		}
		if (viewerIWidget) {
			viewerIWidget.fireEvent("com.ibm.bux.widget.setShowBordersWhenInnactive", null, true);
		}
	} else {
		if (viewerIWidget) {
			viewerIWidget.fireEvent("com.ibm.bux.widget.setShowBordersWhenInnactive", null, false);
		}
	}
};

CCognosViewer.prototype.getLayoutElement = function(iLayoutIndex) {
	if (!this.m_layoutElements) {
		var reportTable = document.getElementById("rt" + this.getId());
		this.m_layoutElements = getElementsByAttribute(reportTable, "*", "lid");
	}

	if (this.m_layoutElements) {
		return this.m_layoutElements[iLayoutIndex];
	}

	return null;
};

CCognosViewer.prototype.addInfoBar = function()
{
	if(typeof this.envParams["rap.reportInfo"] != "undefined")
	{
		var reportTable = document.getElementById("rt" + this.getId());

		var reportInfoContainerArray = this.envParams["rap.reportInfo"];

		this.m_layoutElements = getElementsByAttribute(reportTable, "*", "lid");

		for(var containerIdx = 0; containerIdx < reportInfoContainerArray.length; ++containerIdx)
		{
			if (typeof reportInfoContainerArray[containerIdx].parentContainer == "undefined")
			{
				for(var layoutElementIdx = 0; layoutElementIdx < this.m_layoutElements.length; ++layoutElementIdx)
				{
					if(this.ifLayoutElementReportInfoMatch(this.m_layoutElements[layoutElementIdx], reportInfoContainerArray[containerIdx]))
					{
						var childContainers = this.collectChildContainers(reportInfoContainerArray[containerIdx].container, reportInfoContainerArray);
						var infoBar = new InfoBar(this, this.m_layoutElements[layoutElementIdx], reportInfoContainerArray[containerIdx], childContainers, layoutElementIdx);
						infoBar.render();
					}
				}
			}
		}
	}
};

CCognosViewer.prototype.collectChildContainers = function(parentContainerId, reportInfoContainerArray)
{
	var childContainers = [];
	for (var cidx = 0; cidx < reportInfoContainerArray.length; ++cidx)
	{
		if (typeof reportInfoContainerArray[cidx].parentContainer != "undefined" &&
					reportInfoContainerArray[cidx].parentContainer == parentContainerId)
		{
			childContainers.push(reportInfoContainerArray[cidx]);
		}
	}
	return childContainers;
};

//Return a map of the sliders in this report indexed by ClientID.
CCognosViewer.prototype.collectSliderSetFromReportInfo = function() {
	//collect the sliders from reportInfo as a set.
	var reportInfo = this.envParams["rap.reportInfo"];
	var reportInfoSliderIDs = {};
	if (typeof reportInfo == "undefined") {
		return reportInfoSliderIDs;
	}
	for(index = 0; index < reportInfo.length; ++index) {
		var container = reportInfo[index];
		var sliders = container.sliders;
		if(typeof sliders != "undefined" && sliders != null) {
			for(var sliderIndex = 0; sliderIndex < sliders.length; ++sliderIndex) {
				var sliderClientID = sliders[sliderIndex].clientId;
				reportInfoSliderIDs[sliderClientID] = sliders[sliderIndex];
			}
		}
	}
	return reportInfoSliderIDs;
};

CCognosViewer.prototype.ifLayoutElementReportInfoMatch = function(layoutElement, reportInfoContainer)
{
	var lid = layoutElement.getAttribute("lid");
	var container = reportInfoContainer.container;
	var id = this.getId();
	//the layout element with its lid attribute having RAP_ prefix is added by RAP for no data handler.
	return (lid == (container + id)) || (lid == ("RAP_NDH_" + container + id));
};

CCognosViewer.prototype.addReportInfo = function()
{

	var widget = this.getViewerWidget();
	if( typeof widget === "undefined" || widget === null)
	{
		return;
	}

	var baseReportModificationTime = this.envParams["baseReportModificationTime"];
	var savedBaseReportModificationTime = widget.getAttributeValue( "baseReportModificationTime" );
	if( typeof baseReportModificationTime !== "undefined" &&
		typeof savedBaseReportModificationTime !== "undefined" &&
		savedBaseReportModificationTime &&
		savedBaseReportModificationTime != '<empty>' &&
		baseReportModificationTime !== savedBaseReportModificationTime )
	{

		var cvid = this.getId();
		var rvContent = document.getElementById("CVReport" + cvid);
		var rvContentParent = rvContent.parentNode;

		var id = "ReportInfo" + cvid;
		var reportInfoTitle = RV_RES.IDS_JS_REPORT_INFO_TITLE;
		var reportInfoText = RV_RES.IDS_JS_REPORT_INFO_TEXT;
		var reportInfoLinkText = RV_RES.IDS_JS_REPORT_INFO_LINK_TEXT;
		var divElement = document.createElement("div");
		divElement.setAttribute("id", id + "_container");
		divElement.setAttribute("cvid", cvid);
		divElement.className = "new-info-indicator";

		var imgElement = document.createElement( "img");
		imgElement.src = this.getWebContentRoot() + "/rv/images/action_show_info.gif" ;
		imgElement.setAttribute("tabIndex", "0");
		imgElement.setAttribute("alt", "");
		imgElement.setAttribute("title", "");
		imgElement.setAttribute("role", "presentation");

		var cvScope = this;
		var onCVClickFunction = function()
		{
			cvScope.executeAction( "ResetToOriginal" );
		};

		imgElement.onmouseover = function()
		{
			this.m_tooltip = new bux.reportViewer.ReportInfo({
					connectId: [id + "_container"],
					position: ["above","below"],
					title: reportInfoTitle,
					text: reportInfoText,
					linkText: reportInfoLinkText,
					link: onCVClickFunction,
					allowMouseOverToolTip: true
			});
			this.m_tooltip.open();
		};
		imgElement.onmouseout = function()
		{
			this.m_tooltip = null;
		};

		imgElement.onfocus = function(evt)
		{
			this.m_tooltip = new bux.reportViewer.ReportInfo({
					connectId: [id + "_container"],
					position: ["above","below"],
					title: reportInfoTitle,
					text: reportInfoText,
					linkText: reportInfoLinkText,
					link: onCVClickFunction,
					allowMouseOverToolTip: true
			});
			this.m_tooltip.open();
		};

		imgElement.onblur = function(evt) {
			if (this.m_tooltip)
			{
				this.m_tooltip.close();
				this.m_tooltip = null;
			}
		};

		divElement.appendChild( imgElement );
		rvContentParent.insertBefore( divElement, rvContent );
	}
};

/*
 * This method hides ReportInfo (if it exist) when 'Reset to Original' option is invoked
 * Currently, only invoked by ResetToOriginalAction
 */
CCognosViewer.prototype.hideReportInfo = function()
{
	var reportInfoContainerDiv = document.getElementById( "ReportInfo" + this.getId() + "_container" );
	if( typeof reportInfoContainerDiv !== "undefined" && reportInfoContainerDiv !== null )
	{
		reportInfoContainerDiv.style.visibility =  "hidden";
	}
};

CCognosViewer.prototype.insertSortIconsForAllLists = function()
{
	if( this.isLimitedInteractiveMode() )
	{
		return;
	}
	var reportDiv = document.getElementById("CVReport" + this.getId());
	var listTableElements = getElementsByClassName(reportDiv, "table", 'ls');
	for (var i = 0; i < listTableElements.length; ++i)
	{
		this.insertSortIconsToList(listTableElements[i]);
	}
};

CCognosViewer.prototype.insertSortIconsToList = function(listTableElement)
{
	var columnHeaderElements = getElementsByAttribute(listTableElement, '*', "type", 'columnTitle');
	for (var i = 0; i < columnHeaderElements.length; ++i) {
		var columnHeader = columnHeaderElements[i];

		// we need to create a selection object for the column header so any missing context information
		// can be calculated
		this.getSelectionController().getSelectionObjectFactory().getSelectionObject(columnHeader);
		// double check to ensure we're not adding the sort icon to a crosstab node memeber or a crosstab corner
		if (columnHeader.getAttribute("canSort") != "false" && columnHeader.getAttribute("CTNM") === null && columnHeader.getAttribute("CC") === null) {
			var sortImgAlreadyInserted = false;

			for (var iChild=0; iChild < columnHeader.childNodes.length; iChild++) {
				var child = columnHeader.childNodes[iChild];
				if (child.nodeName.toLowerCase() == "img" )
				{
					// make sure we didn't already add a sort icon. Happens when we have lists inside of lists
					if( child.id && child.id.indexOf("sortimg") === 0)
					{
						sortImgAlreadyInserted = true;
						break;
					}

					//remove the sort icon from QS report
					var sLid = child.getAttribute( "lid");
					if(sLid && sLid.indexOf("SortIcon") !== -1)
					{
						columnHeader.removeChild( child );
						break;
					}
				}

			}

			if (!sortImgAlreadyInserted) {
				this.insertSortIconToColumnHeader(columnHeader);
			}
		}
	}
};


CCognosViewer.prototype.insertSortIconToColumnHeader = function(oColumnHeaderElement) {
	oColumnHeaderElement.setAttribute( "nowrap", "true");

	var imgElement = document.createElement("img");
	imgElement.setAttribute("id", 'sortimg' + Math.random());
	imgElement.onmouseover = function() {this.setAttribute("oldClassName", this.className); this.className += " sortIconOver";};
	imgElement.onmouseout = function() {this.className = this.getAttribute("oldClassName"); this.removeAttribute("oldClassName");};
	imgElement.src = this.getImgSrc(oColumnHeaderElement );


	var sortInfo = this.getSortInfo(oColumnHeaderElement);
	var sSort = this.getSortOrder(sortInfo);
	imgElement.setAttribute( 'alt', this.getSortAltText( sSort ));
	imgElement.className = this.getSortClass( sortInfo );
	imgElement.setAttribute('sortOrder', sSort);

	oColumnHeaderElement.appendChild( imgElement );
};

CCognosViewer.prototype.fillInContextData = function() {
	if (!this.isLimitedInteractiveMode()) {
		var reportDiv = document.getElementById("CVReport" + this.getId());
		var listTableElements = getElementsByClassName(reportDiv, "table", 'ls');
		for (var i = 0; i < listTableElements.length; ++i) {
			var columnHeaderElements = getElementsByAttribute(listTableElements[i], '*', "type", 'columnTitle');
			for (var j = 0; j < columnHeaderElements.length; ++j) {
				this.getSelectionController().getSelectionObjectFactory().getSelectionObject(columnHeaderElements[j]);
			}
		}
	}
};

CCognosViewer.prototype.getSortAltText = function (sSort)
{
	if( sSort === "ascending" )
	{
		return RV_RES.IDS_JS_SORT_ASCENDING;
	}
	else if( sSort === "descending")
	{
		return RV_RES.IDS_JS_SORT_DESCENDING;
	} else if (sSort === "nosort")
	{
		return RV_RES.IDS_JS_DONT_SORT;
	}
};

CCognosViewer.prototype.getSortInfo = function (oColumnHeaderElement)
{
	var sortInfo = oColumnHeaderElement.getAttribute('rp_sort');
	if (sortInfo)
	{
		sortInfo = sortInfo.split('.');
	}

	return sortInfo;
};


CCognosViewer.prototype.getSortClass = function( sortInfo )
{
	var classname = 'sortIconHidden';
	if (sortInfo)
	{
		if( sortInfo[0] === 'd'|| sortInfo[0] === 'a')
		{
			classname = 'sortIconVisible';
		}
	}
	return classname;
};

CCognosViewer.prototype.getSortOrder = function( sortInfo )
{
	var sortOrder = 'nosort';
	if( sortInfo )
	{
		if( sortInfo[0] === 'd')
		{
			sortOrder = 'descending';
		}
		else if( sortInfo[0] === 'a')
		{
			sortOrder = 'ascending';
		}
	}
	return sortOrder;
};


CCognosViewer.prototype.getImgSrc = function( oColumnHeaderElement )
{
	var sortOrder = oColumnHeaderElement.getAttribute('rp_sort');
	var src = this.getWebContentRoot() + "/rv/images/" + this.getSortIconName( sortOrder );
	return src;
};

CCognosViewer.prototype.getSortIconName = function( sortOrder )
{
	var iconName = 'sort_no.gif';
	if (sortOrder) {
		sortOrder = sortOrder.split('.');
		if( sortOrder[0] === 'd')
		{
			iconName = 'sort_descending.gif';
		}
		else if( sortOrder[0] === 'a')
		{
			iconName = 'sort_ascending.gif';
		}
	}
	return iconName;
};

/**
	@private
	@param {object} oState
*/
CCognosViewer.prototype.processResponseState = function(oState)
{
	this.setStatus(oState.status);
	var formWarpRequest;
	if(oState.status != "fault")
	{
		if (oState.conversation && oState.conversation.length > 0) {
			this.setConversation(oState.conversation);
		}
		this.setTracking(oState.tracking);
		this.setCAFContext(oState.caf);
		this.setExecutionParameters(oState.parameters);
		this.setSecondaryRequests(oState.secondaryRequests);
		this.setSessionPrompts(oState.session_prompts);
		this.setActionState(oState.action_state);

		if(typeof oState.spec != "undefined")
		{
			if(oState.spec.indexOf("&lt;") === 0)
			{
				oState.spec = xml_decode(oState.spec);
			}

			this.envParams["ui.spec"] = oState.spec;
		}

		if (typeof oState.rapState != "undefined")
		{
			this.envParams["rap.state"] = oState.rapState;
		}

		if(typeof oState.rapReportInfo != "undefined")
		{
			this.envParams["rap.reportInfoObject"] = eval("(" + oState.rapReportInfo + ")");
			this.envParams["rap.reportInfo"] = this.envParams["rap.reportInfoObject"].containers;
			this.envParams["json.infoBar"] = oState.rapReportInfo;
		}

		if(typeof oState.clientunencodedexecutionparameters != "undefined")
		{
			formWarpRequest = document.getElementById("formWarpRequest" + this.getId());
			if(formWarpRequest != null && typeof formWarpRequest["clientunencodedexecutionparameters"] != "undefined")
			{
				formWarpRequest["clientunencodedexecutionparameters"].value = oState.clientunencodedexecutionparameters;
			}

			if(typeof document.forms["formWarpRequest"] != "undefined" && typeof document.forms["formWarpRequest"]["clientunencodedexecutionparameters"] != "undefined")
			{
				document.forms["formWarpRequest"]["clientunencodedexecutionparameters"].value = oState.clientunencodedexecutionparameters;
			}
		}

		if (oState.sResponseSpecification)
		{
			this.updateResponseSpecification(oState.sResponseSpecification);
		}

		if(typeof this.envParams != "undefined")
		{
			this.envParams["ui.primaryAction"] = oState.primary_action;
		}

		if( typeof oState['flashChartOption'] !== "undefined")
		{
			this.envParams["flashChartOption"] = eval( oState.flashChartOption);
		}

		if( typeof oState['modelPath'] !== "undefined")
		{
			this.envParams["modelPath"] = oState.modelPath;
		}

		if( typeof oState['metaDataModelModificationTime'] !== "undefined")
		{
			this.envParams["metaDataModelModificationTime"] = oState.metaDataModelModificationTime;
		}

		if( typeof oState.productLocale != "undefined")
		{
			this.envParams["productLocale"] = oState.productLocale;
		}

		if( typeof oState.contentLocale != "undefined")
		{
			this.envParams["contentLocale"] = oState.contentLocale;
		}

		if( typeof oState.contentDecimalSeparator != "undefined")
		{
			this.envParams["contentDecimalSeparator"] = oState.contentDecimalSeparator;
		}

		if (this.getViewerWidget())
		{
			if(oState['buxToolbarPayload'])
			{
				this.buxToolbarPayload = eval(oState['buxToolbarPayload']);
			}

			if(oState['buxContextMenuPayload'])
			{
				this.buxContextMenuPayload = eval(oState['buxContextMenuPayload']);
			}

			if (oState.reportPrompts)
			{
				this.envParams["reportPrompts"] = oState.reportPrompts;
				this.getViewerWidget().setPromptParametersRetrieved(true);
			}

			if( oState.baseReportModificationTime )
			{
				this.envParams["baseReportModificationTime"] = oState.baseReportModificationTime;
			}
			if (oState.objectPermissions)
			{
				this.envParams["cv.objectPermissions"] = oState.objectPermissions;
			}

			if (oState.creationTime)
			{
				this.envParams["creationTime"] = oState.creationTime;
			}

			if (oState.object && oState.object.length > 0)
			{
				this.envParams["ui.object"] = oState.object;
			}

			if (oState.reRunObject && oState.reRunObject.length > 0)
			{
				formWarpRequest = document.getElementById("formWarpRequest" + this.getId());
				if (typeof formWarpRequest["reRunObj"] != "undefined")
				{
					formWarpRequest["reRunObj"].value = oState.reRunObject;
				}
				else
				{
					formWarpRequest.appendChild(createHiddenFormField('reRunObj', oState.reRunObject));
				}
				this.envParams["ui.reRunObj"] = oState.reRunObject;
			}

			if( oState.limitedInteractiveMode && oState.limitedInteractiveMode.length > 0 )
			{
				this.envParams["limitedInteractiveMode"] = oState.limitedInteractiveMode;
			}

			if( oState.updateToolbar && oState.updateToolbar.length > 0 )
			{
				this.envParams["updateToolbar"] = oState.updateToolbar;
			}

			if( oState.action && oState.action.length > 0)
			{
				this.envParams["ui.action"] = oState.action;
			}
		}
	}
	else
	{
		// clear the tracking, so we don't attempt to re-use it on subsequent requests
		this.setTracking("");
	}

	var bShowWaitPage = ( this.isWorking(oState.status) || oState.status == "default" );
	if( !bShowWaitPage )
	{
		this.removeTransparentBackgroundLayer();
	}

	if (typeof g_firefoxPluginHelper != "undefined" && g_firefoxPluginHelper != null)
	{
		g_firefoxPluginHelper.processResponseState(oState, this);
	}

	if(this.getStatus() == "complete")
	{
		this.m_undoStack = [new CognosViewerSession(this)];
	}

	if (bShowWaitPage)
	{
		if (this.isWorking(oState.status) || oState.status == "default")
		{
			var waitTable = document.getElementById("CVWaitTable" + this.getId());
			if (waitTable && waitTable.getAttribute("simpleDialog") == "true")
			{
				this.m_waitPage.destroy();
				waitTable = null;
			}

			if (typeof waitTable == "undefined" || waitTable == null || waitTable.parentNode.id == "CVWait" + this.getId())
			{
				this.m_waitPage.setCognosViewer(this);
				this.m_waitPage.show(this.bIsSavedReport);
			}
		}

		// add a delay for this callback to allow the server request to clean up, and not block any futher server requests
		setTimeout(getCognosViewerObjectRefAsString(this.getId())+".executeCallback(\"wait\");",10);
	}
	else if(oState.status == "cancel")
	{
		this.executeCallback("cancel");
	}
	else if (oState.status == "fault")
	{
		this.setSoapFault(oState.soapFault);
		this.m_waitPage.hide();
		this.executeCallback("fault");
	}
	else
	{
		this.getWaitPageImpl().setCancelSubmitted(false);
		this.updateSkipToReportLink(oState.status == "prompting");

		if (oState.status != "prompting" || !this.executeCallback("prompt"))
		{
			if (this.shouldWriteNavLinks()) {
				this.writeNavLinks(this.getSecondaryRequests().join(" "));
			}

			if (this.rvMainWnd)
			{
				this.updateLayout(oState.status);
				var oToolbar = this.rvMainWnd.getToolbar();
				if (oToolbar)
				{
					this.rvMainWnd.updateToolbar(oState.outputFormat);
					oToolbar.draw();
				}

				var oBannerToolber = this.rvMainWnd.getBannerToolbar();
				if (oBannerToolber)
				{
					oBannerToolber.draw();
				}
			}

			this.m_waitPage.hide();
			this.executeCallback("done");
		}
		else
		{
			this.updateSkipToNavigationLink(true);
		}
	}

	if (this.getViewerWidget() && window["CVEditContentActionInstance"] && (window["CVEditContentActionInstance"].getViewerId() == this.m_sId))
	{
		window["CVEditContentActionInstance"].setViewerEventReceived();
	}

	var oReportDiv = document.getElementById("CVReport" + this.getId());
	if (typeof oReportDiv != "undefined" && oReportDiv != null)
	{
		oReportDiv.style.display = "";
	}

	this.executeCallback("processResponseCompleted");
};

CCognosViewer.prototype.shouldWriteNavLinks = function() {
	if (this.rvMainWnd || this.isBux) {
		return true;
	}

	return false;
};

CCognosViewer.prototype.buxAjaxResponseHandler = function()
{
	this.hideExecutionCursor();
	var viewerIWidget = this.getViewerWidget();
	if(viewerIWidget != null)
	{
		switch(this.getStatus())
		{
			case "complete":
			// make sure all our selections are cleared.
				this.getSelectionController().clearSelectionData();
				viewerIWidget.savedOutputMenuUpdated = false;
				viewerIWidget.fetchAnnotationData();
				viewerIWidget.getUndoRedoQueue().add({"reportUpdated": true});
				viewerIWidget.updateToolbar();
				break;
			case "prompting":
				viewerIWidget.hideToolbarAndContextMenu();
				break;
			case "fault":
				if( this.envParams.updateToolbar === "true")
				{
					viewerIWidget.savedOutputMenuUpdated = false;
					viewerIWidget.updateToolbar();
				}
				viewerIWidget.handleFault();
				break;
		}

		this.m_stateSet = true;
		this.doneLoading();
	}
	else
	{
		setTimeout("window[\"oCV" + this.getId() + "\"].buxAjaxResponseHandler();", 100);
	}
};

CCognosViewer.prototype.doRaiseSharePromptEvent = function()
{
	var result = false;
	var action = this.m_oServerRequest.m_sAction;
	if ( (typeof action !== "undefined" && ( action === "forward" ||   action === "back" ) ) && this.m_raiseSharePromptEvent )
	{
		result = true;
		var requestParam = null;
		var credentialParamExists = false;
		var promptParamsCount = 0;
		for(requestParam in this.m_oServerRequest.m_oParams)
		{
			requestParam = decodeURIComponent(requestParam);
			// don't share credential parameters or search request
			if( requestParam == "_promptControl" && this.m_oServerRequest.m_oParams[requestParam] == "search")
			{
				result = false;
				break;
			} else if (requestParam.indexOf("p_") === 0) {
				if (requestParam.indexOf("p_credential:") === 0) {
					credentialParamExists = true;
				} else {
					promptParamsCount += 1;
				}
			}
		}
		if (result && credentialParamExists && promptParamsCount === 0) {
			result = false;
		}
	}

	return result;
};

CCognosViewer.prototype.postAJAXRequest= function()
{
	this.m_currentNodeFocus = null;

	var showExecutionCursor = true;

	if( this.doRaiseSharePromptEvent() )
	{
		if (this.widgetHasPromptParameters())
		{
			showExecutionCursor = false;
			this.m_oServerRequest.setAction(this.envParams["ui.action"]);
			this.raisePromptEvent(this.envParams["reportPrompts"]);
		}
		else
		{
			var cvWidget = this.getViewerWidget();
			if ( !(cvWidget && cvWidget.promptParametersRetrieved == true) ) {
				//run getTransients to get the prompt parameters for this widget
				var getParametersAction = new GetParametersAction();
				getParametersAction.setCognosViewer(this);
				getParametersAction.isPrimaryPromptWidget = true;
				getParametersAction.execute();
			}
		}
	}

	this.resetRaiseSharePromptEventFlag();

	if(showExecutionCursor)
	{
		this.showExecutionCursor();
	}

};

CCognosViewer.prototype.resetRaiseSharePromptEventFlag = function()
{
	this.m_raiseSharePromptEvent = true;
};

CCognosViewer.prototype.disableRaiseSharePromptEvent = function()
{
	this.m_raiseSharePromptEvent = false;
};

CCognosViewer.prototype.widgetHasPromptParameters = function()
{
	var cvWidget = this.getViewerWidget();
	return (cvWidget && cvWidget.promptParametersRetrieved == true && this.envParams && typeof this.envParams["reportPrompts"] != "undefined" && this.envParams["reportPrompts"] != null && this.envParams["reportPrompts"].length > 0);
};
// ***
/**
 *  Returns the pompt parameters info
 * @return { transientSpec: TransientSpec,value:parameter values}
*/
CCognosViewer.prototype.getPromptParametersInfo = function()
{
	var result = null;
	if (this.widgetHasPromptParameters()) {
		result = "<widget><parameterValues>" + sXmlEncode(this.getExecutionParameters()) + "</parameterValues>" + this.envParams["reportPrompts"] + "</widget>" ;
	}
	return result;
};

CCognosViewer.prototype.raisePromptEvent = function(sharedPrompts)
{
	try
	{
		var cvWidget = this.getViewerWidget();
		var xmlDom = XMLBuilderLoadXMLFromString(sharedPrompts);
		var cvTransientSpec = xmlDom.firstChild;
		var reportParameters = XMLHelper_FindChildByTagName( cvTransientSpec, "reportParameters", true);
		var reportParameterNodes = XMLHelper_FindChildrenByTagName( reportParameters, "reportParameter", false);
		var parametersArray = [];
		for ( var node in reportParameterNodes )
		{
			var sParameterName = reportParameterNodes[node].getAttribute( "parameterName");
			var sp_ParameterName = 'p_' + encodeURIComponent( sParameterName );
			if( typeof this.m_oServerRequest.m_oParams[ sp_ParameterName  ] !== "undefined")
			{
				var sModelItem = reportParameterNodes[node].getAttribute( "modelItem");
				var multivalued = reportParameterNodes[node].getAttribute( "multivalued");
				var paramAttributes = {
					parmName: sParameterName,
					parmValue: decodeURIComponent(this.m_oServerRequest.m_oParams[ sp_ParameterName ]),
					modelItem: sModelItem
				};
				if ( multivalued !== null ) {
					paramAttributes.multivalued = true;
				}
				parametersArray.push( paramAttributes );
			}
		}

		var payload = { parameters: parametersArray, cv_id: cvWidget.iContext.widgetId, action: this.m_oServerRequest.m_sAction };
		cvWidget.fireEvent("com.ibm.bux.viewer.prompt", null, payload);

	} catch ( e )
	{
	}
};

CCognosViewer.prototype.showExecutionCursor = function()
{
	var cvid = this.getId();
	var executionCursor = document.getElementById("executionCursor" + cvid);
	if(executionCursor == null)
	{
		executionCursor = document.createElement("span");
		executionCursor.className = "executionCursor";
		executionCursor.setAttribute("id", "executionCursor" + cvid);

		var imageSrcHtml = "<img alt=\"\" src=\"" + this.getWebContentRoot() + "/rv/images/action_busy_32x32.gif" + "\"/>";

		executionCursor.innerHTML = imageSrcHtml;

		var rvContentDiv = document.getElementById("formWarpRequest" + cvid);
		rvContentDiv.appendChild(executionCursor);
	}
};

CCognosViewer.prototype.hideExecutionCursor = function()
{
	var executionCursor = document.getElementById("executionCursor" + this.getId());
	if(executionCursor != null)
	{
		executionCursor.parentNode.removeChild(executionCursor);
	}
};


CCognosViewer.prototype.setKeepFocus = function(value) {
	this.m_bKeepFocus = value;
};

CCognosViewer.prototype.getKeepFocus = function() {
	if (typeof this.m_bKeepFocus != "undefined") {
		return this.m_bKeepFocus;
	}

	return false;
};

CCognosViewer.prototype.updateCellAccessibility = function(srcNode, force) {
	if (!srcNode) {
		return false;
	}
	var canDrillDown = false;
	var canDrillUp = false;
	var canDrillThrough = false;
	var ctxNode = srcNode.getAttribute("ctx") != null ? srcNode : srcNode.parentNode;

	if (srcNode.getAttribute("flashChartContainer") != "true") {
		if (ctxNode.getAttribute("ctx") != null) {
			if (this.isBux) {
				var action = new DrillUpDownAction();
				action.updateDrillability(this, ctxNode);
				canDrillDown = action.canDrillDown();
				canDrillUp = action.canDrillUp();
			} else {
				var ctxAttribute = ctxNode.getAttribute("ctx");
				var ctxID = ctxAttribute.indexOf(':') == -1 ? ctxAttribute : ctxAttribute.substring(0, ctxAttribute.indexOf(":"));
				var selCon = this.getSelectionController();
				canDrillDown = selCon.canDrillDown(ctxID);
				canDrillUp = selCon.canDrillUp(ctxID);
			}
		}

		canDrillThrough = srcNode.parentNode.getAttribute("dtTargets") ? true : false;
	}

	var isImage = srcNode.nodeName.toLowerCase() == "img";
	var isColumnTitle = srcNode.parentNode.getAttribute("type") == "columnTitle";
	if ( !isImage && (force || ((srcNode.getAttribute("aria-labelledby") != null || isColumnTitle || this.isAccessibleMode())))) {

		var innerHTML = "";

		// crosstab corner
		if (srcNode.parentNode.getAttribute("cc") == "true") {
			innerHTML += " " + RV_RES.IDS_JS_CROSSTAB_CORNER;
		}

		if (srcNode.innerHTML.length === 0) {
			innerHTML += " " + RV_RES.IDS_JS_EMPTY_CELL;
		}

		if (canDrillDown && canDrillUp) {
			innerHTML += " " + RV_RES.IDS_JS_DRILL_DOWN_UP_JAWS;
		} else if (canDrillDown) {
			innerHTML += " " + RV_RES.IDS_JS_DRILL_DOWN_JAWS;
		} else if (canDrillUp) {
			innerHTML += " " + RV_RES.IDS_JS_DRILL_UP_JAWS;
		}

		if (canDrillThrough) {
			innerHTML += " " + RV_RES.IDS_JS_DRILL_THROUGH_JAWS;
		}

		if (srcNode.altText && srcNode.altText.length > 0) {
			innerHTML = srcNode.altText;
		} else if (srcNode.getAttribute("flashChartContainer") == "true") {
			innerHTML = RV_RES.IDS_JS_CHART_IMAGE;
		}


		if( this.isBux )
		{
			var sibling = srcNode.previousSibling;
			if (sibling) {
				var wid = sibling.getAttribute("widgetid");
				if (wid && wid.indexOf("comment")) {
					innerHTML += " " + RV_RES.IDS_JS_ANNOTATION_JAWS;
				}
			}

			if (srcNode.getAttribute("rp_name") || srcNode.parentNode.getAttribute("rp_name")) {
				innerHTML += " " + RV_RES.IDS_JS_LABEL_HAS_BEEN_RENAMED;
			}
		}

		// is there any extra information that JAWS needs to speak out
		if (innerHTML.length > 0) {
			this.addAriaLabelledByOnCell(srcNode, innerHTML);
		}
	}

	if (canDrillUp || canDrillDown || canDrillThrough) {
		this.addDrillAccessibilityAttributes(srcNode, canDrillThrough);
	}

	if(document.all) {
		srcNode.attachEvent("onblur", this.onBlur);
	} else {
		srcNode.addEventListener("blur", this.onBlur, false);
	}

	if ((document.all && srcNode.getAttribute("tabIndex") != 0) || isImage) {
		srcNode.setAttribute("modifiedTabIndex", "true");
		srcNode.setAttribute("oldTabIndex", srcNode.getAttribute("tabIndex"));
		srcNode.setAttribute("tabIndex", 0);
	}

};

CCognosViewer.prototype.addDrillAccessibilityAttributes = function(srcNode, canDrillThrough) {
	if (!srcNode.getAttribute("oldClassName")) {
		// drill throughs already have a link
		if (!canDrillThrough) {
			srcNode.setAttribute("oldClassName", srcNode.className);
			srcNode.className = "dl " + srcNode.className;
		}
		srcNode.setAttribute("oldRole", srcNode.getAttribute("role"));
		srcNode.setAttribute("role", "link");
	}
};

CCognosViewer.prototype.addAriaLabelledByOnCell = function(srcNode, labelledBy) {
	// can have multiple spans inside a td, get the position to help make the id unique
	var srcNodePos = 0;
	var tempNode = srcNode;
	while (tempNode.previousSibling) {
		srcNodePos++;
		tempNode = tempNode.previousSibling;
	}

	var hiddenSpanId = srcNode.getAttribute("ariaHiddenSpanId");

	// if we already have a hidden span, use it
	if (hiddenSpanId && document.getElementById(hiddenSpanId)) {
		document.getElementById(hiddenSpanId).innerHTML = labelledBy;
	}
	else {
		var newSpan = document.createElement("span");
		newSpan.style.visibility = "hidden";
		newSpan.style.display = "none";
		newSpan.id = (srcNode.id == "" ? srcNode.parentNode.id : srcNode.id) + "_" + srcNodePos;
		newSpan.innerHTML = labelledBy;
		srcNode.parentNode.appendChild(newSpan);

		var ariaLabelledBy = "";
		if (srcNode.getAttribute("aria-labelledby") != null) {
			ariaLabelledBy += srcNode.getAttribute("aria-labelledby");
		} else {
			if (srcNode.id == "") {
				srcNode.id = srcNode.parentNode.id + "_main_" + srcNodePos;
			}
			ariaLabelledBy += srcNode.id;
		}

		ariaLabelledBy += " " + newSpan.id;

		srcNode.setAttribute("aria-labelledby", ariaLabelledBy);
		srcNode.setAttribute("ariaHiddenSpanId", newSpan.id);
	}
};

CCognosViewer.prototype.findChildOfTableCell = function(startNode) {
	var srcNode = startNode;

	while(srcNode && srcNode.parentNode) {
		var nodeName = srcNode.parentNode.nodeName.toLowerCase();
		if (nodeName === "td" || nodeName === "th") {
			break;
		}
		srcNode = srcNode.parentNode;
	}

	return srcNode;
};

CCognosViewer.prototype.onBlur = function(evt) {
	var srcNode = null;
	if(document.all) {
		srcNode = getNodeFromEvent(evt, true);
	} else {
		srcNode = this;
	}

	// can't call this.findChildOfTableCell since in IE 'this' is actually the DOM node. Copied
	// the contents of the method here.
	while(srcNode && srcNode.parentNode) {
		var nodeName = srcNode.parentNode.nodeName.toLowerCase();
		if (nodeName === "td" || nodeName === "th") {
			break;
		}
		srcNode = srcNode.parentNode;
	}

	if (srcNode) {
		if (srcNode.getAttribute("oldClassName")) {
			srcNode.className = srcNode.getAttribute("oldClassName");
			srcNode.removeAttribute("oldClassName");
		}

		if (srcNode.getAttribute("oldRole")) {
			srcNode.setAttribute("role", srcNode.getAttribute("oldRole"));
			srcNode.removeAttribute("oldRole");
		}

		if (srcNode.getAttribute("modifiedTabIndex") == "true") {
			srcNode.removeAttribute("modifiedTabIndex");
			srcNode.removeAttribute("tabIndex");
			if (srcNode.getAttribute("oldTabIndex")) {
				srcNode.setAttribute("tabIndex", srcNode.getAttribute("oldTabIndex"));
			}
			srcNode.removeAttribute("oldTabIndex");
		}

		// blank out any extra info for JAWS when we leave the cell.
		var ariaSpanId = srcNode.getAttribute("ariaHiddenSpanId");
		if (ariaSpanId)
		{
			var ariaSpanEle = document.getElementById(ariaSpanId);
			if (ariaSpanEle)
			{
				ariaSpanEle.innerHTML = "";
			}
		}
	}
};

CCognosViewer.prototype.onFocus = function(evt) {

	var srcNode = getNodeFromEvent(evt);

	srcNode = this.findChildOfTableCell(srcNode);

	this.updateCellAccessibility(srcNode, false);
};

CCognosViewer.prototype.onKeyDown = function(evt) {
	//get the event in a cross-browser fashion
	evt = (evt) ? evt : ((event) ? event : null);

	var srcNode = evt.originalTarget ? evt.originalTarget : evt.srcElement;

	// In IE, if the user clicked on the white space instead of the text in a cell, the srcNode
	// will point to the TD or TH. Get the span within the TD or TH
	var currentNodeName = srcNode.nodeName.toLowerCase();
	if (currentNodeName === "td" || currentNodeName === "th") {
		for (var i=0; i < srcNode.childNodes.length; i++) {
			if (srcNode.childNodes[i].nodeName.toLowerCase() == "span") {
				srcNode = srcNode.childNodes[i];
				break;
			}
		}
	}

	// if the event didn't come from an element we'd select then let it bubble up
	if (!this.isValidNodeToSelect(srcNode)) {
		return true;
	}

	srcNode = this.findChildOfTableCell(srcNode);

	if (srcNode) {
		if (evt.keyCode == "39") {	// right arrow
			this.moveRight(srcNode);
			return stopEventBubble(evt);
		}
		else if (evt.keyCode == "37") {	// left arrow
			this.moveLeft(srcNode);
			return stopEventBubble(evt);
		}
		else if (evt.keyCode == "38") { // up arrow
			this.moveUp(srcNode);
			return stopEventBubble(evt);
		}
		else if (evt.keyCode == "40") { // down arrow
			this.moveDown(srcNode);
			return stopEventBubble(evt);
		}
		else if (evt.keyCode == "13") { // enter
			if (this.isBux) {
				this.getSelectionController().pageClicked(evt);
				var selectionAction = this.getActionFactory().load("Selection");
				selectionAction.onKeyDown(evt);
			} else {
				this.de(evt);
			}
		}
		else if (evt.keyCode == "32") { // space
			this.getSelectionController().pageClicked(evt);
			if (this.isBux)
			{
				this.getViewerWidget().updateToolbar();
			}
			return stopEventBubble(evt);
		}
		else if(evt.keyCode == "46") { // delete key
			if (typeof this.envParams != "undefined" &&
				typeof this.envParams["ui.action"] != "undefined" &&
				this.envParams["ui.action"] != "view" &&
				!this.isLimitedInteractiveMode())
			{
				var deleteAction = this.getActionFactory().load("Delete");
				if(deleteAction.canDelete())
				{
					deleteAction.execute();
					return stopEventBubble(evt);
				}
			}
		}
		else if (this.isBux && evt.ctrlKey == true && evt.shiftKey == true && evt.keyCode == "49") { // Ctrl + shift + !
			var lid = this.getSelectionController().getSelectionObjectFactory().getLayoutElementId(srcNode);

			if (lid != "") {
				var infoBarHeaderButton = document.getElementById("infoBarHeaderButton" + lid);
				if (infoBarHeaderButton !== null) {
					this.m_currentNodeFocus = evt.originalTarget ? evt.originalTarget : evt.srcElement;
					infoBarHeaderButton.focus();
				}
			}

			return stopEventBubble(evt);
		}
		else if (this.isBux && evt.shiftKey == true && evt.keyCode == "121") {  // Shift - F10  (context menu)
			this.getViewerWidget().onContextMenu(evt);
		}
	}
};

CCognosViewer.prototype.setFocusToNode = function(node) {

	this.m_currentNodeFocus = node;
	this.updateCellAccessibility(node, false);
	node.focus();
};

CCognosViewer.prototype.isValidNodeToSelect = function(node) {
	if (node && node.style && node.style.visibility != "hidden") {
		var sNodeName = node.nodeName.toLowerCase();
		if (
			sNodeName == "span" ||
			(sNodeName == "div" && node.getAttribute("flashchartcontainer") == "true") ||
			(sNodeName == "img" && (!node.id || node.id.indexOf("sortimg") !== 0 ) )
		) {
			return true;
		}
	}

	return false;
};

CCognosViewer.prototype.moveDown = function(srcNode) {
	var tdNode = srcNode.parentNode;
	var srcColSpan = this.getColumnIndex(tdNode);
	srcColSpan += getColSpanFromRowSpans(tdNode);

	// if the current node has a rowSpan, we need to jump over a bunch of TR's
	var trNode = tdNode.parentNode;
	if (tdNode.rowSpan && tdNode.rowSpan > 1) {
		var nodeRowSpan = tdNode.rowSpan;
		for (var rowSpanIndex=1; rowSpanIndex < nodeRowSpan; rowSpanIndex++) {
			trNode = trNode.nextSibling;
		}
	}
	var bTriedNextColumn = false;

	while(trNode) {
		if (trNode.nextSibling) {	// get the next TR
			trNode = trNode.nextSibling;
		} else if (tdNode.nextSibling && !bTriedNextColumn) {	// move to the next column
			trNode = trNode.parentNode.firstChild;
			bTriedNextColumn = true;
			srcColSpan++;
		} else {  // last span is selected
			return false;
		}

		if (this.doMoveUpDown(trNode, srcColSpan)) {
			return true;
		}
	}

	return false;
};

CCognosViewer.prototype.moveUp = function(srcNode) {
	var tdNode = srcNode.parentNode;
	var trNode = tdNode.parentNode;
	var srcColSpan = this.getColumnIndex(tdNode);
	srcColSpan += getColSpanFromRowSpans(tdNode);

	var bTriedPreviousColumn = false;

	while(trNode) {
		if (trNode.previousSibling) {	// get the next TR
			trNode = trNode.previousSibling;
		} else if (tdNode.nextSibling && !bTriedPreviousColumn) {	// move to the next column
			trNode = trNode.parentNode.lastChild;
			bTriedPreviousColumn = true;
			srcColSpan--;
			if (srcColSpan < 0) {
				return false;
			}
		} else {  // last span is selected
			return false;
		}

		if (this.doMoveUpDown(trNode, srcColSpan)) {
			return true;
		}
	}

	return false;
};

CCognosViewer.prototype.doMoveUpDown = function(trNode, srcColSpan) {
	if (trNode != null) {
		var currentColumn = trNode.firstChild;

		var pos = getColSpanFromRowSpans(currentColumn);

		while (currentColumn) {
			if (pos == srcColSpan) {
				return this.moveToTDFromTheRight(currentColumn);
			} else if (pos > srcColSpan) {
				break;
			}

			var nodeColSpan = 0;
			if (currentColumn.colSpan) {
				nodeColSpan = currentColumn.colSpan;
			} else {
				nodeColSpan++;
			}

			pos += nodeColSpan;

			currentColumn = currentColumn.nextSibling;
		}
	}
};

/**
 * Method that walks the tree from the given TD and calculates
 * the colspans that are from TD's with rowspan's on them
 */
function getColSpanFromRowSpans(tdNode) {
	var nodeColSpan = 0;
	var trNode = tdNode.parentNode;
	var trChildCount = 0;

	while (trNode) {
		var rowChildNode = trNode.firstChild;
		// if there's a diff in the #of columns, we must of found a new TD with a rowspan
		var colCountDiff = getColumnCount(trNode) - trChildCount;
		while (rowChildNode && rowChildNode.rowSpan > 1 && colCountDiff > 0 && rowChildNode != tdNode) {
			nodeColSpan += rowChildNode.colSpan;
			rowChildNode = rowChildNode.nextSibling;
			colCountDiff--;
		}

		// never decrease the column count, only increase it
		if (trNode.childNodes.length > trChildCount) {
			trChildCount = getColumnCount(trNode);
		}

		// get the previous TR.. keep walking the DOM
		trNode = trNode.previousSibling;
	}

	return nodeColSpan;
}

/**
 * Gets the column count for the given TR which includes all the colspans
 */
function getColumnCount(trNode) {
	var columnCount = 0;
	var node = trNode.firstChild;
	while (node) {
		columnCount += node.colSpan;
		node = node.nextSibling;
	}

	return columnCount;
}

/**
 * Returns the column index of the node with all the colspans.
 * This function excludes any td's that have a rowspan
 */
CCognosViewer.prototype.getColumnIndex = function(node) {
	var colIndex = 0;

	while (node.previousSibling) {
		node = node.previousSibling;

		if (node.rowSpan == 1) {
			if (node.colSpan) {
				colIndex += node.colSpan;
			} else {
				colIndex++;
			}
		}
	}

	return colIndex;
};

CCognosViewer.prototype.getPreviousNonTextSibling = function(node) {
	while (node.previousSibling) {
		node = node.previousSibling;
		if (node.nodeName.toLowerCase() != '#text') {
			return node;
		}
	}

	return null;
};

CCognosViewer.prototype.moveLeft = function(srcNode) {
	var previousNode = this.getPreviousNonTextSibling(srcNode);
	// case where we have multiple spans inside a td
	if (this.isValidNodeToSelect(previousNode)) {
		this.setFocusToNode(previousNode);
		return true;
	}

	var tdNode = srcNode.parentNode;
	while (tdNode.previousSibling) {
		if (this.moveToTDFromTheRight(tdNode.previousSibling)) {
			return true;
		}
		tdNode = tdNode.previousSibling;
	}

	var trNode = tdNode.parentNode;
	while (trNode.previousSibling) {
		var previousTR = trNode.previousSibling;
		if (this.moveToTDFromTheRight(previousTR.lastChild)) {
			return true;
		}
		trNode = trNode.previousSibling;
	}

	return false;
};

CCognosViewer.prototype.moveToTDFromTheRight = function(td) {
	var childNodes = td.childNodes;
	for (var iChildIndex=childNodes.length-1; iChildIndex >= 0; iChildIndex--) {
		var node = childNodes[iChildIndex];
		if (this.isValidNodeToSelect(node)) {
			// sometimes have a span inside a span
			if (node.childNodes && node.childNodes[0] && node.childNodes[0].nodeName.toLowerCase() == "span") {
				node = node.childNodes[0];
			}

			if (node.tabIndex != -1 && node.tabIndex != 0) {
				node.tabIndex = -1;
			}

			this.setFocusToNode(node);
			return true;
		}
	}

	return false;
};

CCognosViewer.prototype.getNextNonTextSibling = function(node) {
	while (node.nextSibling) {
		node = node.nextSibling;
		if (node.nodeName.toLowerCase() != '#text') {
			return node;
		}
	}

	return null;
};

CCognosViewer.prototype.moveRight = function(srcNode) {
	var nextSiblingNode = this.getNextNonTextSibling(srcNode);
	// case where we have multiple spans inside a td
	if (this.isValidNodeToSelect(nextSiblingNode)) {
		this.setFocusToNode(nextSiblingNode);
		return true;
	}

	var tdNode = srcNode.parentNode;
	while (tdNode.nextSibling) {
		if (this.moveToTD(tdNode.nextSibling)) {
			return true;
		}
		tdNode = tdNode.nextSibling;
	}

	var trNode = tdNode.parentNode;
	while (trNode.nextSibling) {
		var nextTR = trNode.nextSibling;
		if (this.moveToTD(nextTR.childNodes[0])) {
			return true;
		}
		trNode = trNode.nextSibling;
	}

	return false;
};

CCognosViewer.prototype.moveToTD = function(td) {
	var childNodes = td.childNodes;
	for (var iChildIndex=0; iChildIndex < childNodes.length; iChildIndex++) {
		if (this.isValidNodeToSelect(childNodes[iChildIndex])) {
			var node = childNodes[iChildIndex];
			// sometimes have a span inside a span
			if (node.childNodes && node.childNodes[0] && node.childNodes[0].nodeName.toLowerCase() == "span") {
				node = node.childNodes[0];
			}

			if (node.tabIndex != -1 && node.tabIndex != 0) {
				node.tabIndex = -1;
			}

			this.setFocusToNode(node);
			return true;
		}
	}

	return false;
};

/**
 * Function that will hide/show the jump to report link
 */
CCognosViewer.prototype.updateSkipToReportLink = function(bHide)
{
	var oSkipToReport = document.getElementById("cvSkipToReport" + this.getId());
	if (oSkipToReport)
	{
		oSkipToReport.style.display = bHide ? "none" :  "";
	}
};

/**
 * Function that will hide/show the jump to navigation link
 */
CCognosViewer.prototype.updateSkipToNavigationLink = function(bHide)
{
	var oSkipToNavigation = document.getElementById("cvSkipToNavigation" + this.getId());
	if (oSkipToNavigation)
	{
		oSkipToNavigation.style.display = bHide ? "none" :  "";
	}
};


CCognosViewer.prototype.pageAction = function(pageAction) {
	this.setKeepFocus("CVNavLinks" + this.getId());
	this.sendRequest(new CCognosViewerRequest(pageAction));
};

/**
	Helper function to generate one link for the page navigation.
	@private
	@param {object} oLink A object that contains the link info. oLink.text is the string to show to the user. oLink.sImg is the name of the icon to show for the normal (active) state. oLink.sImgDisabled is the name of the icon for the disabled state.
	@param {string} sRequest The request to send when the link is clicked. Possible values are <code>firstPage</code>, <code>nextPage</code>, <code>previousPage</code> and <code>lastPage</code>.
	@param {string} bActive True if the link should be active (normal state). False to disabled the link.
	@type string
	@return The HTML string that makes up the navigation links.
*/
CCognosViewer.prototype.writeNavLink = function(oLink, sRequest, bActive, bSavedOutput)
{
	var sPattern = "";
	if (bActive)
	{
		sPattern =
			'<td class="portlet-table-text" nowrap="nowrap">' +
				'<img src="LINK_IMG" width="15" height="15" alt="">' +
			'</td>' +
			'<td class="portlet-table-text" nowrap="nowrap">';

		if (bSavedOutput)
		{
			sPattern += '<a href="#" tabindex="0" onclick="' + getCognosViewerObjectRefAsString(this.getId()) + '.getViewerWidget().getSavedOutput().pageAction(\'LINK_REQUEST\');return false;"';
		}
		else
		{
			sPattern += '<a href="#" tabindex="0" onclick="' + getCognosViewerObjectRefAsString(this.getId()) + '.pageAction(\'LINK_REQUEST\');return false;"';
		}
		sPattern +=	'>LINK_TEXT</a>&#160;' +
		'</td>';
	}
	else
	{
		sPattern =
			'<td class="portlet-table-text" nowrap="nowrap">' +
				'<img src="LINK_IMG" width="15" height="15" alt="">' +
			'</td>' +
			'<td class="portlet-table-text" nowrap="nowrap">LINK_TEXT&#160;</td>';
	}

	var sImg = this.sSkin + (!bActive && oLink.sImgDisabled ? oLink.sImgDisabled : oLink.sImg);
	return sPattern.replace(/LINK_REQUEST/g, sRequest).replace(/LINK_TEXT/g, oLink.sText).replace(/LINK_IMG/g, sImg);
};

/**
	Loads up the necessary info for the navigation links (localized strings, icon names, skin to use).
	@private
*/
CCognosViewer.prototype.loadNavLinks = function()
{
	// load navlinks string and image path dynamically, synchronously.
	var sText = window.gScriptLoader.loadFile(this.getGateway(), "b_action=xts.run&m=portal/report-viewer-navlinks.xts");
	if (sText)
	{
		this.init(eval("(" + sText + ")"));
	}
};

/**
	Build the HTML code for the navigation links and update them in the UI.
	@param {string} sSR space separated secondary requests
	@private
*/
CCognosViewer.prototype.writeNavLinks = function(sSR, bSavedOutput)
{
	var oNavLinksDiv = document.getElementById("CVNavLinks" + this.getId());
	if (oNavLinksDiv)
	{
		var nNavLinksDivContainer = document.getElementById("CVNavLinks_Container" + this.getId());

		if (typeof this.oNavLinks != "object" || typeof sSR != "string" || !sSR.match(/\bfirstPage\b|\bpreviousPage\b|\bnextPage\b|\blastPage\b|\bplayback\b/i))
		{
			oNavLinksDiv.style.display = "none";
			if (nNavLinksDivContainer) {
				nNavLinksDivContainer.style.display = "none";
			}
			this.updateSkipToNavigationLink(true);
			return;
		}

		this.updateSkipToNavigationLink(false);

		if (nNavLinksDivContainer) {
			nNavLinksDivContainer.style.display = "";
		}

		oNavLinksDiv.style.display = (document.all ? "block" : "table-cell");
		oNavLinksDiv.style.padding = "1px";
		var aLinks = [
			this.writeNavLink(this.oNavLinks.oFirst, 'firstPage', sSR.match(/\bfirstPage\b/gi), bSavedOutput),
			this.writeNavLink(this.oNavLinks.oPrevious, 'previousPage', sSR.match(/\bpreviousPage\b/gi), bSavedOutput),
			this.writeNavLink(this.oNavLinks.oNext, 'nextPage', sSR.match(/\bnextPage\b/gi), bSavedOutput),
			this.writeNavLink(this.oNavLinks.oLast, 'lastPage', sSR.match(/\blastPage\b/gi), bSavedOutput)];

		var sHTML = "";
		sHTML += '<table border="0" cellpadding="0" cellspacing="0" class="pageControls"><tbody><tr>';

		sHTML += aLinks.join("");

		if ( sSR.match(/\bplayback\b/gi) )
		{
			sHTML += '<td width="100%" align="right" valign="bottom" class="portlet-table-text" style="height: 15px">' +
				'<table border="0" cellpadding="0" cellspacing="0" class="pageControls">' +
					'<tr>' +
						'<td valign="bottom" class="portlet-table-text">' +
							'<table border="0" cellpadding="0" cellspacing="0" class="pageControls">' +
								'<tr>' +
									'<td align="right" width="100%">' +
										'<img src="' + this.getWebContentRoot() + '/rv/images/playback.gif" width="15" height="15"' +
											' alt="' + RV_RES.PLAYBACK + '" tooltip="' + RV_RES.PLAYBACK + '">' +
									'</td>' +
								'</tr>' +
							'</table>' +
						'</td>' +
						'<td valign="bottom" class="portlet-table-text">' +
							'<a href="javascript:void(0);" onclick="' + getCognosViewerObjectRefAsString(this.getId()) + '.sendRequest(new CCognosViewerRequest(\'playback\'))">' +
								RV_RES.PLAYBACK +
							'</a>&#160;' +
						'</td>' +
					'</tr>' +
				'</table>' +
			'</td>';
		}
		sHTML += '</tr></tbody></table>';

		// if there was a hidden span for JAWS, keep it around.
		var oNavLinksLabel = document.getElementById("CVNavLinks_label" + this.getId());
		var navLinkLabel = "";
		if (oNavLinksLabel) {
			navLinkLabel += "<span id=\"CVNavLinks_label" + this.getId() + "\" style=\"visibilty:hidden; display:none;\">" + oNavLinksLabel.innerHTML + "</span>";
		}

		oNavLinksDiv.innerHTML = navLinkLabel + sHTML;
	}
	else
	{
		setTimeout(getCognosViewerObjectRefAsString(this.getId()) + '.writeNavLinks("' + sSR + '","' + bSavedOutput + '");', 100);
	}
};

/**
	@private
	@type CDispatcher
*/
CCognosViewer.prototype.getDispatcher = function()
{
	if (!this.m_oDispatcher)
	{
		this.m_oDispatcher = new CDispatcher();
	}
	return this.m_oDispatcher;
};

/**
	@private
	@param {CCognosViewerRequest} oRequest The request to send.
	@type string
	@return the URL to execute.
*/
CCognosViewer.prototype.generateRequestParams = function(oRequest)
{
	var oParams = {};
	// generate server specific request params
	this.m_oServerRequest.generateRequestParams(oParams, this.envParams);

	var actionState = this.getActionState();
	var action = oRequest.getAction();
	if (actionState != "" && ( action == "wait" || action == "forward" || action == "back"))
	{
		oParams["cv.actionState"] = actionState;
	}

	oParams["ui.action"] = action;

	var aOptions, idxOption, aParameters, idxParameter, param, sEP;
	if(this.getUsePageRequest() == true)
	{
		aOptions = oRequest.m_oOptions.keys();
		for (idxOption = 0; idxOption < aOptions.length; idxOption++)
		{
			oParams[aOptions[idxOption]] = oRequest.getOption(aOptions[idxOption]);
		}

		aParameters = oRequest.m_oParams.keys();
		for (idxParameter = 0; idxParameter < aParameters.length; idxParameter++)
		{
			oParams[aParameters[idxParameter]] = oRequest.getParameter(aParameters[idxParameter]);
		}

		for(param in this.envParams)
		{
			if(typeof oParams[param] == "undefined" && param != "cv.actionState")
			{
				oParams[param] = this.envParams[param];
			}
		}

		sEP = this.getExecutionParameters();
		if (sEP)
		{
			oParams.executionParameters = sEP;
		}

		oParams["ui.conversation"] = this.getConversation();
		oParams.m_tracking = this.getTracking();
		if (this.envParams["ui.routingServerGroup"] != null)
		{
			oParams["ui.routingServerGroup"] = this.envParams["ui.routingServerGroup"];
		}
	}
	else
	{
		if (typeof this.envParams["ui.object"] !== "undefined" && this.envParams["ui.object"] !== null)
		{
			oParams["ui.object"] = encodeURIComponent(this.envParams["ui.object"]);
		}

		if (typeof this.envParams["ui.primaryAction"] != "undefined")
		{
			oParams["ui.primaryAction"] = this.envParams["ui.primaryAction"];
		}

		if (this.envParams["cv.header"] != null)
		{
			oParams["cv.header"] = this.envParams["cv.header"];
		}
		if (this.envParams["cv.toolbar"] != null)
		{
			oParams["cv.toolbar"] = this.envParams["cv.toolbar"];
		}

		if (this.envParams["m_session"] != null)
		{
			oParams["m_session"] = this.envParams["m_session"];
		}

		if (this.envParams["ui.backURL"] != null)
		{
			oParams["errURL"] = encodeURIComponent(this.envParams["ui.backURL"]);
		}

		if(typeof this.envParams["ui.errURL"] != "undefined")
		{
			oParams["errURL"] = encodeURIComponent(this.envParams["ui.errURL"]);
		}

		if (this.envParams["cv.showFaultPage"] != null)
		{
			oParams["cv.showFaultPage"] = this.envParams["cv.showFaultPage"];
		}

		if (this.envParams["cv.debugDirectory"] != null)
		{
			oParams["cv.debugDirectory"] = this.envParams["cv.debugDirectory"];
		}

		if (this.envParams["ui.objectClass"] != null)
		{
			oParams["ui.objectClass"] = this.envParams["ui.objectClass"];
		}

		if (this.envParams["bux"] != null)
		{
			oParams["bux"] = this.envParams["bux"];
		}

		if (this.envParams["baseReportModificationTime"] != null)
		{
			oParams["baseReportModificationTime"] = this.envParams["baseReportModificationTime"];
		}

		if (this.envParams["originalReport"] != null)
		{
			oParams["originalReport"] = this.envParams["originalReport"];
		}

		if (this.envParams["specificationType"] != null)
		{
			oParams["specificationType"] = this.envParams["specificationType"];
		}

		if (this.envParams["cv.promptForDownload"] != null)
		{
			oParams["cv.promptForDownload"] = this.envParams["cv.promptForDownload"];
		}

		sEP = this.getExecutionParameters();
		if (sEP)
		{
			oParams.executionParameters = encodeURIComponent(sEP);
		}

		var sSP = this.getSessionPrompts();
		if (sSP)
		{
			oParams.session_prompts = encodeURIComponent(sSP);
		}

		aOptions = oRequest.m_oOptions.keys();
		for (idxOption = 0; idxOption < aOptions.length; idxOption++)
		{
			oParams[ encodeURIComponent( aOptions[idxOption] ) ] = encodeURIComponent( oRequest.getOption(aOptions[idxOption]) );
		}

		oParams["ui.conversation"] = encodeURIComponent(this.getConversation());
		oParams.m_tracking = encodeURIComponent(this.getTracking());

		aParameters = oRequest.m_oParams.keys();
		for (idxParameter = 0; idxParameter < aParameters.length; idxParameter++)
		{
			oParams[ encodeURIComponent( aParameters[idxParameter] ) ] = encodeURIComponent( oRequest.getParameter(aParameters[idxParameter]) );
		}

		var sCAF = this.getCAFContext();
		if (sCAF)
		{
			oParams["ui.cafcontextid"] = sCAF;
		}

		if (this.envParams["ui.routingServerGroup"] != null)
		{
			oParams["ui.routingServerGroup"] = encodeURIComponent(this.envParams["ui.routingServerGroup"]);
		}

		if( this.isBux )
		{
			var widget = this.getViewerWidget();
			var flashChartOptionValue = widget.getProperties().getFlashCharts(); //properties is in undo/redo stack, so we can get current setting.
			if( flashChartOptionValue != null )
			{
				oParams["savedFlashChartOption"] = flashChartOptionValue;
				if (oRequest.getParameter("hasAVSChart") !== null) {
					oParams.hasAVSChart = oRequest.getParameter("hasAVSChart");
				}
				else if (this.envParams.hasAVSChart) {
					oParams.hasAVSChart = this.envParams.hasAVSChart;
				}
				else {
					oParams.hasAVSChart = this.hasAVSChart();
				}
			}
			if (this.envParams["ui.reportDrop"] != null)
			{
				oParams["ui.reportDrop"] = this.envParams["ui.reportDrop"];
			}

			var cvGateway = widget.getAttributeValue("gateway");
			if(cvGateway) {
				oParams["cv.gateway"] = cvGateway;
			}
			var cvWebcontent = widget.getAttributeValue("webcontent");
			if(cvWebcontent) {
				oParams["cv.webcontent"] = cvWebcontent;
			}
		}
	}

	return oParams;
};

CCognosViewer.prototype.setLastActionFromRequest = function(oRequest)
{
	var sAction = oRequest.getAction();

	if((this.isBux && sAction != "wait") ||
	   (!this.isBux && (this["LAST_ACTION_PARAMS"] == "undefined" || this["LAST_ACTION_PARAMS"] == null || this["LAST_ACTION_PARAMS"] == "")))
	{
		this["LAST_ACTION_PARAMS"] = {"param1": oRequest};
	}

	this["LAST_ACTION"] = "sendRequest";
};

/**
	Sends a request to the server using an Ajax request.
	@param {CCognosViewerRequest} oRequest The request to send.
*/
CCognosViewer.prototype.sendRequest = function(oRequest)
{
	this.setLastActionFromRequest(oRequest);

	var sAction = oRequest.getAction();
	this.m_oServerRequest.setAction(sAction);

	var oParams = this.generateRequestParams(oRequest);
	for (var oParam in oParams)
	{
		this.m_oServerRequest.addArgument(oParam, oParams[oParam]);
	}
	this.m_oServerRequest.setRequestURL(this.getGateway());

	var callBack = oRequest.getCallback();
	if(callBack == null)
	{
		this.m_oServerRequest.setCallback(this);
	}
	else
	{
		this.m_oServerRequest.setCallback(callBack);
	}

	if(this.getUsePageRequest() === true)
	{
		this.setKeepSessionAlive(true);
		if(typeof document.progress != "undefined")
		{
			setTimeout("document.progress.src=\"" + this.getWebContentRoot() + "/skins/corporate/branding/progress.gif" + "\";", 1);
		}
	}
	else if(sAction != "wait")
	{
		// only fire the callback for non wait requests since we internally handle the wait
		this.executeCallback("ajaxRequestExecuted");
	}

	if( ( sAction === "forward" || sAction === "back" ) && typeof this.m_viewerFragment == "undefined" && !this.getViewerWidget())
	{
		this.createTransparentBackgroundLayer();
	}

	this.m_oServerRequest.execute();

	var id = this.getId();
	var waitTable = document.getElementById("CVWaitTable" + id);
	var modalDialog = document.getElementById("CMODAL_FRAME");
	if (!modalDialog && id == "_NS_" && sAction != "cancel" && (!waitTable || (this.m_waitPage && this.m_waitPage.m_waitPageDiv && this.m_waitPage.m_waitPageDiv.style.display == "none")))
	{
		this.m_waitPage.destroy();
		setTimeout("oCV"+id+".m_waitPage.showSimpleWaitDialog(oCV" + id + ")",10);
	}
};

/**
 *   Ignores mouse clicks in the background layer when request is running
 */
function CVBackgroundLayer_ignoreMouseClick( e )
{
	if (e.returnValue) { e.returnValue = false; }
	else if (e.preventDefault) { e.preventDefault(); }
	else { return false; }
}

/**
 * Creates a transparent background to be displayed when request is running so that mouse clicks can be ignored
 */
CCognosViewer.prototype.createTransparentBackgroundLayer = function()
{
	this.removeTransparentBackgroundLayer();

	var oBL = document.createElement( "div" );
	oBL.id = CV_BACKGROUND_LAYER_ID;
	oBL.style.display = "none";
	oBL.style.position = "absolute";

	oBL.style.top = "0px";
	oBL.style.left = "0px";
	oBL.style.zIndex = 98;

	oBL.style.width = "100%";
	oBL.style.height = "100%";

	oBL.style.backgroundColor = 'rgb(238, 238, 238)'; //arbitrary colour
	oBL.style.opacity = '0';
	oBL.style.filter = 'alpha(opacity:0)';

	oBL.innerHTML = '<table tabindex="1" width="100%" height="100%"><tr><td onclick="CVBackgroundLayer_ignoreMouseClick(event)"></td></tr></table>';
	oBL.style.display = 'inline';

	document.body.appendChild( oBL );
};

/**
 * Removes the transparent background layer
 */
CCognosViewer.prototype.removeTransparentBackgroundLayer = function()
{
	var oBL = document.getElementById( CV_BACKGROUND_LAYER_ID );
	if( oBL )
	{
		oBL.parentNode.removeChild( oBL );
	}
};

/**
 * Closes active http connections. Note this method will not issue a cancel. If you need to cancel an RSVP request, use the cancel method
 * @return (boolean)
 */
CCognosViewer.prototype.closeActiveHTTPConnection = function()
{
	return this.m_oServerRequest.cancel();
};

/**
 * Returns true if the viewer is in asynch mode, and the request is cancellable
 * @return (boolean)
 */
 CCognosViewer.prototype.canCancel = function()
 {
	var sTracking = this.getTracking();
	var sStatus = this.getStatus();

	return sTracking != "" && sStatus != "complete";
 };

/**
 * Cancels the current request, and restores the viewer back to the previous state.
 */
CCognosViewer.prototype.cancel = function(hrefLink)
{
	if (this.getWaitPageImpl().cancelSubmitted())
	{
		return false;
	}
	this.getWaitPageImpl().setCancelSubmitted(true);
	this.removeTransparentBackgroundLayer();

	this.m_oServerRequest.cancel();
		
	this.clearPrompts();

	if(this.canCancel() === true)
	{
		var cognosViewerRequest = new CCognosViewerRequest("cancel");

		var action = this.envParams["ui.action"];
		var oriPageRequest = this.getUsePageRequest();
		var oriStackSize = this.m_undoStack.length;

		if(this.m_undoStack.length > 0)
		{
			var cognosViewerUndo = this.m_undoStack.pop();

			this.m_sConversation = cognosViewerUndo.m_sConversation;
			this.m_sParameters = cognosViewerUndo.m_sParameters;
			this.m_envParams = cognosViewerUndo.envParams;
			this.m_sStatus = "complete";

			if(cognosViewerUndo.m_bRefreshPage == true)
			{
				var undoSpec = "<CognosViewerUndo><conversation>";
				undoSpec += this.m_sConversation;
				undoSpec += "</conversation></CognosViewerUndo>";
				cognosViewerRequest.addParameter("cv.previousSession", undoSpec);
			}
			else
			{
				this.m_undoStack.push(cognosViewerUndo);
				cognosViewerRequest.setCallback(function(){});
				this.m_waitPage.hide();
			}
		}
		else if (action == "view")
		{
			this.m_sStatus = "complete";
			cognosViewerRequest.setCallback(function(){});
			this.m_waitPage.hide();
			this.setUsePageRequest(false);
		}
		// if we have no stack but we have report history we'll end up doing executePreviousReport
		// so clear the callback so the cancel callback doesn't get called and do a backURL
		else if (oriStackSize <= 0 && this.rvMainWnd && this.rvMainWnd.getReportHistoryLength() > 0)
		{
			cognosViewerRequest.setCallback(function(){});
		}

		this.sendRequest(cognosViewerRequest);

		this.setTracking("");
		if (action == "view")
		{
			this.setUsePageRequest(oriPageRequest);
		}
		// If the stack size was 0, then the user hit cancel from the wait dialog or from the initial prompt page. Try and go
		// back to the previous report if there was one.
		else if (oriStackSize <= 0 && this.rvMainWnd)
		{
			this.rvMainWnd.executePreviousReport(-1);
		}

		return true;
	}
	else if (this.rvMainWnd && typeof this.envParams != "undefined" && (this.envParams["ui.primaryAction"] == "authoredDrillThrough" || this.envParams["ui.primaryAction"] == "authoredDrillThrough2"))
	{
		this.rvMainWnd.executePreviousReport(-1);
		return true;
	}
	else
	{
		if (!this.isBux) {
			executeBackURL(this.getId());
		}
		return true;
	}

	return false;
};

CCognosViewer.prototype.clearPrompts = function()
{
	if (this.preProcessControlArray)
	{
		var kCount = this.preProcessControlArray.length;
		var k = 0;
		for (k=0; k<kCount; k++)
		{
			var oPrmtCtrl = eval(this.preProcessControlArray[k]);
			if (oPrmtCtrl)
			{
				if (oPrmtCtrl.clearSubmit) {
					oPrmtCtrl.clearSubmit();
				}
			}
		}
	}
};

/**
 *	Sends a wait request to the report server. If the current status is not "working" or "stillWorking", the request will not be sent
 *  @return (boolean)
 */
CCognosViewer.prototype.wait = function()
{
	if(this.isWorking())
	{
		this.sendRequest(new CCognosViewerRequest('wait'));
		return true;
	}

	return false;
};

/**
	@param {string} sValue CAF encoded string.
*/
CCognosViewer.prototype.setCAFContext = function(sValue)
{
	this.m_sCAFContext = sValue;
};

/**
	@param {string} sXML XML for the context.
*/
CCognosViewer.prototype.setContextInfo = function(sXML)
{
	this.m_sContextInfoXML = sXML;
};

/**
	@param {string} sValue CAF encoded string.
*/
CCognosViewer.prototype.setConversation = function(sValue)
{
	this.m_sConversation = sValue;
};

/**
	@param {string} sValue CAF encoded string.
*/
CCognosViewer.prototype.setActionState = function(sValue)
{
	this.m_sActionState = sValue;
};

/**
 * Sets the conversation status
 * @param string (conversation status)
 */
CCognosViewer.prototype.setStatus = function(sStatus)
{
	this.m_sStatus = sStatus;
};

/**
	@param {bool} bDebug Enable/disable debugging in {@link CCognosViewer}.
	@private
*/
CCognosViewer.prototype.setDebug = function(bDebug)
{
	this.m_bDebug = bDebug;
};

/**
	@param {string} sValue CAF encoded string.
*/
CCognosViewer.prototype.setExecutionParameters = function(sValue)
{
	this.m_sParameters = sValue;
};

/**
	@param {string} sXML XML for the metadata.
*/
CCognosViewer.prototype.setMetadataInfo = function(sXML)
{
	this.m_sMetadataInfoXML = sXML;
};

/**
	@param {array} aValue
*/
CCognosViewer.prototype.setSecondaryRequests = function(aValue)
{
	if (aValue) {
		this.m_aSecRequests = aValue;
	}
	else {
		this.m_aSecRequests = [];
	}
};

/**
	@param {string} sValue
*/
CCognosViewer.prototype.setSessionPrompts = function(sValue)
{
	this.m_sSessionPrompts = sValue;
};

/**
	@param {string} sValue CAF encoded string.
*/
CCognosViewer.prototype.setTracking = function(sValue)
{
	this.m_sTracking = sValue;
};

/**
	@param (string) sValue fault code
 */
CCognosViewer.prototype.setSoapFault = function(sValue)
{
	this.m_sSoapFault = sValue;
};

CCognosViewer.prototype.showOutputInNewWindow = function(sURL)
{
	window.open(sURL, "", "");
	this.updateNewBrowserWindow();
};

CCognosViewer.prototype.showExcel = function(sURL)
{
	if( this.envParams["cv.excelWindowOpenProperties"] )
	{
		window.open(sURL, "", this.envParams["cv.excelWindowOpenProperties"]);
	}
	else
	{
		window.open(sURL, "", "");
	}

	this.updateNewBrowserWindow();
};

CCognosViewer.prototype.updateNewBrowserWindow = function()
{
	var id = this.getId();
	var doPostBack = document.forms["formWarpRequest" + id].elements["ui.postBack"];
	var backURL = document.forms["formWarpRequest" + id].elements["ui.backURL"];
	if (doPostBack && doPostBack.value)
	{
		setTimeout(getCognosViewerObjectRefAsString(id) + '.getRV().doPostBack();', 100);
	}
	else if (backURL && backURL.value)
	{
		if (backURL.value.length < 2048)
		{
			setTimeout('location.replace("' + backURL.value + '");', 100);
		}
		else
		{
			backURL = decodeURIComponent(backURL.value);
			var URLandParameters = backURL.split("?");
			var backURLForm = document.createElement("form");

			backURLForm.style.display = "none";
			backURLForm.setAttribute("method", "post");
			backURLForm.setAttribute("action", URLandParameters[0]);

			var parameterList = URLandParameters[1].split("&"); // must be ampersand symbol

			for(var nextParameter = 0; nextParameter < parameterList.length; nextParameter++)
			{
				// We cannot use "split" here using "=" because there are "=" within the parameters
				// that must be kept.
				var equalsIndexPos = parameterList[nextParameter].indexOf("=");
				var parameterName = parameterList[nextParameter].substr(0, equalsIndexPos);
				var parameterValue = parameterList[nextParameter].substr(equalsIndexPos + 1);

				var urlFormField = document.createElement("img");
				urlFormField.setAttribute("type", "hidden");
				urlFormField.setAttribute("name", decodeURIComponent(parameterName));
				urlFormField.setAttribute("value", decodeURIComponent(parameterValue));

				backURLForm.appendChild(urlFormField);
			}

			document.body.appendChild(backURLForm);
			backURLForm.submit();
		}
	}
	else
	{
		window.close();
	}
};

/**
	@private
	@param {CModal} oDlg A reference to a CModal
	@param {string} sPath path of the XTS dialog to run
*/
CCognosViewer.prototype.showModal = function(oDlg, sPath)
{
	var sParams = "b_action=xts.run";
	sParams += "&m=" + sPath;

	var oDispatcher = this.getDispatcher();
	var oDispReq = oDispatcher.createRequest(this.getGateway(), sParams, oDlg);
	oDispReq.setResponseType("HTTP");
	oDispatcher.dispatchRequest(oDispReq);
};

/**
	Used by prompt controls to submit (forward) requests.
	@private
	@param {string} sAction <code>cancel</code>, <code>back</code>, <code>next</code> or <code>finish</code>.
	@param {string} sUrl "Back url" to use after the prompt has been cancelled.
*/
CCognosViewer.prototype.promptAction = function(sAction, sUrl)
{
	// make sure the widget retains focus once the page is loaded
	this.setKeepFocus(true);

	if ( typeof datePickerObserverNotify == "function" )
	{
		datePickerObserverNotify();
	}

	var widget = this.getViewerWidget();

	if (sAction == "cancel")
	{
		this.cancelPrompt(sUrl);
		if (widget)
		{
			if (!this.isReportRenderingDone())
			{
				var payload = { action: 'deleteWidget' };
				widget.fireEvent("com.ibm.bux.widget.action", null, payload);
			}
		}
	}
	else
	{
		var oReq = new CCognosViewerRequest( sAction == "back" ? "back" : "forward" );

		if (sAction == "finish")
		{
			oReq.addOption("run.prompt", false);
		}
		else if (sAction == "back" || sAction == "next")
		{
			oReq.addOption("run.prompt", true);
		}
		if (sAction == "reprompt")
		{
			oReq.addOption("_promptControl", sAction);
		}
		else {
			oReq.addOption("_promptControl", "prompt");
		}

		if (widget) {
			widget.fireEvent("com.ibm.bux.widget.modified", null, {'modified':true});
		}

		this.submitPromptValues(oReq);
	}
};

/**
	@private
	@param {string} sUrl "Back url" to use after the prompt has been cancelled.
*/
CCognosViewer.prototype.cancelPrompt = function(sUrl)
{
	this.cancel();
};

/**
	Notifies other controls when a prompt control is updated
	@private
	@param {integer} iState
	@param {object} oNotifier
*/
CCognosViewer.prototype.notify = function(iState, oNotifier)
{
	var kCount = 0, k = 0;
	var oPromptElement = null;
	if (this.rangeObserverArray && this.rangeObserverArray instanceof Array)
	{
		kCount = this.rangeObserverArray.length;
		for (k=0; k<kCount; k++)
		{
			oPromptElement = eval(this.rangeObserverArray[k]);
			if (oPromptElement && typeof oPromptElement == "object" && typeof oPromptElement.update == "function")
			{
				oPromptElement.update();
			}
		}
	}

	var bPageEnabled = true;
	if (this.preProcessControlArray && this.preProcessControlArray instanceof Array)
	{
		kCount = this.preProcessControlArray.length;
		for (k=0; k<kCount; k++)
		{
			oPromptElement = eval(this.preProcessControlArray[k]);
			if ((typeof oPromptElement.getValid == "function") && !oPromptElement.getValid())
			{
				bPageEnabled = false;
				break;
			}
		}
	}
	this.notifyPageNavEnabled(bPageEnabled);

	if (this.multipleObserverArray && this.multipleObserverArray instanceof Array)
	{
		kCount = this.multipleObserverArray.length;
		for (k=0; k<kCount; k++)
		{
			oPromptElement = eval(this.multipleObserverArray[k]);
			if (oPromptElement && typeof oPromptElement == "object" && typeof oPromptElement.checkInsertRemove == "function")
			{
				oPromptElement.checkInsertRemove();
			}
		}
	}

	for (var idxNotif = 0; idxNotif < gaNotifyTargets.length; idxNotif++)
	{
		var oTarget = gaNotifyTargets[idxNotif];
		if (typeof oTarget != "undefined" && typeof oTarget.notify == "function")
		{
			oTarget.notify(iState, oNotifier);
		}
	}
};

/**
	@private
	@param {bool} bEnabled
*/
CCognosViewer.prototype.notifyPageNavEnabled = function(bEnabled)
{
	if (this.pageNavigationObserverArray && this.pageNavigationObserverArray instanceof Array)
	{
		var kCount = this.pageNavigationObserverArray.length;

		//determine if there is a finish button on the page, if so we'll disable the next.
		var bFinishPresent = false;
		var oPromptElement = null;
		var iPromptElementType = null;
		var k = 0;
		for (k=0; k<kCount; k++)
		{
			try
			{
				oPromptElement = eval(this.pageNavigationObserverArray[k]);
				iPromptElementType = oPromptElement.getType();
				if (iPromptElementType == PROMPTBUTTON_FINISH)
				{
					bFinishPresent = true;
					break;
				}
			}
			catch(e)
			{
			}
		}

		for (k=0; k<kCount; k++)
		{
			try
			{
				oPromptElement = eval(this.pageNavigationObserverArray[k]);
				iPromptElementType = oPromptElement.getType();
				if (!bEnabled)
				{
					if ((iPromptElementType == PROMPTBUTTON_NEXT) || (iPromptElementType == PROMPTBUTTON_OK) || (iPromptElementType == PROMPTBUTTON_FINISH))
					{
						oPromptElement.setEnabled(false);
					}
				}
				else
				{
					if (iPromptElementType == PROMPTBUTTON_FINISH)
					{
						oPromptElement.setEnabled(this.bCanFinish);
					}
					else if (iPromptElementType == PROMPTBUTTON_NEXT)
					{
						oPromptElement.setEnabled(this.bNextPage || !bFinishPresent);
					}
					else if (iPromptElementType == PROMPTBUTTON_OK)
					{
						oPromptElement.setEnabled(true);
					}
				}
			}
			catch(e2)
			{
			}
		}
	}
};

/**
	@private
	@param {object} oReq
*/
CCognosViewer.prototype.submitPromptValues = function(oReq)
{
	if ( this.gbPromptRequestSubmitted === true )
	{
		return false;
	}
	this.gbPromptRequestSubmitted = true;

	oReq = this.preparePromptValues(oReq);

	this.sendRequest(oReq);
};

CCognosViewer.prototype.preparePromptValues = function(oReq)
{
	// Use aInputs to keep track of input fields submitted.
	var aInputSubmitted = [];

	if (this.preProcessControlArray)
	{
		var kCount = this.preProcessControlArray.length;
		var k = 0;
		for (k=0; k<kCount; k++)
		{
			var oPrmtCtrl = eval(this.preProcessControlArray[k]);
			var bPrmtEnabled = (typeof oPrmtCtrl.isEnabled == "function" ? oPrmtCtrl.isEnabled() : true);
			if (oPrmtCtrl && typeof oPrmtCtrl.preProcess == "function" && bPrmtEnabled)
			{
				oPrmtCtrl.preProcess();
				if (oPrmtCtrl.m_oSubmit)
				{
					oReq.addParameter(oPrmtCtrl.m_oSubmit.name, oPrmtCtrl.m_oSubmit.value);
					aInputSubmitted.push(oPrmtCtrl.m_oSubmit);

					if (oPrmtCtrl.m_sPromptId && oPrmtCtrl.m_oForm && oPrmtCtrl.m_oForm.elements && typeof oPrmtCtrl.m_oForm.elements['p_' + oPrmtCtrl.m_sRef] == "object")
					{
						// we have a promptID based control, add its value separately (ie multiple search controls on the same parameter on the same page).
						oReq.addParameter('p_' + oPrmtCtrl.m_sPromptId, oPrmtCtrl.m_oForm.elements['p_' + oPrmtCtrl.m_sRef].value);
					}

				}
			}
		}
	}

	var elFWR = document.getElementById("formWarpRequest" + this.getId());
	if (elFWR)
	{
		var aInputs = elFWR.elements;
		for (var idxInput = 0; idxInput < aInputs.length; idxInput++)
		{
			var elInput = aInputs[idxInput];
			if ( !elInput.name || !elInput.name.match(/^p_/) )
			{
				continue;
			}
			var bToAdd = true;
			for (var idxSubmitted = 0; idxSubmitted < aInputSubmitted.length; idxSubmitted++)
			{
				if (aInputSubmitted[idxSubmitted] == elInput)
				{
					bToAdd = false; break;
				}
			}
			if (bToAdd)
			{
				oReq.addParameter(elInput.name, elInput.value);
				aInputSubmitted.push(elInput);
			}
		}
	}

	var oRM = this['CognosReport'];
	if (oRM)
	{
		var aParams = oRM.prompt.getParameters();
		for (var i = 0; i < aParams.length; i++)
		{
			var sName = "p_" + aParams[i].getName();
			if ( !oReq.getParameter(sName) )
			{
				oReq.addParameter(sName, aParams[i].getXML());
			}
		}
	}

	return oReq;
};

CCognosViewer.prototype.setViewerWidget = function( cvWidget )
{
	this.m_viewerWidget = cvWidget;
};
CCognosViewer.prototype.getViewerWidget = function()
{
	return this.m_viewerWidget;
};

CCognosViewer.prototype.getFlashChartOption = function()
{
	var cvWidget = this.getViewerWidget();
	var flashCharts = null;
	if (cvWidget) {
		var props = cvWidget.getProperties();
		if (props) {
			flashCharts = props.getFlashCharts();
		}
	}
	return flashCharts;
};

CCognosViewer.prototype.fireWidgetEvent = function( evt, payload )
{
	var viewerIWidget = this.getViewerWidget();
	if (viewerIWidget != null) {
		viewerIWidget.fireEvent(evt, null, payload);
	}
};

/**
 * Set the currently visible modal dialog
 * @param {Object} dialog
 */
CCognosViewer.prototype.setVisibleDialog = function(dialog) {
	this.m_visibleDialog = dialog;
};

/**
 * Get the visible modal dialog (if any)
 */
CCognosViewer.prototype.getVisibleDialog = function() {
	if (typeof this.m_visibleDialog != "undefined") {
		return this.m_visibleDialog;
	}

	return null;
};

CCognosViewer.prototype.getContentLocale = function() {
	var formWarpRequest = document.getElementById("formWarpRequest" + this.getId());
	 if (formWarpRequest && typeof formWarpRequest["ui.contentLocale"] != "undefined" && formWarpRequest["ui.contentLocale"] != null && formWarpRequest["reRunObj"].value.length > 0)
	 {
		 return formWarpRequest["ui.contentLocale"].value;
	 }
	 return null;
};



/**
	@constructor
	@param sAction {string} See {@link #setAction} method for possible values.
	@type CCognosViewerRequest
*/
function CCognosViewerRequest(sAction)
{
	/**
		@private
		@type string
	*/
	this.m_sAction = "";
	/**
		@private
		@type string
	*/
	this.m_sData = "";
	/**
		@private
		@type CDictionary
		@deprecated
	*/
	this.m_oOptions = new CDictionary();
	/**
		@private
		@type CDictionary
		@deprecated
	*/
	this.m_oParams = new CDictionary();

	/**
	 * @private
	 * @type CDictionary
	 */
	this.m_oFormFields = new CDictionary();

	/**
	 * @private
	 * @type string
	 */
	this.m_sRequestType = "ajax";

	/**
	 * The callback to be executed
	 * @private
	 */
	this.m_callback = null;

	this.setAction(sAction);


}

/**
 * Sets the callback function to be executed
 * @param callbackFunction (function)
 */
CCognosViewerRequest.prototype.setCallback = function(callback)
{
	this.m_callback = callback;
};

/**
 * Returns the callback function to be used
 * @return (function)
 */
CCognosViewerRequest.prototype.getCallback = function()
{
	return this.m_callback;
};

/**
 * Sets the request type (ajax,post,get)
 * @param string
 */
CCognosViewerRequest.prototype.setRequestType = function(sRequestType)
{
	if(typeof sRequestType != "undefined" && typeof sRequestType == "string")
	{
		if(sRequestType.match(/\ajax\b|\bpost\b|\bget\b/i))
		{
			this.m_sRequestType = sRequestType;
		}
	}
};

/**
 * Returns the request type
 * @return string
 */
CCognosViewerRequest.prototype.getRequestType = function()
{
	return this.m_sRequestType;
};

/**
	Add a option (name/value pair). See URL Api for possible options.
	@param {string} sName
	@param {string} sValue
	@deprecated
*/
CCognosViewerRequest.prototype.addOption = function(sName, sValue)
{
	this.m_oOptions.add(sName, sValue);
};

/**
	Remove an option (by name). See URL Api for possible options.
	@param {string} sName
	@param {string} sValue
	@deprecated
*/
CCognosViewerRequest.prototype.removeOption = function(sName)
{
	this.m_oOptions.remove(sName);
};

/**
	Add a parameter name/value pair to this request (for prompts).
	@param {string} sName
	@param {string} sValue
	@deprecated
*/
CCognosViewerRequest.prototype.addParameter = function(sName, sValue)
{
	this.m_oParams.add(sName, sValue);
};

/**
 * Adds a form field to the request
 * @param (string) sName
 * @param (string) sValue
 */
 CCognosViewerRequest.prototype.addFormField = function(sName, sValue)
 {
	this.m_oFormFields.add(sName, sValue);
 };

 /**
  * Returns the list of form fields associated with the request
  * @return CDictionary
  */
 CCognosViewerRequest.prototype.getFormFields = function()
 {
	return this.m_oFormFields;
 };

/**
	@type string
*/
CCognosViewerRequest.prototype.getAction = function()
{
	return this.m_sAction;
};

/**
	@type string
*/
CCognosViewerRequest.prototype.getData = function()
{
	return this.m_sData;
};

/**
	@param {string} sName
	@type string
	@deprecated
*/
CCognosViewerRequest.prototype.getOption = function(sName)
{
	return this.m_oOptions.get(sName);
};

/**
	@param {string} sName
	@type string
	@deprecated
*/
CCognosViewerRequest.prototype.getParameter = function(sName)
{
	return this.m_oParams.get(sName);
};

/**
	@param {string} sName
	@type bool
	@deprecated
*/
CCognosViewerRequest.prototype.hasOption = function(sName)
{
	return this.m_oOptions.exists(sName);
};

/**
	@param {string} sName
	@type bool
	@deprecated
*/
CCognosViewerRequest.prototype.hasParameter = function(sName)
{
	return this.m_oParams.exists(sName);
};

/**
	@param sAction {string} possible values: <ul><li>add</li><li>back</li><li>cancel</li><li>currentPage</li><li>drill</li><li>email</li><li>firstPage</li><li>forward</li><li>getOutput</li><li>lastPage</li><li>nextPage</li><li>previousPage</li><li>print</li><li>release</li><li>render</li><li>run</li><li>runSpecification</li><li>save</li><li>saveas</li><li>wait</li></ul>
*/
CCognosViewerRequest.prototype.setAction = function(sAction)
{
	this.m_sAction = sAction;
};

/**
	Set the data for the current action (see URL Api).
	@param {string} sValue
*/
CCognosViewerRequest.prototype.setData = function(sValue)
{
	this.m_sData = sValue;
};

/**
	Show/hide header, toolbar based on the current state.
	@private
	@param {string} sState Conversation's state
*/
CCognosViewer.prototype.updateLayout = function(sState)
{
	var cvid = this.getId();
	var oHeader = document.getElementById('CVHeader' + cvid);
	var oToolbar = document.getElementById('CVToolbar' + cvid);
	if (!oHeader && !oToolbar)
	{
		setTimeout(getCognosViewerObjectRefAsString(cvid) + '.updateLayout("' + sState + '");', 100);
		return;
	}

	if (oHeader) {
		if (sState == "prompting" && !this.bShowHeaderWithPrompts) {
			oHeader.parentNode.style.display = "none";
		}
		else {
			oHeader.parentNode.style.display = "";
		}
	}
	if (oToolbar) {
		if (sState == "prompting") {
			oToolbar.parentNode.style.display = "none";
		}
		else {
			oToolbar.parentNode.style.display = "";
		}
	}
};

/**
	Update the state of the response specification.
	@private
	@param {string} sResponseSpecification specification state
*/
CCognosViewer.prototype.updateResponseSpecification = function(sResponseSpecification)
{
	this.sResponseSpecification = sResponseSpecification;
};

/**
	Returns the response specification if available.
	@private
	@param
*/
CCognosViewer.prototype.getResponseSpecification = function()
{
	return this.sResponseSpecification;
};

/**
 * Releases the currect conversation. Returns true if release was called, otherwise false
 * @param {Object} callback
 */
CCognosViewer.prototype.release = function(callback)
{
	var form = document.getElementById("formWarpRequest" + this.getId());
	var tracking = this.getTracking();
	if(!tracking && form && form["m_tracking"] && form["m_tracking"].value) {
		tracking = form["m_tracking"].value;
		form["m_tracking"].value = "";
	}

	// blank out the tracking so we never do multiple releases for the same tracking
	this.setTracking("");

	if(tracking && tracking != "")
	{
		var params="b_action=cognosViewer&cv.responseFormat=successfulRequest";


		if(this.envParams["cv.debugDirectory"] != null)
		{
			params += "&cv.debugDirectory=" + this.envParams["cv.debugDirectory"];
		}

		if(this.envParams["ui.primaryAction"] != null)
		{
			params += "&ui.primaryAction=" + this.envParams["ui.primaryAction"];
		}

		if(this.envParams["ui.objectClass"] != null)
		{
			params += "&ui.objectClass=" + this.envParams["ui.objectClass"];
		}

		if( this.isWorkingOrPrompting() ) {
			params += "&ui.action=cancel";
		}
		else {
			params += "&ui.action=release";
		}

		params += "&m_tracking=";
		params += tracking;

		if(!callback)
		{
			// If no callback was provided, this http request needs to be synchronous. The dispatcher doesn't
			// currently support synchronous requests, so for the time being, we'll create our own http request and send
			// it along
			var httpRequest;
			if (typeof ActiveXObject != "undefined")
			{
				httpRequest = new ActiveXObject("Msxml2.XMLHTTP");
			}
			else
			{
				httpRequest = new XMLHttpRequest();
			}

			httpRequest.onreadystatechange = XMLHttpRequestReadyStateChanged;
			httpRequest.open("POST", this.getGateway(), false);
			httpRequest.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
			httpRequest.send(params);
			if (httpRequest.responseXML != null)
			{
				var responseXML = httpRequest.responseXML;
				if (typeof g_firefoxPluginHelper != "undefined" && g_firefoxPluginHelper != null)
				{
					g_firefoxPluginHelper.addDebugLogsFromResponseXML(responseXML);
				}
			}
		}
		else
		{
			var request = gDispatcher.createRequest(this.getGateway(), params, callback);
			gDispatcher.dispatchRequest(request);
		}

		return true;
	}

	return false;
};

/**
 * Will release the conversation and clean up the Viewer object
 * @param {Object} callback
 */
CCognosViewer.prototype.destroy = function(callback) {
	this.release(callback);
	if (!this.m_destroyed)
	{
		//cleanup action
		if( typeof window.gaRV_INSTANCES != "undefined") {
			for (var iIndex=0; iIndex < window.gaRV_INSTANCES.length; iIndex++)
			{
				if (window.gaRV_INSTANCES[iIndex].m_sId == this.getId())
				{
					window.gaRV_INSTANCES.splice(iIndex, 1);
					this.m_destroyed = true;
					break;
				}
			}
		}

		cleanupGlobalObjects(this.getId());
	}
};

/**
 * Releases the conversation
 * @param {Object} callback
 */
CCognosViewer.prototype.exit = function(callback)
{
	this.release(callback);
};

/**
 * Executes a viewer action
 * @param (string) The action to execute (ie. run)
 */
CCognosViewer.prototype.executeAction = function(sAction, parms )
{
	var action = eval("new " + sAction + "Action();");
	action.setCognosViewer(this);
	action.setRequestParms( parms );
	action.execute();
};

/**
 * @private
 */
CCognosViewer.prototype.buildRefreshRequest = function()
{
	var request = null;

	if(typeof this.envParams["ui.spec"] != "undefined")
	{
		request = new CCognosViewerRequest("runSpecification");
		request.addParameter("ui.spec", this.envParams["ui.spec"]);
		request.addParameter("reuseResults", "true");
		request.addParameter("keepIterators", "true");
	}

	return request;
};

CCognosViewer.prototype.refresh = function()
{
	var request = this.buildRefreshRequest();

	if(request != null)
	{
		this.sendRequest(request);
	}
};


/**
 * Gets a calculation object
 */
CCognosViewer.prototype.getCalculation = function(sCalculation){
	var calc = null;
	var calcCache = this.getCalculationCache();
	if (calcCache[sCalculation]) {
		calc = calcCache[sCalculation];
	} else {
		calc = eval("new " + sCalculation + "();");
		calc.setCognosViewer(this);
		calcCache[sCalculation] = calc;
	}

	return calc;
};

CCognosViewer.prototype.findBlueDotMenu = function()
{
	var root = null;
	var toolbarPayload = this.buxToolbarPayload;

	for(var toolbarItem = 0; toolbarItem < toolbarPayload.length; ++toolbarItem) {
		if (typeof toolbarPayload[toolbarItem]._root != "undefined") {
			root = toolbarPayload[toolbarItem]._root;
			break;
		}
	}

	return root;
};

/**
 * Finds an item in the toolbar. If toolbarSpec is null, it'll search through
 * the top level buttons (i.e. Not the blue dot menu)
 */
CCognosViewer.prototype.findToolbarItem = function(itemName, toolbarSpec)
{
	var spec = typeof toolbarSpec == "undefined" || toolbarSpec == null ? this.buxToolbarPayload : toolbarSpec;
	var buttonSpec = null;

	for(var iIndex = 0; iIndex < spec.length; ++iIndex) {
		var name = spec[iIndex]["name"];
		if (typeof name != "undefined" && name == itemName) {
			buttonSpec = spec[iIndex];
			break;
		}
	}

	return buttonSpec;
};

CCognosViewer.prototype.addDrillTargets = function(drillTargets)
{
	this.m_drillTargets = drillTargets;
};

CCognosViewer.prototype.getDrillTargets = function()
{
	return this.m_drillTargets;
};

CCognosViewer.prototype.getDrillTarget = function(idx)
{
	if(idx >= this.m_drillTargets.length)
	{
		return null;
	}
	return this.m_drillTargets[idx];
};

CCognosViewer.prototype.getNumberOfDrillTargets = function()
{
	return this.m_drillTargets.length;
};

CCognosViewer.prototype.isReportRenderingDone = function()
{
	return this.m_reportRenderingDone;
};

CCognosViewer.prototype.setReportRenderingDone = function(flag)
{
	this.m_reportRenderingDone = flag;
};


CCognosViewer.prototype.hasAVSChart = function(reportInfo)
{

	if (typeof reportInfo == "undefined")
	{
		reportInfo = this.envParams["rap.reportInfo"];
	}
	if (typeof reportInfo == "undefined")
	{
		return false;
	}
	for(var index = 0; index < reportInfo.length; ++index)
	{
		var displayTypeId = reportInfo[index].displayTypeId;
		if (typeof displayTypeId != "undefined" && (displayTypeId.match("_v2") != null || displayTypeId.match("v2_") != null))
		{
			return true;
		}
	}
	return false;

};

/**
 * Returns the warp request form used by this viewer.
 */
CCognosViewer.prototype.getWarpRequestForm = function() {
	return document.getElementById("formWarpRequest" + this.getId());
};

/**
 * Common class between the Viewer and IWidget wait pages
 */
function CCommonWaitPage()
{
	this.m_bUse = true;
	this.m_bCancelSubmitted = false;
	this.m_oCV = null;
}

CCommonWaitPage.prototype.setCognosViewer = function(oCV)
{
	this.m_oCV = oCV;
};

CCommonWaitPage.prototype.getCognosViewer = function()
{
	return this.m_oCV;
};

CCommonWaitPage.prototype.cancelSubmitted = function()
{
	return this.m_bCancelSubmitted;
};

CCommonWaitPage.prototype.setCancelSubmitted = function(bSubmitted)
{
	this.m_bCancelSubmitted = bSubmitted;
};

CCommonWaitPage.prototype.setUseWaitPage = function(bUse)
{
	this.m_bUse = bUse;
};

CCommonWaitPage.prototype.getUseWaitPage = function()
{
	return this.m_bUse;
};

/**
 * Returns the unique namespace
 * @return (string)
 */
CCommonWaitPage.prototype.getNamespace = function()
{
	if (typeof this.m_sNamespace != "string" && this.getCognosViewer() != null)
	{
		this.m_sNamespace = this.getCognosViewer().getId();
	}

	return this.m_sNamespace;
};

/**
	Initialize the CWaitPage object.
	@constructor
	@param {string} sId The unique namespace to be used by this object
	@param {string} sGateway Path to "cognos.cgi". For example: "/cognos8/cgi-bin/cognos.cgi".
 */

function CWaitPage(sNamespace, sGateway)
{
	this.m_sNamespace = sNamespace;
	this.m_sGateway = sGateway;
	this.m_waitPageDiv = null;
	this.m_waitPageIframe = null;
	this.m_sHTML = "";
}
CWaitPage.prototype = new CCommonWaitPage();

/**
 * Sets the wait page HTML
 * @private
 */
CWaitPage.prototype.initializeWaitPage = function(sHTML)
{
	this.m_sHTML = sHTML;
	this.setCancelSubmitted(false);
};

/**
 * Returns the gateway
 * @return (string)
 */
CWaitPage.prototype.getGateway = function()
{
	return this.m_sGateway;
};

CWaitPage.prototype.destroy = function()
{
	if (this.m_waitPageDiv != null)
	{
		this.m_waitPageDiv.parentNode.removeChild(this.m_waitPageDiv);
		this.m_waitPageDiv = null;
	}

	if (this.m_waitPageIframe != null)
	{
		this.m_waitPageIframe.parentNode.removeChild(this.m_waitPageIframe);
		this.m_waitPageIframe = null;
	}
};

/**
 * Returns a boolean value indicating whether or not the wait page has been initialized
 * @return (boolean)
 */
CWaitPage.prototype.isInitialized = function()
{
	return (this.m_waitPageDiv != null && this.m_waitPageIframe != null);
};

/**
 * Will initialize and draw the wait page. This method should be used to render the page
 * @param bSavedReport (boolean) Indicates running a saved report. This will dictate whether the wait page shows "delivery options" or not.
 */
CWaitPage.prototype.show = function(bSavedReport)
{
	if(this.getUseWaitPage() === true && this.m_sHTML != "")
	{
		this.preShow();

		var waitTable = document.getElementById("CVWaitTable" + this.getNamespace());
		if((typeof waitTable == "undefined" || waitTable == null) &&(this.isInitialized() === false))
		{
			this.create(bSavedReport);
		}

		this.render(true, bSavedReport);
	}
};

/**
 * override this method for special processing
 */
CWaitPage.prototype.preShow = function() {};

/**
 * Will hide the wait page
 */
CWaitPage.prototype.hide = function()
{
	this.preHide();
	this.render(false);
	this.showDeliveryOptions(false);
};

/**
 * override this method for special processing
 */
CWaitPage.prototype.preHide = function() {};

/**
 * Displays a very simply wait dialog with a wait icon and some text.
 * @param oCV the cognos viewer object
 */
CWaitPage.prototype.showSimpleWaitDialog = function(oCV)
{
	this.initializeWaitPage(this.getSimpleWaitDialogHtml(oCV));
	this.show(false);
};

/**
 * returns the html for a very simply wait dialog with a wait icon and some text.
 * @param oCV the cognos viewer object
 */
CWaitPage.prototype.getSimpleWaitDialogHtml = function(oCV)
{
	var waitPageHtml = "<table id=\"CVWaitTable" + oCV.getId() + "\" simpleDialog=\"true\" style=\"border: 1px solid #000000;\" cellpadding=\"0\" cellspacing=\"0\">";
	waitPageHtml += "<tr><td align=\"center\">";
	waitPageHtml += "<div class=\"body_dialog_modal\" style=\"border:1px solid #999999;\">";
	waitPageHtml += "<table align=\"center\" cellspacing=\"0\" cellpadding=\"0\" style=\"vertical-align:middle;;\">";
	waitPageHtml += "<tr><td rowspan=\"2\">";
	waitPageHtml += "<img alt=\"" + RV_RES.GOTO_WORKING + "\" src=\"" + oCV.getWebContentRoot() + "/skins/corporate/branding/progress.gif\" style=\"margin:5px;\" width=\"48\" height=\"48\" name=\"progress\"/>";
	waitPageHtml += "</td><td nowrap=\"nowrap\"><span class=\"busyUpdatingStr\">";
	waitPageHtml += RV_RES.GOTO_WORKING;
	waitPageHtml += "</span></td></tr><tr><td nowrap=\"nowrap\"><span class=\"busyUpdatingStr\">";
	waitPageHtml += RV_RES.RV_PLEASE_WAIT;
	waitPageHtml += "</span></td></tr><tr><td style=\"height:7px;\" colspan=\"2\"></td></tr></table></div></td></tr></table>";

	return waitPageHtml;
};

/**
 * Will hide/show the delivery options toolbar buttons within the wait page
 * @param bShow (boolean) Boolean which dictates whether to show the delivery option toolbar buttons
 */
CWaitPage.prototype.showDeliveryOptions = function(bShow)
{
	var namespace = this.getNamespace();
	var htmlElement = document.getElementById("DeliveryOptionsVisible" + namespace);
	if (htmlElement)
	{
		htmlElement.style.display = (bShow === false ? "none" : "block");
		if (bShow)
		{
			var links = htmlElement.getElementsByTagName("a");
			for (var i=links.length; i > 0; i--)
			{
				if (links[i] && links[i].getAttribute("tabIndex") == "0") {
					links[i].focus();
				}
			}
		}
	}

	htmlElement = document.getElementById("OptionsLinkSelected" + namespace);
	if (htmlElement)
	{
		htmlElement.style.display = (bShow === false ? "none" : "block");
	}

	htmlElement = document.getElementById("OptionsLinkUnselected" + namespace);
	if (htmlElement)
	{
		htmlElement.style.display = (bShow === false ? "block" : "none");
	}

	this.updateCoords();
};

/**
 * Initializes the wait page div and hidden iframe
 * @param bSavedReport (boolean) Indicates whether or not the wait page should make available the delivery options
 * @private
 */
CWaitPage.prototype.create = function(bSavedReport)
{
	if (typeof document.body != "undefined")
	{
		this.m_waitPageDiv = this.createWaitPageDiv(bSavedReport);
		this.m_waitPageIframe = this.createWaitPageIframe(this.m_waitPageDiv);
	}
};

/**
 * Updates the x and y coordinates of the wait page
 * @private
 */
CWaitPage.prototype.updateCoords = function()
{
	if(this.isInitialized())
	{
		// position the wait page in the middle of the report
		var iBottom = 0;
		var iLeft = 0;
		if (typeof window.innerHeight != "undefined")
		{
			iBottom = Math.round((window.innerHeight/2) - (this.m_waitPageDiv.offsetHeight/2));
			iLeft = Math.round((window.innerWidth/2) - (this.m_waitPageDiv.offsetWidth/2));
		}
		else
		{
			iBottom = Math.round((document.body.clientHeight/2) - (this.m_waitPageDiv.offsetHeight/2));
			iLeft = Math.round((document.body.clientWidth/2) - (this.m_waitPageDiv.offsetWidth/2));
		}

		this.m_waitPageDiv.style.bottom = iBottom + "px";
		this.m_waitPageDiv.style.left = iLeft + "px";

		this.m_waitPageIframe.style.left = this.m_waitPageDiv.style.left;
		this.m_waitPageIframe.style.bottom = this.m_waitPageDiv.style.bottom;
		this.m_waitPageIframe.style.width = this.m_waitPageDiv.offsetWidth + "px";
		this.m_waitPageIframe.style.height = this.m_waitPageDiv.offsetHeight + "px";
	}
};

/**
 * Builds the wait page div
 * @param bSavedReport (boolean) Indicates whether or not the wait page should render the delivery options
 * @private
 */
CWaitPage.prototype.createWaitPageDiv = function(bSavedReport)
{
	var oWaitDiv = document.createElement("DIV");
	oWaitDiv.setAttribute("id", "CVWait" + this.getNamespace());

	oWaitDiv.style.position = "absolute";
	oWaitDiv.style.textAlign = "center";
	oWaitDiv.style.zIndex = 100;

	this.appendWaitPageObj(oWaitDiv);

	try
	{
		oWaitDiv.innerHTML = this.m_sHTML;
	}
	catch (oExcpt) {}

	return oWaitDiv;
};

/**
 * Builds the wait page hidden iframe (used so the wait page draws over iframes)
 * @param bSavedReport (boolean) Indicates whether or not the wait page should render the delivery options
 * @private
 */
CWaitPage.prototype.createWaitPageIframe = function(waitPageDiv)
{
	var hiddenIframe = document.createElement("iframe");

	var sWebContentRoot = "..";

	var oCV = this.getCognosViewer();
	if (oCV != null)
	{
		sWebContentRoot = oCV.getWebContentRoot();
	}

	hiddenIframe.setAttribute("id","CVWaitIframe" + this.getNamespace());
	hiddenIframe.setAttribute("src",sWebContentRoot + '/common/images/spacer.gif');
	hiddenIframe.setAttribute("scrolling",'no');
	hiddenIframe.setAttribute("frameborder",'0');
	hiddenIframe.style.position="absolute";
	hiddenIframe.style.zIndex=99;
	hiddenIframe.style.display="none";

	this.appendWaitPageObj(hiddenIframe);

	return hiddenIframe;
};

CWaitPage.prototype.appendWaitPageObj = function(obj)
{
	document.body.appendChild(obj);
};

/**
 * Displays or hides the wait page
 * @param bShow (boolean) Indicates whether to hide or show the wait page
 * @private
 */
CWaitPage.prototype.render = function(bShow)
{
	if(this.m_waitPageDiv != null && this.m_waitPageIframe != null)
	{
		var sDisplay = bShow ? "block" : "none";

		this.m_waitPageDiv.style.display = sDisplay;
		this.m_waitPageIframe.style.display = sDisplay;

		this.updateCoords();
	}
};

/**
	Initialize the CDocumentWriter object.
	@constructor
	@param {string} sId The unqiue id of the anchor tag which is a placholder for the script to execute
	@param {string} sScript The script to be executed
 */
function CDocumentWriter(sId, sScript)
{
	this.m_sId = sId;
	this.m_sText = "";
	this.m_sScript = sScript;
}

/**
 * Determines if the document writer can handle the embedded script successfully.
 * @return (boolean)
 */
CDocumentWriter.prototype.isValid = function()
{
	if(typeof this.m_sScript != "undefined" && this.m_sScript && window.gScriptLoader)
	{
		//Previously, there was a check that m_sScript contained <script> and </script>
		//tags, but now that the code is extracted directly from the DOM, tags are not
		//included in the script string.
		return true;
	}

	return false;
};

/**
 * Executes the inline script. This method will return false if an error occurs
 * @return (boolean)
 */
CDocumentWriter.prototype.execute = function()
{

	if(this.isValid() && window.gScriptLoader)
	{
		var reDocumentWrite = /document\.write(ln)?\s*\(/gi;
		var sScript = this.m_sScript.replace(reDocumentWrite, "this.write(").replace(window.gScriptLoader.m_reScriptTagOpen, "").replace(window.gScriptLoader.m_reScriptTagClose, "");

		try
		{
			eval(sScript);
			var placeHolderNode = document.getElementById(this.m_sId);

			if(placeHolderNode)
			{
				placeHolderNode.innerHTML = this.m_sText;
				return true;
			}
		}
		catch(e){}
	}

	return false;
};

/**
 * Builds a string and caches it locally to be inserted later on during execution
 */
CDocumentWriter.prototype.write = function(oArgument)
{
	var sResult = "";
	if(typeof oArgument == "function")
	{
		sResult = eval(oArgument);
	}
	else if(typeof oArgument == "string")
	{
		sResult = oArgument;
	}

	this.m_sText += sResult;
};

function setFocusToFirstTabItem(container) {
	var firstElem = dojo.query("*[tabindex='0']", container)[0];
	if (firstElem) {
		dijit.focus(firstElem);
	}
}
