/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

dojo.provide("ViewerIWidgetInlineDialog");

dojo.require("dijit._base.focus");

/**
 * Class that will take care of showing an inline dialog centered in the widget.
 * @param {Object} oCV
 * @param {Object} sHTML
 */
dojo.declare("ViewerIWidgetInlineDialog", null, {
	constructor: function(oCV, sHTML) {
		this.m_oCV = oCV;
		this.m_html = sHTML;

		this.m_viewerDiv = dojo.byId("RVContent" + this.m_oCV.getId());
		this.m_containerDiv = this.findContainerDiv();	// this is 'small' div from the canvas which has the scrollbars
		this.positionOptions = {horizontal:"center", vertical:"center"};

		this.onScrollHandle = null;
		this.m_dialogDiv = null;
		this.m_background = null;
		this.m_backgroundClassOverride = null;
	},

	/**
	 * Sets the vertical option for the dialog. Currently only supports top and middle
	 * @param {Object} option
	 */
	setVerticalPositionOption: function(option) {
		this.positionOptions.vertical = option;
	},

	setBackgroundClassOverride: function(classname) {
		this.m_backgroundClassOverride = classname;
		this.applyBackgroundClassOverride();
	},

	applyBackgroundClassOverride: function() {

		var backgroundOverride = dojo.byId("BACKGROUND_OVERRIDE" + this.m_oCV.getId());
		if (backgroundOverride != null && this.m_backgroundClassOverride != null) {
			dojo.addClass(backgroundOverride, this.m_backgroundClassOverride);
		}
	},


	getDialogDiv: function() {
		return this.m_dialogDiv;
	},

	initilizeDialog: function() {
		this.m_dialogDiv = document.createElement("div");
		this.m_dialogDiv.style.position = "absolute";
		this.m_dialogDiv.style.zIndex = "100";
		this.m_dialogDiv.style.top = "0";
		this.m_dialogDiv.style.left = "0";
		this.m_dialogDiv.style.display = "none";
		this.m_dialogDiv.style.backgroundColor = "#FFFFFF";
		this.m_dialogDiv.innerHTML = this.m_html;

		dojo.addClass(this.m_dialogDiv, 'inlineDialog');

		this.m_containerDiv.insertBefore(this.m_dialogDiv, this.m_background.nextSibling);
	},

	initilizeBackground: function() {
		// we only ever have to create one background div for all our dialogs, so lets
		// see if it's already created
		if (this.m_background == null) {
			this.m_background = dojo.byId("BACKGROUND" + this.m_oCV.getId());

			if (!this.m_background) {
				this.m_background = document.createElement("div");
				this.m_background.id = "BACKGROUND" + this.m_oCV.getId();
				this.m_background.style.display = "none";
				this.m_background.style.position = "absolute";
				this.m_background.style.zIndex = "99";
				this.m_background.style.left = "0";
				this.m_background.style.top = "0";
				this.m_background.style.background = "#FFFFFF none repeat scroll 0 0";
				this.m_background.style.opacity = '0.90';
				this.m_background.style.filter = 'alpha(opacity:90)';
				this.m_background.tabIndex = "0";
				this.m_background.innerHTML = '<table width="100%" height="100%" id="BACKGROUND_OVERRIDE' + this.m_oCV.getId() +'"><tr><td onclick="CVBackgroundLayer_ignoreMouseClick(event)"></td></tr></table>';

				this.m_containerDiv.insertBefore(this.m_background, this.m_containerDiv.firstChild);
				this.applyBackgroundClassOverride();
			}
		}

		// we need a div with a tabIndex of zero as the first child of the Viewer div. This will catch the user
		// hitting F12 on a widget and place the focus on the first focusable item in the dialog.
		if (this.m_tabCatch == null) {
			this.m_tabCatch = dojo.byId("BACKGROUND_TABCATCH" + this.m_oCV.getId());

			if (!this.m_tabCatch) {
				this.m_tabCatch = document.createElement("div");
				this.m_tabCatch.id = "BACKGROUND_TABCATCH" + this.m_oCV.getId();
				this.m_tabCatch.style.display = "none";
				this.m_tabCatch.style.position = "absolute";
				this.m_tabCatch.style.left = "0";
				this.m_tabCatch.style.top = "0";
				this.m_tabCatch.style.height = "0px";
				this.m_tabCatch.style.width = "0px";
				this.m_viewerDiv.insertBefore(this.m_tabCatch, this.m_viewerDiv.firstChild);
			}
		}
	},

	initializeKeyboardSupport: function() {
		// setup the keyboard support so we keep looping through
		// the open dialog when the user hits TAB
		var tabElems = dijit._getTabNavigable(this.m_dialogDiv);
		var firstElem = tabElems.lowest || tabElems.first;
		var lastElem = tabElems.last || tabElems.highest || firstElem;

		if (firstElem) {
			this.m_tabCatch.onfocus = function() {dijit.focus(firstElem);};
			this.m_background.onfocus = function() {dijit.focus(firstElem);};
		}

		if (lastElem && firstElem) {
			lastElem.onblur = function() {dijit.focus(firstElem);};
		} else if (firstElem) {
			firstElem.onblur = function() {dijit.focus(firstElem);};
		}
	},

	show: function() {
		this.m_oCV.setVisibleDialog(this);

		if (this.m_background == null) {
			this.initilizeBackground();
		}

		if (this.m_dialogDiv == null) {
			this.initilizeDialog();
		}

		// set the original height and width to zero or it might change
		// the size of the div when we display it
		this.m_background.style.width = "0px";
		this.m_background.style.height = "0px";

		this.m_background.style.display = "block";
		this.m_tabCatch.style.display = "block";
		this.m_dialogDiv.style.display = "block";

		this.onScrollHandle = dojo.connect(this.m_containerDiv, "onscroll", dojo.hitch(this, this.updateCoords));
		this.resize();
		this.updateCoords();
		this.initializeKeyboardSupport();
	},

	hide: function() {
		this.m_oCV.setVisibleDialog(null);

		if (this.onScrollHandle != null) {
			dojo.disconnect(this.onScrollHandle);
			this.onScrollHandle = null;
		}

		if (this.m_background != null) {
			this.m_background.style.display = "none";
		}
		if (this.m_tabCatch != null) {
			this.m_tabCatch.style.display = "none";
		}
		if (this.m_dialogDiv != null) {
			this.m_dialogDiv.style.display = "none";
		}
	},

	destroy: function() {
		this.hide();

		if (this.m_dialogDiv != null) {
			this.m_dialogDiv.parentNode.removeChild(this.m_dialogDiv);
			this.m_dialogDiv = null;
		}
		if (this.m_tabCatch != null) {
			this.m_tabCatch.parentNode.removeChild(this.m_tabCatch);
			this.m_tabCatch = null;
		}
		if (this.m_background != null) {
			this.m_background.parentNode.removeChild(this.m_background);
			this.m_background = null;
		}
	},

	findContainerDiv: function() {
		var containerDiv = null;

		// we need to find the correct container div. It'll be the one with a height set in pixels
		if (this.m_containerDiv == null) {
			containerDiv = this.m_viewerDiv;
			while (containerDiv) {
				if (containerDiv.style && containerDiv.style.height && containerDiv.style.height.indexOf("px") != -1) {
					return containerDiv;
				}
				containerDiv = containerDiv.parentNode;
			}
		}

		if (typeof console != "undefined" && console && console.log) {
			console.log("ViewerIWidgetInlineDialog: Could not find the container div for showing the dialog.");
		}

		return this.m_viewerDiv;
	},

	widgetResizeEvent: function() {
		this.resize();
		this.updateCoords();
	},

	resize: function() {
		// set the background size to 0 so it doesn't affect the size of the div when we try and get the information
		this.m_background.style.width = "0px";
		this.m_background.style.height = "0px";

		this.m_containerWidth = this.m_containerDiv.scrollWidth > 0 ?  this.m_containerDiv.scrollWidth : this.m_containerDiv.style.width;
		this.m_background.style.width = this.m_containerWidth + "px";

		this.m_containerHeight = this.m_containerDiv.scrollHeight > 0 ? this.m_containerDiv.scrollHeight : this.m_containerDiv.style.height;
		this.m_background.style.height = this.m_containerHeight + "px";
	},

	updateCoords: function() {
		if (this.m_containerDiv != null) {
			var scrollPosition = {"x":0,"y": 0};
			var dojoCoords = dojo.coords(this.m_containerDiv);
			if (typeof this.m_containerDiv.pageYOffset != "undefined" && (this.m_containerDiv.pageYOffset > 0 || this.m_containerDiv.pageXOffset > 0)) {
				scrollPosition = {"x":this.m_containerDiv.pageXOffset,"y":this.m_containerDiv.pageYOffset};
			}
			else if (typeof this.m_containerDiv.scrollTop != "undefined" && (this.m_containerDiv.scrollTop > 0 || this.m_containerDiv.scrollLeft > 0) ) {
				scrollPosition = {"x":this.m_containerDiv.scrollLeft,"y":this.m_containerDiv.scrollTop};
			}

			var newTop;
			if (this.positionOptions.vertical == "top") {
				newTop = scrollPosition.y;
			} else {
				newTop = Math.round(dojoCoords.h/2) - (this.m_dialogDiv.offsetHeight/2) + scrollPosition.y;
			}

			// weird bug in IE where if we push the dialog past the widget edge, the scrollbar will scroll forever.
			if ( (newTop + this.m_dialogDiv.offsetHeight) < this.m_containerHeight) {
				this.m_dialogDiv.style.top = newTop + "px";
			}

			var newLeft = Math.round(dojoCoords.w/2) - (this.m_dialogDiv.offsetWidth/2) + scrollPosition.x;
			if (newLeft < 0) {
				newLeft = 0;
			}
			if ( (newLeft + this.m_dialogDiv.offsetWidth) < this.m_containerWidth) {
				this.m_dialogDiv.style.left = newLeft + "px";
			}
		}
	},

	/**
	* Will add a open/close link at that the top of page for debugging
	* @param {Object} sDialogObj
	*/
	setDebugHelper: function(sDialogObj) {
		var debugDiv = document.createElement("div");
		debugDiv.style.zIndex = "100";
		debugDiv.style.display = "block";
		debugDiv.style.position = "relative";
		debugDiv.style.backgroundColor = "#FFFFFF";
		debugDiv.innerHTML = "<table><tr><td>" +
			"<a href=\"#\" onclick=\"" + sDialogObj + ".show();\">show</a>" + "     " +
			"<a href=\"#\" onclick=\"" + sDialogObj + ".hide();\">hide</a></td></tr></table>";
		document.body.insertBefore(debugDiv, document.body.firstChild);
	}
});