/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

/**
 * DrillUpDownAction - implements drill-related actions including drillability
 * and returning/performing drilling in the "default drill direction" in cognos viewer
 */
function DrillUpDownAction()
{
	this.m_sAction = "DrillDown";
	this.m_drillOption = "drillDown";
	this.undoTooltip = "";
}

DrillUpDownAction.prototype = new DrillAction();
DrillUpDownAction.prototype.getHoverClassName = function() { return "dl"; };

DrillUpDownAction.prototype.getUndoHint = function()
{
	return this.undoTooltip;
};

DrillUpDownAction.prototype.keepRAPCache = function()
{
	return false;
};

DrillUpDownAction.prototype.updateDrillability = function(cognosViewer, ctxNode)
{
	this.m_oCV = cognosViewer;

	var ctxValueString = ctxNode.getAttribute("ctx");
	var ctxValues = ctxValueString.split("::");

	this.drillability = 0;
	if (ctxValues.length == 1) {
		this.drillability = this.getDrillabilityForCtxValue(ctxValues[0].split(":")[0]);
	} else if (ctxValues.length == 2) {
		this.drillability = this.getDrillabilityForCtxValue(ctxValues[1].split(":")[0]);
	} else if (ctxValues.length > 2) {
		this.drillability = this.getDrillabilityForIntersection(ctxValues[1].split(":")[0],
									 ctxValues[2].split(":")[0]);
	}

	//set the default action to match the default drill
	if (this.isDefaultDrillUp(ctxNode)) {
		this.m_sAction = "DrillUp";
		this.m_drillOption = "drillUp";
		this.undoTooltip = RV_RES.RV_DRILL_UP;
	} else {
		this.m_sAction = "DrillDown";
		this.m_drillOption = "drillDown";
		this.undoTooltip = RV_RES.RV_DRILL_DOWN;
	}
	return this.drillability;
};

DrillUpDownAction.prototype.updateDrillabilityFromSelections = function()
{
	var selectionController = this.m_oCV.getSelectionController();
	var selectedObjects = selectionController.getAllSelectedObjects();
	this.drillability=0;
	if (selectedObjects != null && typeof selectedObjects != "undefined"
		&& selectedObjects.length == 1 && selectedObjects[0].m_contextIds!=null)
	{
		if (selectedObjects[0].getLayoutType() == "section") {
			this.drillability = 0; //no drilling on section header in a sectioned list.
		} else {
			if (selectedObjects[0].m_contextIds.length == 1) {
				this.drillability = this.getDrillabilityForCtxValue(selectedObjects[0].m_contextIds[0][0]);
			} else if (selectedObjects[0].m_contextIds.length == 2) {
				this.drillability = this.getDrillabilityForCtxValue(selectedObjects[0].m_contextIds[1][0]);
			} else if (selectedObjects[0].m_contextIds.length > 2) {
				this.drillability = this.getDrillabilityForIntersection(selectedObjects[0].m_contextIds[1][0],
										selectedObjects[0].m_contextIds[2][0]);
			}
		}
	}
	return this.drillability;
};

DrillUpDownAction.prototype.getDrillabilityForCtxValue = function(ctxValue)
{
	var drillability = 0;
	var selectionController = this.m_oCV.getSelectionController();
	if (selectionController.getMun(ctxValue) != "" && selectionController.getUsageInfo(ctxValue) != '2')
	{
		//Start with the drill flags, then augment with reportInfo....
		drillability = (+selectionController.getDrillFlagForMember(ctxValue));
		var drillabilityObj = this.getDrillabilityForItemFromReportInfo(selectionController.getRefDataItem(ctxValue));
		if (drillabilityObj != null) {
			if (drillabilityObj.disableDown == true) {
				if (drillability == 1 || drillability >= 3) {	//up or both
					drillability = 1;	//up
				} else {
					drillability = 0;	//none
				}
			}

			if (drillabilityObj.disableUp == true) {
				if (drillability >= 2) {	//down or both
					drillability = 2;	//down
				} else {
					drillability = 0;	//none
				}
			}
		}
	}
	return drillability;
};

DrillUpDownAction.prototype.getDrillabilityForIntersection = function(ctxValue1, ctxValue2)
{
	var drillability1 = this.getDrillabilityForCtxValue(ctxValue1);
	var drillability2 = this.getDrillabilityForCtxValue(ctxValue2);

	if (drillability1 == drillability2) {
		return drillability1;
	}

	//swap so that d2 > d1
	if (drillability1 > drillability2) {
		var temp = drillability1;
		drillability1 = drillability2;
		drillability2 = temp;
	}

	if (drillability1 == 1 && drillability2 == 2) {
		return 3;	//down or up
	}

	return drillability2;
};

DrillUpDownAction.prototype.hasPermission = function()
{
	if( this.m_oCV)
	{
		var envParams = this.m_oCV.envParams;
		if( envParams )
		{
			return !( this.m_oCV.isLimitedInteractiveMode() || ( envParams['cv.objectPermissions'].indexOf( 'read' ) === -1 ));
		}
	}
	return false;
};

DrillUpDownAction.prototype.canDrillUp = function()
{
	//0=none, 1=up, 2=down, 3=downorup, 4=upordown
	return ((this.drillability == 1 || this.drillability == 3 || this.drillability == 4) && this.hasPermission() );
};

DrillUpDownAction.prototype.canDrillDown = function()
{
	//0=none, 1=up, 2=down, 3=downorup, 4=upordown
	return ( (this.drillability == 2 || this.drillability == 3 || this.drillability == 4) && this.hasPermission() );
};

DrillUpDownAction.prototype.isDefaultDrillUp = function(ctxNode)
{
	if (this.drillability == 1 || this.drillability == 4 || (ctxNode && ctxNode.getAttribute("ischarttitle") === "true")) {
		return true;
	} else {
		return false;
	}
};

DrillUpDownAction.prototype.onMouseOver = function(evt)
{
	if (this.drillability > 0 && !this.getCognosViewer().isLimitedInteractiveMode()) {
		var ctxNode = getCtxNodeFromEvent(evt);
		this.addDrillableClass(ctxNode);
		if (evt.toElement && evt.toElement.nodeName && evt.toElement.nodeName.toLowerCase() == "img") {
			this.addDrillableClass(evt.toElement);
		}
	}
};

DrillUpDownAction.prototype.onMouseOut = function(evt)
{
	var ctxNode = getCtxNodeFromEvent(evt);
	if (ctxNode) {
		this.removeDrillableClass(ctxNode);
		if (evt.toElement && evt.toElement.nodeName && evt.toElement.nodeName.toLowerCase() == "img") {
			this.removeDrillableClass(evt.toElement);
		}
	}
	
};

DrillUpDownAction.prototype.onDoubleClick = function(evt)
{
	if (this.drillability > 0 && this.hasPermission() ) {
		this.execute();
		var ctxNode = getCtxNodeFromEvent(evt);
		if (ctxNode!=null) {
			this.removeDrillableClass(ctxNode);
		}
	}
};

DrillUpDownAction.prototype.addDrillableClass = function(node) {
	if (! node.className.match(new RegExp('(\\s|^)' + this.getHoverClassName() + '(\\s|$)'))) {
		node.className += " " + this.getHoverClassName();
	}
};

DrillUpDownAction.prototype.removeDrillableClass = function(node) {
	var className = node.className; 
	className = className.replace(new RegExp('(\\s|^)' + this.getHoverClassName() + '(\\s|$)'), ' ');
	node.className = className.replace(/^\s*/, "").replace(/\s*$/, "");
}; 
