/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */

/**
 * CognosViewerAction constructor (base class for all cognos viewer action
 * @constructor
 */
function CognosViewerAction()
{
	this.m_oCV = null;
}

CognosViewerAction.prototype.setRequestParms = function( parms ){};
CognosViewerAction.prototype.onMouseOver = function(evt) { return false; };
CognosViewerAction.prototype.onMouseOut = function(evt) { return false; };
CognosViewerAction.prototype.onMouseDown = function(evt) { return false; };
CognosViewerAction.prototype.onClick = function(evt) { return false; };
CognosViewerAction.prototype.onDoubleClick = function(evt) { return false; };
CognosViewerAction.prototype.updateMenu = function(jsonSpec) { return jsonSpec; };
CognosViewerAction.prototype.addAdditionalOptions = function(request) {};
CognosViewerAction.prototype.genSelectionContextWithUniqueCTXIDs = function() { return false; };

/**
 * Method that gets called before the action context gets built. If
 * there's anything special that the action needs to do, they should
 * override this method.
 */
CognosViewerAction.prototype.preProcess = function() {};

/**
 * Sets the cognos viewer object (called by the action factory
 * @param CCognosViewer object
 * @private
 */
CognosViewerAction.prototype.setCognosViewer = function(oCV)
{
	this.m_oCV = oCV;
};

/**
 * Returns an instance to the cognos viewer object
 * @return CCognosViewer object
 */
CognosViewerAction.prototype.getCognosViewer = function()
{
	return this.m_oCV;
};

CognosViewerAction.prototype.getUndoRedoQueue = function()
{
	return this.getCognosViewer().getViewerWidget().getUndoRedoQueue();
};

CognosViewerAction.prototype.getViewerWidget = function()
{
	return this.m_oCV.getViewerWidget();
};

CognosViewerAction.prototype.getActionDispatcher = function()
{
	return this.m_oCV.getViewerWidget().getActionDispatcher();
};

/**
 * Returns the object display name (custom name/report name/report part name)
 */
CognosViewerAction.prototype.getObjectDisplayName = function()
{
	var displayName = "";

	if(this.m_oCV != null)
	{
		if(typeof this.m_oCV.envParams["reportpart_id"] != "undefined")
		{
			displayName = this.m_oCV.envParams["reportpart_id"];
		}
		else if(typeof this.m_oCV.envParams["ui.name"] != "undefined")
		{
			displayName = this.m_oCV.envParams["ui.name"];
		}
	}

	return displayName;
};

/**
 * Gets the container Id
 */
CognosViewerAction.prototype.getContainerId = function(selectionController) {
	var container = "";
	if (selectionController && selectionController.getAllSelectedObjects) {
		var allSel = selectionController.getAllSelectedObjects();
		if (allSel) {
			var selection = allSel[0];
			if (selection && selection.getLayoutElementId) {
				container = this.removeNamespace(selection.getLayoutElementId());
			}
		}
	}

	return container;
};

CognosViewerAction.prototype.removeNamespace = function(value)
{
	var originalValue = value;

	try
	{
		if(value != "")
		{
			var idIndex = value.indexOf(this.m_oCV.getId());
			if(idIndex != -1)
			{
				value = value.replace(this.m_oCV.getId(), "");
			}
		}

		return value;
	}
	catch(e)
	{
		return originalValue;
	}
};

CognosViewerAction.prototype.doAddActionContext = function()
{
	return true;
};

CognosViewerAction.prototype.getSelectionContext = function()
{
	return getViewerSelectionContext(this.m_oCV.getSelectionController(), new CSelectionContext(this.m_oCV.envParams["ui.object"]), this.genSelectionContextWithUniqueCTXIDs());
};

CognosViewerAction.prototype.getNumberOfSelections = function()
{
	var numberOfSelections = -1;
	if(this.m_oCV != null && this.m_oCV.getSelectionController() != null)
	{
		numberOfSelections = this.m_oCV.getSelectionController().getSelections().length;
	}

	return numberOfSelections;
};

CognosViewerAction.prototype.buildDynamicMenuItem = function(jsonSpec, actionClass)
{
	jsonSpec.action =  {name: "LoadMenu", payload: {action:actionClass}};
	jsonSpec.items = [{ "name": "loading",
	 "label" : RV_RES.GOTO_LOADING,
	 iconClass: "loading"}];
	return jsonSpec;
};

CognosViewerAction.prototype.createCognosViewerRequest = function( requestType )
{
	var cognosViewerRequest = new CCognosViewerRequest(requestType);

	this.preProcess();

	if( this.doAddActionContext() === true )
	{
		var actionContext = this.addActionContext();
		cognosViewerRequest.addOption("cv.actionContext", actionContext);
		if (typeof g_firefoxPluginHelper != "undefined" && g_firefoxPluginHelper != null)
		{
			g_firefoxPluginHelper.debug('Action context', actionContext, "xml");
		}
	}

	cognosViewerRequest.addOption("ui.object", this.m_oCV.envParams["ui.object"]);

	if(typeof this.m_oCV.envParams["ui.spec"] != "undefined")
	{
		cognosViewerRequest.addOption("ui.spec", this.m_oCV.envParams["ui.spec"]);
	}

	if(typeof this.m_oCV.envParams["modelPath"] != "undefined")
	{
		cognosViewerRequest.addOption("modelPath", this.m_oCV.envParams["modelPath"]);
	}

	if(typeof this.m_oCV.envParams["packageBase"] != "undefined")
	{
		cognosViewerRequest.addOption("packageBase", this.m_oCV.envParams["packageBase"]);
	}

	if (typeof this.m_oCV.envParams["rap.state"] != "undefined")
	{
		cognosViewerRequest.addOption("rap.state", this.m_oCV.envParams["rap.state"]);
	}

	if (typeof this.m_oCV.envParams["json.infoBar"] != "undefined")
	{
		cognosViewerRequest.addOption("rap.reportInfo", this.m_oCV.envParams["json.infoBar"]);
	}

	this.addAdditionalOptions(cognosViewerRequest);

	return cognosViewerRequest;
};

CognosViewerAction.prototype.fireModifiedReportEvent = function()
{
	try
	{
		var viewerWidget = this.getCognosViewer().getViewerWidget();
		var payload = {'modified':true};
		viewerWidget.fireEvent("com.ibm.bux.widget.modified", null, payload);
	} catch(e) {}
};

CognosViewerAction.prototype.showCustomCursor = function(evt, id, imageRef)
{
	var customCursor = document.getElementById(id);
	if(customCursor == null)
	{
		customCursor = document.createElement("span");
		customCursor.className = "customCursor";
		customCursor.setAttribute("id", id);
		document.body.appendChild(customCursor);
	}

	var imageSrcHtml = "<img src=\"" + this.getCognosViewer().getWebContentRoot() + imageRef + "\"/>";

	customCursor.innerHTML = imageSrcHtml;
	customCursor.style.position = "absolute";
	customCursor.style.left = (evt.clientX + 15) + "px";
	customCursor.style.top = (evt.clientY + 15) + "px";
	customCursor.style.display = "inline";
};

CognosViewerAction.prototype.hideCustomCursor = function(id)
{
	var customCursor = document.getElementById(id);
	if(customCursor != null)
	{
		customCursor.style.display = "none";
	}
};

CognosViewerAction.prototype.selectionHasContext = function()
{
	var selections = this.getCognosViewer().getSelectionController().getAllSelectedObjects();
	var bContext = false;
	if(selections != null && selections.length > 0)
	{
		for (var i=0; i < selections.length; i++)
		{
			if (selections[i].hasContextInformation())
			{
				bContext = true;
				break;
			}
		}
	}

	return bContext;
};

CognosViewerAction.prototype.getSelectedReportInfo = function() {
	var viewer = this.getCognosViewer();

	var selectionController = viewer.getSelectionController();
	var containerId = null;
	if( selectionController != null && typeof selectionController != "undefined")
	{
		containerId = this.getContainerId( selectionController );
	}

	var selectedObject = this.getReportInfo(containerId);
	if( selectedObject == null )
	{
		//if there is more than one object, we'll return null
		var reportInfo =  viewer.envParams["rap.reportInfo"];
		if(reportInfo.length == 1)
		{
			selectedObject = reportInfo[0];
		}
	}

	return selectedObject;
};

CognosViewerAction.prototype.getReportInfo = function(containerId) {
	var viewer = this.getCognosViewer();
	var reportInfo =  viewer.envParams["rap.reportInfo"];
	var selectedObject = null;
	if( containerId != null && containerId.length > 0 )
	{
		for ( var i in reportInfo)
		{
			if( reportInfo[i].container === containerId )
			{
				selectedObject = reportInfo[i];
				break;
			}
		}
	}
	return selectedObject;
};

CognosViewerAction.prototype.isSelectionOnChart = function() {

	var viewer = this.getCognosViewer();

	if (viewer.getSelectionController().getSelectedChartArea() != null)
	{
		return true;
	}

	var containerId = this.getContainerId( viewer.getSelectionController());

	if (typeof containerId != "undefined")
	{
		var reportInfo = this.getReportInfo(containerId);
		if (reportInfo != null && reportInfo.displayTypeId)
		{
			var displayTypeId = reportInfo.displayTypeId.toLowerCase();
			return this.isChart(displayTypeId);
		}
	}
	return false;
};

CognosViewerAction.prototype.ifContainsInteractiveDataContainer = function()
{
	var reportInfo =  this.getCognosViewer().envParams["rap.reportInfo"];
	if (typeof reportInfo == "undefined")
	{
		return false;
	}
	for ( var i in reportInfo)
	{
		var displayTypeId = reportInfo[i].displayTypeId;
		if (typeof displayTypeId != "undefined" && this.isInteractiveDataContainer(displayTypeId))
		{
			return true;
		}
	}
	return false;
};

CognosViewerAction.prototype.isChart = function(displayTypeId)
{
	var id = displayTypeId.toLowerCase();
	return id != 'mapchart' && id.match('chart$') == 'chart';
};

CognosViewerAction.prototype.isInteractiveDataContainer = function(displayTypeId)
{
	var result = false;
	if (typeof displayTypeId != "undefined" && displayTypeId != null) {
		var id = displayTypeId.toLowerCase();
		result = id == 'crosstab' || id == 'list' || this.isChart(id);
	}
	return result;
};

/**
*	Detect from report Info if the widget is a prompt control or a prompt page and  only one container (the global one)
*
*/
CognosViewerAction.prototype.isPromptWidget = function()
{
	var reportInfo =  this.getCognosViewer().envParams["rap.reportInfoObject"];
	var result = (
		reportInfo &&
		reportInfo.reportLevelProperties &&
		reportInfo.reportLevelProperties.promptWidget &&
		reportInfo.reportLevelProperties.promptWidget == true) ? true : false;
	return result;
};

CognosViewerAction.prototype.getLayoutComponents = function()
{
	var layoutComponents = [];
	var reportTable = document.getElementById("rt" + this.m_oCV.getId());
	if(reportTable != null)
	{
		layoutComponents = getElementsByAttribute(reportTable, "*", "lid");
	}

	return layoutComponents;
};


//For applicable actions, add a subset of the context/metadata table as action arguments....
CognosViewerAction.prototype.addClientContextData = function(maxValuesPerRDI)
{
	var selectionController = this.m_oCV.getSelectionController();
	if (typeof selectionController!="undefined" && selectionController!=null &&
		typeof selectionController.getCCDManager!="undefined" && selectionController.getCCDManager()!=null) {
		var oCCDManager = selectionController.getCCDManager();
		return ("<md>" + xml_encode(oCCDManager.MetadataToJSON()) +	"</md>" +
				"<cd>" + xml_encode(oCCDManager.ContextDataSubsetToJSON(maxValuesPerRDI)) +	"</cd>");
	}
	return "";
};

CognosViewerAction.prototype.getRAPLayoutTag = function(cellRef)
{
	var tagValue = null;
	if (typeof cellRef == "object" && cellRef != null ) {
		tagValue = cellRef.getAttribute("rap_layout_tag");
	}
	return tagValue;
};

/**
 * Helper method to add the correct properties to a menuItem so it shows up 'checked'
 * @param {Object} bChecked - boolean if the menuItem should be checked
 * @param {Object} oMenuItem - the menuItem object
 * @param {Object} sUncheckedIconClass - optional css class to use if the menuItem is unchecked
*/
CognosViewerAction.prototype.addMenuItemChecked = function(bChecked, oMenuItem, sUncheckedIconClass) {
	if (bChecked) {
		if (this.getCognosViewer().isHighContrast()) {
			oMenuItem["class"] = "menuItemSelected";
		}

		oMenuItem.iconClass = "menuItemChecked";
	}
	else if (sUncheckedIconClass && sUncheckedIconClass.length > 0) {
		oMenuItem.iconClass = sUncheckedIconClass;
	}
};

CognosViewerAction.prototype.gatherFilterInfoBeforeAction = function(action) {
	var widget = this.getCognosViewer().getViewerWidget();
	widget.filterRequiredAction = action;
	widget.clearRAPCache();
	widget.fireEvent("com.ibm.bux.widget.action", null, { action: 'canvas.filters' } );
};

/**
 * LineageAction - implements lineage in cognos viewer
 */
function LineageAction(){}
LineageAction.prototype = new CognosViewerAction();

LineageAction.prototype.getCommonOptions = function(lineageOptions)
{
	lineageOptions["b_action"] = "cognosViewer";
	lineageOptions["cv.responseFormat"] = "asynchDetailMIMEAttachment";
	lineageOptions["ui.showSimpleWait"] = "true";
	lineageOptions["bux"] = this.m_oCV.getViewerWidget() ? "true" : "false";
	lineageOptions["cv.id"] = this.m_oCV.envParams["cv.id"];
};

LineageAction.prototype.getSelectionOptions = function(lineageOptions)
{
	var selectionController = this.m_oCV.getSelectionController();
	var contextIds = getSelectionContextIds(selectionController);

	lineageOptions["context.format"] = "initializer";
	lineageOptions["context.type"] = "reportService";
	lineageOptions["context.selection"] = "metadata," + contextIds.toString();
};

LineageAction.prototype.getPrimaryRequestOptions = function(lineageOptions)
{
	lineageOptions["specificationType"] = "metadataServiceLineageSpecification";
	lineageOptions["ui.action"] = "runLineageSpecification";
	lineageOptions["ui.object"] = this.m_oCV.envParams["ui.object"];
};

LineageAction.prototype.getSecondaryRequestOptions = function(lineageOptions)
{
	lineageOptions["ui.conversation"] = this.m_oCV.getConversation();
	lineageOptions["m_tracking"] = this.m_oCV.getTracking();
	lineageOptions["ui.action"] = "lineage";
};

LineageAction.prototype.constructUrl = function(lineageOptions)
{
	var url = "";
	for(var lineageOption in lineageOptions)
	{
		url += lineageOption + "=" + encodeURIComponent(lineageOptions[lineageOption]) + "&";
	}

	return url;
};

LineageAction.prototype.updateMenu = function(jsonSpec)
{
	if (!this.getCognosViewer().bCanUseLineage) {
		return "";
	}

	jsonSpec.disabled = !this.selectionHasContext();

	return jsonSpec;
};

/**
 * Execute the lineage request
 */
LineageAction.prototype.execute = function()
{
	var cognosViewer = this.getCognosViewer();

	var lineageOptions = {};

	this.getCommonOptions(lineageOptions);
	this.getSelectionOptions(lineageOptions);

	if(this.m_oCV.getConversation() == "")
	{
		this.getPrimaryRequestOptions(lineageOptions);
	}
	else
	{
		this.getSecondaryRequestOptions(lineageOptions);
	}

	var httpRequest = new XmlHttpObject();
	var successCallback = GUtil.generateCallback(this.handleLineageResponse, [httpRequest], this);
	httpRequest.setCallbacks(successCallback, successCallback);

	var url = this.constructUrl(lineageOptions);

	//send asynchronous request
	httpRequest.sendHtmlRequest("POST", cognosViewer.getGateway(), url, true);
};

LineageAction.prototype.handleLineageResponse = function(oResponse)
{
	try
	{
		oResponse = oResponse.xmlHttp;
		var sContentType = oResponse.getResponseHeader("Content-type");
		if(sContentType.indexOf("text/html") != -1)
		{
			this.m_oCV.m_waitPage.hide();
			clearLastAction();

			if (!this.m_oCV.m_waitPage.cancelSubmitted())
			{
				var faultWindow = window.open("","",'height=400,width=500');
				faultWindow.document.write(oResponse.responseText);
			}
		}
		// application/xml should only be used for data only response (i.e. no HTML)
		else if (sContentType.indexOf("application/xml") != -1)
		{
			var responseXML = oResponse.responseXML;

			var status = XMLHelper_GetText(XMLHelper_FindChildByTagName(responseXML, "status", true));
			var waitPageHTML = XMLHelper_FindChildByTagName(responseXML, "waitPage", true);

			if (waitPageHTML == null && this.m_oCV.m_waitPage.cancelSubmitted())
			{
				this.m_oCV.m_waitPage.hide();
				clearLastAction();
				return;
			}

			this.m_oCV.setStatus(status);

			var conversation = XMLHelper_GetText(XMLHelper_FindChildByTagName(responseXML, "conversation", true));
			if(conversation.length > 0)
			{
				this.m_oCV.setConversation(conversation);
			}

			var tracking = XMLHelper_GetText(XMLHelper_FindChildByTagName(responseXML, "tracking", true));
			if(tracking.length > 0)
			{
				this.m_oCV.setTracking(tracking);
			}

			if(this.m_oCV.isWorking(""+status))
			{
				var waitTable = document.getElementById("CVWaitTable" + this.m_oCV.getId());
				if (waitPageHTML != null && (typeof waitTable == "undefined" || waitTable == null || waitTable.parentNode.id == "CVWait" + this.m_oCV.getId()))
				{
					this.m_oCV.m_waitPage.initializeWaitPage(XMLHelper_GetText(waitPageHTML));
					this.m_oCV.m_waitPage.setCognosViewer(this.m_oCV);
					this.m_oCV.m_waitPage.show(false);
				}

				var isWidget = this.m_oCV.getViewerWidget() ? "true" : "false";
				var url = "b_action=cognosViewer&ui.action=wait&cv.responseFormat=asynchDetailMIMEAttachment&ui.primaryAction=lineage&ui.showSimpleWait=true&";
				url += "bux=" + isWidget + "&";
				url += "m_tracking=" + encodeURIComponent(tracking) + "&";
				url += "ui.conversation=" + encodeURIComponent(conversation) + "&";
				url += "cv.actionState=" + encodeURIComponent(XMLHelper_GetText(XMLHelper_FindChildByTagName(responseXML, "actionState", true))) + "&";

				var httpRequest = new XmlHttpObject();
				var successCallback = GUtil.generateCallback(this.handleLineageResponse, [httpRequest], this);
				httpRequest.setCallbacks(successCallback, successCallback);

				//send asynchronous request
				httpRequest.sendHtmlRequest("POST", this.m_oCV.getGateway(), url, true);
			}
			else if (status == "complete")
			{
				this.m_oCV.m_waitPage.hide();
				this.invokeLineageService(responseXML);
			}
		}
	}
	catch (e) {}
};

LineageAction.prototype.invokeLineageService = function(responseXML)
{
	var lineageDetails = XMLHelper_FindChildByTagName(responseXML, "reportLineageResponse", true);
	if(lineageDetails != null)
	{
		var lineageDetailString = XMLBuilderSerializeNode(lineageDetails);

		var config = null;
		if(typeof MDSRV_CognosConfiguration != "undefined")
		{
			config = new MDSRV_CognosConfiguration();

			var lineageURI = "";
			if(this.m_oCV.envParams["metadataInformationURI"])
			{
				lineageURI = this.m_oCV.envParams["metadataInformationURI"];
			}

			config.addProperty("lineageURI", lineageURI);
			config.addProperty("gatewayURI", this.m_oCV.getGateway());
		}

		var searchPath = this.m_oCV.envParams["ui.object"];
		var sSelectionContext = getViewerSelectionContext(this.m_oCV.getSelectionController(), new CSelectionContext(searchPath));
		var lineageHelper = new MDSRV_LineageFragmentContext(config, sSelectionContext);
		lineageHelper.setExecutionParameters(this.m_oCV.getExecutionParameters());
		if (typeof searchPath == "string")
		{
			lineageHelper.setReportPath( searchPath );
		}
		lineageHelper.setReportLineage(lineageDetailString);

		lineageHelper.open();
	}
};
