/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM
 *| IBM Cognos Products: Viewer
 *| (C) Copyright IBM Corp. 2001, 2010
 *|
 *| US Government Users Restricted Rights - Use, duplication or
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *|
 *+------------------------------------------------------------------------+
 */
/**
 * CognosViewerAnnotation constructor (base class for all annotation rules)
 * @constructor
 */
function CognosViewerAnnotation() {
	this.m_oCV = null;
}

/**
 * Sets the cognos viewer object (called by the action factory)
 * @param CCognosViewer object
 * @private
 */
CognosViewerAnnotation.prototype.setCognosViewer = function(oCV) {
	this.m_oCV = oCV;
};

/**
 * Returns an instance to the cognos viewer object
 * @return CCognosViewer object
 */
CognosViewerAnnotation.prototype.getCognosViewer = function() {
	return this.m_oCV;
};

/**
 * Override this function to determine whether or not the specific annotation
 * object is enabled.
 * @return boolean
 */
CognosViewerAnnotation.prototype.isEnabled = function() {};

/**
 * Override this function to get the base string name for the menu item
 * @return String
 */
CognosViewerAnnotation.prototype.getMenuItemBaseString = function(){
};

/**
 * Override this function to get the icon class name for the menu item
 * @return String
 */
CognosViewerAnnotation.prototype.getMenuItemIconClass = function(){
};


/**
 * Gets the current selections
 */
CognosViewerAnnotation.prototype.getSelections = function(){
	var selections = [];

	var viewer = this.getCognosViewer();
	if (viewer) {
		var selCon = viewer.getSelectionController();
		if (selCon) {
			selections = selCon.getSelections();
		}
	}

	return selections;
};

/**
 * Check if the widget has any annotation
 * @return boolean
 */
CognosViewerAnnotation.prototype.hasAnnotation = function(){
	var has = false;

	var viewer = this.getCognosViewer();
	if (viewer) {
		var widget = viewer.getViewerWidget();
		if (widget) {
			var store = widget.getAnnotationStore();
			if (store) {
				has = store.hasAnnotation();
			}
		}
	}

	return has;
};
function CognosViewerCellAnnotation(){}
CognosViewerCellAnnotation.prototype = new CognosViewerAnnotation();


/**
 * Concatenates the defult menu string with the selected object representation
 * @param defaultText - the default value when nothing is selected
 * @return String
 */
CognosViewerCellAnnotation.prototype.getMenuItemString = function(defaultText){
	var selText = defaultText;
	var selections = this.getSelections();
	if (selections && selections.length == 1) {
		var sel = selections[0];
		selText = sel.getDisplayValues()[0];
	}

	var text = this.getMenuItemBaseString();
	if (selText && selText.length > 0) {
		text += " - " + selText;
	}

	return text;
};



/**
 * Return true if selected cell has any annotations
 * @return boolean
 */
CognosViewerCellAnnotation.prototype.hasAnnotaionsOnSelectedCell = function(){

	var selections = this.getSelections();
	if (selections && selections.length == 1) {
		var cellRef = selections[0].getCellRef();
		var ctxId = this.getCognosViewer().findCtx(cellRef);
		var viewerWidget = this.getCognosViewer().getViewerWidget();
		if (viewerWidget) {
			var annotationStore = viewerWidget.getAnnotationStore();
			if (annotationStore) {
				var annotations = annotationStore.get(ctxId);
				if (annotations && annotations.length > 0) {
					return true;
				}
			}
		}
	}

	return false;
};

/**
 * Return true if selected cell has newly added annotation
 * @return boolean
 */
CognosViewerCellAnnotation.prototype.hasNewAnnotationOnSelectedCell = function(){
	var addedInCurrentSession = false;

	var selections = this.getSelections();
	if (selections && selections.length == 1) {
		var cellRef = selections[0].getCellRef();
		var ctxId = this.getCognosViewer().findCtx(cellRef);
		var viewerWidget = this.getCognosViewer().getViewerWidget();
		if (viewerWidget) {
			var annotationStore = viewerWidget.getAnnotationStore();
			if (annotationStore) {
				if (annotationStore.hasNewAnnotation(ctxId)) {
					addedInCurrentSession = true;
				}
			}
		}
	}
	return addedInCurrentSession;
};

/*
 * New Annotation - Adds a new annotation where none previously exists
 */
function NewAnnotation() {}
NewAnnotation.prototype = new CognosViewerCellAnnotation();

/**
 * Return true if a cell is selected and the cell does not have new annotation already
 * @return boolean
 */
NewAnnotation.prototype.isEnabled = function() {
	var selections = this.getSelections();
	return (selections && selections.length == 1) && !this.hasNewAnnotationOnSelectedCell();
};


/**
 * Return 'Edit comment' if selected cell has any annotaitons else return 'Add comment'
 * @return String
 */
NewAnnotation.prototype.getMenuItemBaseString = function(){
	return this.hasAnnotaionsOnSelectedCell() ?
		   RV_RES.IDS_JS_EDIT_ANNOTATION:
		   RV_RES.IDS_JS_NEW_ANNOTATION;
};

/**
 * Return 'EditAnnotation' if selected cell has any annotaitons else return 'NewAnnotation'
 * @return String
 */
NewAnnotation.prototype.getMenuItemIconClass = function(){
	return this.hasAnnotaionsOnSelectedCell() ? 'EditAnnotation': 'NewAnnotation';
};

/*
 * Edit Annotation - Change the value of an annotation which previously existed
 */
function EditAnnotation() {}
EditAnnotation.prototype = new CognosViewerCellAnnotation();

/**
 * Return true if selected cell has new annotation already
 * @return boolean
 */
EditAnnotation.prototype.isEnabled = function() {
	return this.hasNewAnnotationOnSelectedCell();
};

/**
 * Return 'Edit comment*'
 * @return String
 */
EditAnnotation.prototype.getMenuItemBaseString = function(){
	return RV_RES.IDS_JS_EDIT_ANNOTATION;
};

/**
 * Return 'EditAnnotation'
 * @return String
 */
EditAnnotation.prototype.getMenuItemIconClass = function(){
	return 'EditAnnotation';
};



/*
 * Delete Annotation - Remove a previously existing annotation
 */
function DeleteAnnotation() {}
DeleteAnnotation.prototype = new CognosViewerCellAnnotation();

DeleteAnnotation.prototype.isEnabled = function() {
	return this.hasNewAnnotationOnSelectedCell();
};

DeleteAnnotation.prototype.getMenuItemString = function(defaultText){
	return this.getMenuItemBaseString();
};

DeleteAnnotation.prototype.getMenuItemBaseString = function(){
	return RV_RES.IDS_JS_DELETE_ANNOTATION;
};

DeleteAnnotation.prototype.getMenuItemIconClass = function(){
	return 'DeleteAnnotation';
};

function CognosViewerWidgetAnnotation(){}
CognosViewerWidgetAnnotation.prototype = new CognosViewerAnnotation();

CognosViewerWidgetAnnotation.prototype.getMenuItemString = function(){
	var widgetText = this.getCognosViewer().getViewerWidget().getDisplayName();

	var text = this.getMenuItemBaseString();

	if (widgetText && widgetText.length > 0) {
		text += " - " + widgetText;
	}

	return text;
};

CognosViewerWidgetAnnotation.prototype.hasWidgetLevelAnnotations = function(){
	var hasAnnotations = false;
	var viewerWidget = this.getCognosViewer().getViewerWidget();

	if (viewerWidget) {
		var annotationStore = viewerWidget.getAnnotationStore();
		var widgetAnnotations = annotationStore.get(annotationStore.WIDGET_CONTEXT);
		if (widgetAnnotations && widgetAnnotations.length > 0) {
			hasAnnotations = true;
		}
	}

	return hasAnnotations;
};

CognosViewerWidgetAnnotation.prototype.hasWidgetLevelNewAnnotation = function(){
	var hasNewAnnotation = false;
	var viewerWidget = this.getCognosViewer().getViewerWidget();
	if (viewerWidget) {
		var annotationStore = viewerWidget.getAnnotationStore();
		if (annotationStore) {
			if (annotationStore.hasNewAnnotation(annotationStore.WIDGET_CONTEXT)) {
				hasNewAnnotation = true;
			}
		}
	}
	return hasNewAnnotation;
};

/*
 * New Widget Annotation - Adds a new annotation to the widget where none previously existed
 */
function NewWidgetAnnotation() {}
NewWidgetAnnotation.prototype = new CognosViewerWidgetAnnotation();

/**
 * Return true if there is no new annotation on widget level and cell is not selected
 * @return boolean
 */
NewWidgetAnnotation.prototype.isEnabled = function() {
	var selections = this.getSelections();
	return (!selections || selections.length != 1) && !this.hasWidgetLevelNewAnnotation();
};


/**
 * Return 'Edit comment' if any annotaitons on widget level already exist else return 'Add comment'
 * @return String
 */
NewWidgetAnnotation.prototype.getMenuItemBaseString = function(){
	return this.hasWidgetLevelAnnotations() ?
		   RV_RES.IDS_JS_EDIT_WIDGET_ANNOTATION:
		   RV_RES.IDS_JS_NEW_WIDGET_ANNOTATION;
};

/**
 * Return 'EditWidgetAnnotation' if any annotaitons on widget level already exist else return 'NewWidgetAnnotation'
 * @return String
 */
NewWidgetAnnotation.prototype.getMenuItemIconClass = function(){
	return this.hasWidgetLevelAnnotations()? 'EditWidgetAnnotation': 'NewWidgetAnnotation';
};

/*
 * Edit Widget Annotation - Change the value of an annotation which previously existed on the widget
 */
function EditWidgetAnnotation() {}
EditWidgetAnnotation.prototype = new CognosViewerWidgetAnnotation();

/**
 * Return true if there is new annotation on widget level and cell is not selected
 * @return boolean
 */
EditWidgetAnnotation.prototype.isEnabled = function() {
	var selections = this.getSelections();
	return (!selections || selections.length != 1) && this.hasWidgetLevelNewAnnotation();
};

/**
 * Return 'Edit comment*'
 * @return String
 */
EditWidgetAnnotation.prototype.getMenuItemBaseString = function(){
	return RV_RES.IDS_JS_EDIT_WIDGET_ANNOTATION;
};

EditWidgetAnnotation.prototype.getMenuItemIconClass = function(){
	return 'EditWidgetAnnotation';
};


/*
 * Delete Widget Annotation - Remove previously existing annotations from the widget
 */
function DeleteWidgetAnnotation() {}
DeleteWidgetAnnotation.prototype = new CognosViewerWidgetAnnotation();

DeleteWidgetAnnotation.prototype.isEnabled = function() {
	var selections = this.getSelections();
	return (!selections || selections.length != 1) && this.hasWidgetLevelNewAnnotation();
};

DeleteWidgetAnnotation.prototype.getMenuItemBaseString = function(){
	return RV_RES.IDS_JS_DELETE_WIDGET_ANNOTATION;
};

DeleteWidgetAnnotation.prototype.getMenuItemIconClass = function(){
	return 'DeleteWidgetAnnotation';
};

/**
 * AnnotationCTXLookup
 * @constructor
 */
function AnnotationCTXLookup(ccdManager) {
	this.m_ccdManager = ccdManager;
	this.m_md = this.m_ccdManager.m_md;
	this.m_cd = this.m_ccdManager.m_cd;

	this.m_RDIVALUE_KEY = 'annotationHook';
	this.m_oContextDetailsMap = {};
}

/**
 * Populate a map which relates each individual context ID referenced in the annotation array to their backing details (like rdi, modelItem, usage)
 * @param annotationArray array
 * @public
 */
AnnotationCTXLookup.prototype.populateCtxDetailsForAllAnnotations = function(annotationArray) {

	for (var i = 0, len = annotationArray.length; i < len; i++) {
		var anno = annotationArray[i];
		var ctx = anno.ctx;

		if (ctx) {
			this._populateCtxDetailsForOneCtxValue(ctx);
		}
	}
};

/**
 * Extracts Ids from ctxValue string, go through each id, and populate ctx detail
 * @param ctxValue string Examples: 1, *, 1:2:3, 4::5::6, *::2:3::5:8, and so on
 * @private
 */
AnnotationCTXLookup.prototype._populateCtxDetailsForOneCtxValue = function(ctxValue) {
	if(ctxValue) {
		var ctxIDs = this.generateIDArrayFromCellCtxValue(ctxValue);
		for (var i = 0, len = ctxIDs.length; i < len; i++) {
			var id = ctxIDs[i];
			this._populateCtxDetailsForOneId(id);
		}
	}
};

/**
 * Generates string array of ctx Ids which appears in ctxValue string.
 * @param ctxValue string Examples: 1, *, 1:2:3, 4::5::6, *::2:3::5:8, and so on
 * @private
 */
AnnotationCTXLookup.prototype.generateIDArrayFromCellCtxValue = function(ctxValue) {
	if (ctxValue) {
		return ctxValue.replace(/::/g, ':').split(':');
	}
	return null;
};

/**
 * Populates ctx detail of id passed if it is numeric value
 * @param id string - numeric value or '*'
 * @private
 */
AnnotationCTXLookup.prototype._populateCtxDetailsForOneId = function(id) {
	if (id.match(/\*/)) {
		return;
	}

	//check if exists
	var detail = this.m_oContextDetailsMap[id];
	if(detail) {
		return; //already populated, just return.
	} else {
		detail = {};
		var contextData = this.m_cd[id];

		if(contextData) {
			if (contextData.u) { detail['u'] = contextData.u; }
			if (contextData.r) {
				detail['r'] = contextData.r;

				var metadata = this.m_md[contextData.r];
				if(metadata && metadata.usage!== 'undefined') {
					detail['usage'] = metadata.usage;
				}
			}
			this._populateRdiValueInDetail(detail, id);

			this.m_oContextDetailsMap[id] = detail;
		}
	}
};

/**
 * Populates RdiValue value which annotation service would select for ctx id to detail object
 * @param id string - numeric value or '*'
 * @private
 */
AnnotationCTXLookup.prototype._populateRdiValueInDetail = function(detail, id) {
	var item = this.m_cd[id];

	if (typeof item.l != 'undefined' && typeof this.m_md[item.l].l != 'undefined') {
		detail[this.m_RDIVALUE_KEY] = this.m_md[item.l].l;
	} else if (typeof item.h != 'undefined' && typeof this.m_md[item.h].h != 'undefined') {
		detail[this.m_RDIVALUE_KEY] = this.m_md[item.h].h;
	} else if (typeof item.i != 'undefined' && typeof this.m_md[item.i].i != 'undefined') {
		detail[this.m_RDIVALUE_KEY] = this.m_md[item.i].i;
	} else if (typeof item.r != 'undefined' && typeof this.m_md[item.r].r != 'undefined') {
		detail[this.m_RDIVALUE_KEY] = this.m_md[item.r].r;
	}
};

/**
 * Returns ctx detail by id
 * @param id string
 * @private
 */
AnnotationCTXLookup.prototype.getDetail = function(id) {
	if (!id) {
		return null;
	}
	return this.m_oContextDetailsMap[id];
};

/**
 * Returns true if id and annoFirstId is same numeric value.
 * Returns true if rdi-value of id's ctx detail is same as annoRDI when annoFirstId is '*'.
 * @param annoFirstId string - numeric value or '*'
 * @param annoRDI string
 * @param id string
 * @private
 */
AnnotationCTXLookup.prototype.isAnnotatedId = function(annoFirstId, annoRDI, id) {
	if (!annoFirstId || !id) {
		return false;
	}

	if (annoFirstId.match(/\*/)) {
		var detail = this.getDetail(id);
		if (detail && detail[this.m_RDIVALUE_KEY]) {
			return annoRDI === detail[this.m_RDIVALUE_KEY];
		} else {
			return false;
		}
	} else {
		return annoFirstId === id;
	}
};


/**
 * Checks if cellCtxValue matches with annoatation's ctx and rdi.
 * The following is detail logic of the decision
 * - extracts Ids from cellCtxValue and annotation.ctx
 * - calls isAnnotatedId with first id of ctx and first id of cellCtxValue
 * - if not, cellCtxValue is not for this annotation. returns false
 * - next, compares data (i.e: usage is 0 or 1) item Ids
 * - if annotation ctx has more data item than cellCtxValue's,
 *   it means the cellCtxValue does not have enough data items.
 *   returns false.
 * - Finally, checks all of annotation ctx's data item Ids is present as cellCtxValue's data item.
 *   then returns true.
 * - or, returns false.
 *
 *
 * @param annotation object
 * @param cellCtxValue string
 * @private
 */
AnnotationCTXLookup.prototype.isAnnotatedCell = function(annotation, cellCtxValue) {
	if (!annotation || !annotation.ctx || !cellCtxValue) {
		return false;
	}

	this._populateCtxDetailsForOneCtxValue(cellCtxValue);

	var annoIDArray = this.generateIDArrayFromCellCtxValue(annotation.ctx);
	var cellIDArray = this.generateIDArrayFromCellCtxValue(cellCtxValue);

	//Compare first IDs
	if( !this.isAnnotatedId(annoIDArray[0], annotation.rdi, cellIDArray[0]) ) {
		return false;
	}

	//Compare data (i.e: usage is 0 or 2. means not a measure/calculation) item IDs
	var annoDataItemIdArray = this._generateDataItemIDArrayFromAnnotationIDs(annoIDArray);
	var cellDataItemIdArray = this._generateDataItemIDArrayFromCellIDs(cellIDArray);

	if ( annoDataItemIdArray.length > cellDataItemIdArray.length) {
		return false;
	}

	for (var i=0, len=annoDataItemIdArray.length; i<len; i++) {
		for(var j=0, jLen=cellDataItemIdArray.length; j<jLen; j++) {
			if (annoDataItemIdArray[i] === cellDataItemIdArray[j] ) {
				annoDataItemIdArray[i] = true;
				cellDataItemIdArray[j] = true;
				break;
			}
		}
	}

	for (i = 0, len = annoDataItemIdArray.length; i < len; i++) {
		if (annoDataItemIdArray[i] !== true) {
			return false;
		}
	}
	for (i = 0, len = cellDataItemIdArray.length; i < len; i++) {
		if (cellDataItemIdArray[i] !== true) {
			return false;
		}
	}
	return true;
};


/**
 * Generates array of Ids where id is a data item (not a measure/calculation).
 * To be a data item, 'usage' in ctx detail of id must be 0 or 1
 *
 * @param cellIDArray Array
 * @private
 */
AnnotationCTXLookup.prototype._generateDataItemIDArrayFromCellIDs = function(cellIDArray) {
	var dataItemIDs = [];

	for(var i=1, len=cellIDArray.length; i<len; i++){
		var id = cellIDArray[i];
		var detail = this.getDetail(id);

		if( this._isDataItem(detail) ) {
			dataItemIDs.splice(0,0, id);
		}
	}

	return dataItemIDs;
};


AnnotationCTXLookup.prototype._isDataItem = function(detail) {

	if (detail) {
		if (detail['usage'] === 0 || detail['usage'] === 1) {
			return true;
		}
	}

	return false;
};


/**
 * Generates array of Ids where id is not '*'
 *
 * @param annoIDArray Array
 * @private
 */
AnnotationCTXLookup.prototype._generateDataItemIDArrayFromAnnotationIDs = function(annoIDArray) {
	var dataItemIDs = [];

	for(var i=1, len=annoIDArray.length; i<len; i++){
		var id = annoIDArray[i];
		if (id.match(/\*/) === null) {
			dataItemIDs.splice(0,0, id);
		}
	}

	return dataItemIDs;
};